Package: DSIR
Title: Data Science Infrastructure for Global Health
Version: 0.8.0
Authors@R: person(given = "Shanlong",
                  family = "Ding",
                  role = c("aut", "cre"),
                  email = "dings@who.int",
                  comment = c(ORCID = "0000-0001-9048-6670"))
Description: Supports global health data analysis, including a
    publication-ready 'ggplot2' theme, a 'flextable' defaults helper,
    a thin pie chart wrapper, built-in regional country-code datasets
    with a WHO region lookup helper, a geometric mean function for
    indicator aggregation, an average annual rate of reduction function
    for indicator progress tracking, and convenience clients for the World
    Health Organization Global Health Observatory (GHO) OData API
    <https://ghoapi.azureedge.net/api/> and the United Nations
    Sustainable Development Goals (SDG) API
    <https://unstats.un.org/SDGAPI/swagger/>.
License: MIT + file LICENSE
Language: en-US
URL: https://github.com/shanlong-who/DSIR,
        https://shanlong-who.github.io/DSIR/
BugReports: https://github.com/shanlong-who/DSIR/issues
Encoding: UTF-8
Depends: R (>= 4.1.0)
Imports: cli, flextable (>= 0.9.0), ggplot2 (>= 3.4.0), httr2, rlang,
        tibble (>= 3.0.0)
Suggests: patchwork, officer, testthat (>= 3.0.0), httptest2, knitr,
        rmarkdown, dplyr (>= 1.1.0)
Config/testthat/edition: 3
LazyData: true
VignetteBuilder: knitr
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-07-02 02:23:13 UTC; dings
Author: Shanlong Ding [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-9048-6670>)
Maintainer: Shanlong Ding <dings@who.int>
Repository: CRAN
Date/Publication: 2026-07-02 06:00:02 UTC

# Generated by roxygen2: do not edit by hand

export(bind_indicators)
export(dsi_flextable_defaults)
export(geomean)
export(ggpie)
export(gho_clean)
export(gho_count)
export(gho_coverage)
export(gho_data)
export(gho_dimensions)
export(gho_has_data)
export(gho_indicators)
export(iso3_to_m49)
export(iso3_to_region)
export(m49_to_iso3)
export(scale_x_dsi_col)
export(scale_y_dsi_col)
export(sdg_areas)
export(sdg_clean)
export(sdg_coverage)
export(sdg_data)
export(sdg_goals)
export(sdg_indicators)
export(sdg_targets)
export(theme_dsi)
export(theme_dsi_facet)
importFrom(rlang,"%||%")
importFrom(rlang,.data)
importFrom(tibble,as_tibble)
importFrom(tibble,tibble)

# DSIR <a href="https://shanlong-who.github.io/DSIR/"><img src="man/figures/logo.jpg" align="right" height="120" alt="DSIR website" /></a>

> Data Science Infrastructure for Global Health

<!-- badges: start -->
[![CRAN status](https://www.r-pkg.org/badges/version/DSIR)](https://CRAN.R-project.org/package=DSIR)
[![Lifecycle: stable](https://img.shields.io/badge/lifecycle-stable-brightgreen.svg)](https://lifecycle.r-lib.org/articles/stages.html#stable)
[![CRAN downloads](https://cranlogs.r-pkg.org/badges/grand-total/DSIR)](https://cran.r-project.org/package=DSIR)
<!-- badges: end -->

An R package for global-health data work. Bundles country metadata 
and lightweight clients for the [WHO Global Health Observatory][gho] 
and [UN Sustainable Development Goals][sdg] APIs, plus reusable 
WHO-style themes for `ggplot2` and `flextable` so that charts and 
tables produced from this data look consistent across reports.

[gho]: https://www.who.int/data/gho
[sdg]: https://unstats.un.org/sdgs

Documentation: <https://shanlong-who.github.io/DSIR/>

## Installation

```r
# from CRAN
install.packages("DSIR")

# or the development version from GitHub  
# install.packages("remotes")
remotes::install_github("shanlong-who/DSIR")
```

## Features

### Country metadata

**`who_countries`** — a tibble of all 194 WHO Member States with 
ISO3, ISO2, UN M49 codes, official and short names, WHO region, 
and a `is_pic` flag for Pacific Island Countries.

```r
library(DSIR)
library(dplyr)

who_countries

# Filter Member States in the Western Pacific Region
who_countries |>
  filter(who_region == "WPR") |>
  select(iso3, name_short, is_pic)
```

**Regional ISO3 vectors** — convenience character vectors for 
each WHO region, derived from `who_countries`:

```r
wpro_cty   # 28 Western Pacific Member States (since May 2025)
afro_cty   # 47 African Region Member States
amro_cty   # 35 Region of the Americas Member States
searo_cty  # 10 South-East Asia Region Member States
euro_cty   # 53 European Region Member States
emro_cty   # 21 Eastern Mediterranean Region Member States
pic_cty    # 14 Pacific Island Country Member States (subset of WPR)
```

**`iso3_to_region()`** — quick lookup from ISO3 codes to WHO regions.
Vectorized; returns `NA` for codes not matching a Member State.

```r
iso3_to_region("PHL")                          # "WPR"
iso3_to_region(c("PHL", "FRA", "ZAF", "XYZ"))  # "WPR" "EUR" "AFR" NA
```

**`iso3_to_m49()` / `m49_to_iso3()`** — convert between ISO3 codes
and UN M49 numeric codes in either direction. Useful for moving
between GHO (ISO3) and SDG (M49) workflows. Case-insensitive on the
ISO3 side; both accept zero-padded or bare M49 input on the M49 side;
non-Member areas return `NA`.

```r
iso3_to_m49("PHL")                             # "608"
iso3_to_m49(c("PHL", "FRA", "JPN"))            # "608" "250" "392"
iso3_to_m49(c("PRI", "PHL"))                   # NA "608"

m49_to_iso3(c("608", "250", "392"))            # "PHL" "FRA" "JPN"
m49_to_iso3(c("076", "76"))                    # "BRA" "BRA"
m49_to_iso3("900")                             # NA  (world aggregate)
```

In practice you rarely need to call these directly: `sdg_data()` and
`sdg_coverage()` accept ISO3 codes for their `area` argument and
convert via `iso3_to_m49()` internally, and `sdg_clean()` populates
its `iso3` column via `m49_to_iso3()`.

### Visualization

**`theme_dsi()` and `theme_dsi_facet()`** — publication-ready `ggplot2` 
themes. Use `theme_dsi()` for single-panel charts and `theme_dsi_facet()` 
for faceted plots; the facet variant adds panel borders, light strip 
backgrounds, and panel spacing tuned for multi-panel layouts.

```r
library(ggplot2)
library(dplyr)

# Single panel — theme_dsi()
who_countries |>
  count(who_region) |>
  ggplot(aes(reorder(who_region, n), n)) +
  geom_col(fill = "#0093D5") +
  coord_flip() +
  scale_y_dsi_col() +
  theme_dsi() +
  labs(title = "WHO Member States by region", x = NULL, y = NULL)

# Faceted — theme_dsi_facet()
who_countries |>
  count(who_region, is_pic) |>
  ggplot(aes(reorder(who_region, n), n, fill = is_pic)) +
  geom_col() +
  coord_flip() +
  scale_y_dsi_col() +
  facet_wrap(~ is_pic, labeller = as_labeller(
    c(`TRUE` = "Pacific Island Countries", `FALSE` = "Other Member States")
  )) +
  scale_fill_manual(values = c(`TRUE` = "#0093D5", `FALSE` = "grey70"),
                    guide = "none") +
  theme_dsi_facet() +
  labs(title = "WHO Member States by region and PIC status",
       x = NULL, y = NULL)
```

**`scale_y_dsi_col()` and `scale_x_dsi_col()`** — drop-in replacements for 
`scale_y_continuous()` and `scale_x_continuous()` that remove the default 
lower expansion, so columns in bar charts sit flush with the axis instead 
of floating above it. Pick the one that matches where you mapped the 
value: `scale_y_dsi_col()` when `value` is the `y` aesthetic (including 
horizontal bars made with `coord_flip()` — the aesthetic is still `y`), 
and `scale_x_dsi_col()` when `value` is the `x` aesthetic (e.g. 
`geom_col(aes(value, category))`). Both accept any argument that 
`scale_*_continuous()` accepts.

```r
# Vertical bars
ggplot(mtcars, aes(factor(cyl))) +
  geom_bar(fill = "#0093D5") +
  scale_y_dsi_col() +
  theme_dsi() +
  labs(title = "Cars by cylinder count", x = "Cylinders", y = NULL)
```

**`dsi_flextable_defaults()`** — one-line setup for `flextable` 
formatting (booktabs style, bold headers, paddings).

```r
dsi_flextable_defaults()
```

**`ggpie()`** — quick pie charts with automatic percentage labels.

```r
df <- data.frame(
  region = c("AFR", "AMR", "EUR", "WPR", "SEAR", "EMR"),
  countries = c(47, 35, 53, 28, 10, 21)
)
ggpie(df, "region", "countries", .offset = 1.2)
```

### Utilities

**`geomean()`** — geometric mean, with optional weights. Useful for 
aggregating ratio-based health indicators where the composite is 
multiplicative — e.g. UHC service-coverage tracers.

```r
geomean(c(0.6, 0.8, 0.95))                      # ~0.772
geomean(c(0.6, 0.8, 0.95), w = c(2, 1, 1))      # weighted version
```

### WHO GHO API

**Check availability before downloading.** GHO has thousands of 
indicators but any one of them may not cover the countries or years 
you need. Three lightweight helpers ask the server what is available 
without transferring observations:

```r
# Yes / no for a given indicator + filter (TRUE / FALSE / NA on failure)
gho_has_data("WHOSIS_000001", area = "FRA")

# Row count the same filter would return — useful for sizing a download
gho_count("WHOSIS_000001", area = wpro_cty)

# Per-country year coverage and observation counts
gho_coverage("WHOSIS_000001", area = c("FRA", "DEU", "JPN"))
#>   location year_min year_max n_obs
#> 1 DEU          2000     2021    66
#> 2 FRA          2000     2021    66
#> 3 JPN          2000     2021    66
```

**Fetch and clean.** The typical workflow is **search → fetch → clean**:

```r
# Search indicators by keyword
gho_indicators("mortality")

# Discover available dimensions for an indicator
gho_dimensions("NCDMORT3070")
gho_dimensions("NCDMORT3070", dimension = "Dim1")

# Fetch country-level data
gho_data("NCDMORT3070", spatial_type = "country")

# Fetch with area and year filters
gho_data(
  indicator = "WHOSIS_000001", 
  area      = wpro_cty,
  year_from = 2015
)

# Tidy the raw GHO response
raw <- gho_data("NCDMORT3070", spatial_type = "country", area = wpro_cty)
gho_clean(raw)
```

`gho_clean()` produces the **unified DSIR cleaned-indicator schema** —
the same 15-column shape produced by `sdg_clean()`, so GHO and SDG
output can be combined directly with `bind_indicators()` (see below).
Output columns: `source`, `id`, `indicator`, `location`, `iso3`,
`location_name`, `year`, `value`, `value_num`, `low`, `high`, `series`,
`dim1`–`dim3`. Source columns missing from the raw response (e.g.
`Low` / `High` for indicators without confidence intervals) are filled
with typed `NA`.

### UN SDG API

Same pattern as GHO: **search → fetch → clean**.

```r
# Browse goals, targets, indicators, and geographic areas
sdg_goals()
sdg_targets()
sdg_indicators()
sdg_areas()

# Search indicators by keyword — AND semantics, case-insensitive
# substring match on the indicator description (client-side filter)
sdg_indicators("mortality")
sdg_indicators("mortality cancer")
sdg_indicators(c("maternal", "mortality"))

# Fetch indicator data — `area` accepts ISO3 codes (converted internally)
# or UN M49 numeric codes. ISO3 lets DSIR's regional vectors be passed
# directly, the same way they work with the GHO client.
sdg_data("3.2.1", area = "PHL", year_from = 2015, year_to = 2023)
sdg_data("3.4.1", area = wpro_cty)

# M49 also works (e.g. when copy-pasting codes from sdg_areas())
sdg_data("3.2.1", area = "608", year_from = 2015, year_to = 2023)

# Tidy the SDG response
raw <- sdg_data("3.2.1", area = "PHL")
sdg_clean(raw)
```

`sdg_clean()` produces the same **unified 15-column schema** as
`gho_clean()`: `source`, `id`, `indicator`, `location`, `iso3`,
`location_name`, `year`, `value`, `value_num`, `low`, `high`, `series`,
`dim1`–`dim3`. SDG-side fields populate `id` (the indicator code, e.g.
`"3.4.1"`), `indicator` (the human-readable series description),
`location` (UN M49 numeric code), `iso3` (via `m49_to_iso3()` — `NA`
for region / world aggregates and non-Member areas), `location_name`,
and `series`. The GHO-only `dim1`–`dim3` columns are `NA` for SDG
rows. `value` is kept as character to preserve non-numeric entries
(`"<0.1"`, aggregate notes); `value_num` is the numeric coercion
(`NA` where coercion fails).

### Combining GHO and SDG output

Because both cleaners produce the same schema, you can stack their
output with `bind_indicators()` and use the `source` column to keep
track of where each row came from:

```r
gho <- gho_data("NCDMORT3070", area = wpro_cty) |> gho_clean()
sdg <- sdg_data("3.4.1",        area = wpro_cty) |> sdg_clean()
bind_indicators(gho, sdg)
```

**Exploring series.** A single SDG indicator often contains 
several series — for example different vaccines, sex strata, 
or causes of death — each with its own country / year coverage. 
`sdg_coverage()` summarises the year range and observation count 
per `(location, series)` so you can see what is available before 
deciding which series to analyse.

```r
# 3.b.1 (vaccine coverage) is published as four separate series
sdg_coverage("3.b.1", area = c("156", "608"))
#>   location series      year_min year_max n_obs
#> 1 156      SH_ACS_DTP3     2000     2023    24
#> 2 156      SH_ACS_HPV      2018     2023     6
#> 3 156      SH_ACS_MCV2     2000     2023    24
#> 4 156      SH_ACS_PCV3     2017     2023     7
#> 5 608      SH_ACS_DTP3     2000     2023    24
#> 6 608      SH_ACS_HPV      2017     2023     7
#> 7 608      SH_ACS_MCV2     2000     2023    24
#> 8 608      SH_ACS_PCV3     2014     2023    10
```

GHO-style `has_data()` / `count()` helpers are intentionally not 
provided for SDG because SDG data is generally complete enough 
that pre-flight checks add little value.

## License

MIT — © 2026 Shanlong Ding

# DSIR <a href="https://shanlong-who.github.io/DSIR/"><img src="man/figures/logo.jpg" align="right" height="120" alt="DSIR website" /></a>

> Data Science Infrastructure for Global Health

<!-- badges: start -->
[![CRAN status](https://www.r-pkg.org/badges/version/DSIR)](https://CRAN.R-project.org/package=DSIR)
[![Lifecycle: stable](https://img.shields.io/badge/lifecycle-stable-brightgreen.svg)](https://lifecycle.r-lib.org/articles/stages.html#stable)
[![CRAN downloads](https://cranlogs.r-pkg.org/badges/grand-total/DSIR)](https://cran.r-project.org/package=DSIR)
<!-- badges: end -->

An R package for global-health data work. Bundles country metadata 
and lightweight clients for the [WHO Global Health Observatory][gho] 
and [UN Sustainable Development Goals][sdg] APIs, plus reusable 
WHO-style themes for `ggplot2` and `flextable` so that charts and 
tables produced from this data look consistent across reports.

[gho]: https://www.who.int/data/gho
[sdg]: https://unstats.un.org/sdgs

Documentation: <https://shanlong-who.github.io/DSIR/>

## Installation

```r
# from CRAN
install.packages("DSIR")

# or the development version from GitHub  
# install.packages("remotes")
remotes::install_github("shanlong-who/DSIR")
```

## Features

### Country metadata

**`who_countries`** — a tibble of all 194 WHO Member States with 
ISO3, ISO2, UN M49 codes, official and short names, WHO region, 
and a `is_pic` flag for Pacific Island Countries.

```r
library(DSIR)
library(dplyr)

who_countries

# Filter Member States in the Western Pacific Region
who_countries |>
  filter(who_region == "WPR") |>
  select(iso3, name_short, is_pic)
```

**Regional ISO3 vectors** — convenience character vectors for 
each WHO region, derived from `who_countries`:

```r
wpro_cty   # 28 Western Pacific Member States (since May 2025)
afro_cty   # 47 African Region Member States
amro_cty   # 35 Region of the Americas Member States
searo_cty  # 10 South-East Asia Region Member States
euro_cty   # 53 European Region Member States
emro_cty   # 21 Eastern Mediterranean Region Member States
pic_cty    # 14 Pacific Island Country Member States (subset of WPR)
```

**`iso3_to_region()`** — quick lookup from ISO3 codes to WHO regions.
Vectorized; returns `NA` for codes not matching a Member State.

```r
iso3_to_region("PHL")                          # "WPR"
iso3_to_region(c("PHL", "FRA", "ZAF", "XYZ"))  # "WPR" "EUR" "AFR" NA
```

**`iso3_to_m49()` / `m49_to_iso3()`** — convert between ISO3 codes
and UN M49 numeric codes in either direction. Useful for moving
between GHO (ISO3) and SDG (M49) workflows. Case-insensitive on the
ISO3 side; both accept zero-padded or bare M49 input on the M49 side;
non-Member areas return `NA`.

```r
iso3_to_m49("PHL")                             # "608"
iso3_to_m49(c("PHL", "FRA", "JPN"))            # "608" "250" "392"
iso3_to_m49(c("PRI", "PHL"))                   # NA "608"

m49_to_iso3(c("608", "250", "392"))            # "PHL" "FRA" "JPN"
m49_to_iso3(c("076", "76"))                    # "BRA" "BRA"
m49_to_iso3("900")                             # NA  (world aggregate)
```

In practice you rarely need to call these directly: `sdg_data()` and
`sdg_coverage()` accept ISO3 codes for their `area` argument and
convert via `iso3_to_m49()` internally, and `sdg_clean()` populates
its `iso3` column via `m49_to_iso3()`.

### Visualization

**`theme_dsi()` and `theme_dsi_facet()`** — publication-ready `ggplot2` 
themes. Use `theme_dsi()` for single-panel charts and `theme_dsi_facet()` 
for faceted plots; the facet variant adds panel borders, light strip 
backgrounds, and panel spacing tuned for multi-panel layouts.

```r
library(ggplot2)
library(dplyr)

# Single panel — theme_dsi()
who_countries |>
  count(who_region) |>
  ggplot(aes(reorder(who_region, n), n)) +
  geom_col(fill = "#0093D5") +
  coord_flip() +
  scale_y_dsi_col() +
  theme_dsi() +
  labs(title = "WHO Member States by region", x = NULL, y = NULL)

# Faceted — theme_dsi_facet()
who_countries |>
  count(who_region, is_pic) |>
  ggplot(aes(reorder(who_region, n), n, fill = is_pic)) +
  geom_col() +
  coord_flip() +
  scale_y_dsi_col() +
  facet_wrap(~ is_pic, labeller = as_labeller(
    c(`TRUE` = "Pacific Island Countries", `FALSE` = "Other Member States")
  )) +
  scale_fill_manual(values = c(`TRUE` = "#0093D5", `FALSE` = "grey70"),
                    guide = "none") +
  theme_dsi_facet() +
  labs(title = "WHO Member States by region and PIC status",
       x = NULL, y = NULL)
```

**`scale_y_dsi_col()` and `scale_x_dsi_col()`** — drop-in replacements for 
`scale_y_continuous()` and `scale_x_continuous()` that remove the default 
lower expansion, so columns in bar charts sit flush with the axis instead 
of floating above it. Pick the one that matches where you mapped the 
value: `scale_y_dsi_col()` when `value` is the `y` aesthetic (including 
horizontal bars made with `coord_flip()` — the aesthetic is still `y`), 
and `scale_x_dsi_col()` when `value` is the `x` aesthetic (e.g. 
`geom_col(aes(value, category))`). Both accept any argument that 
`scale_*_continuous()` accepts.

```r
# Vertical bars
ggplot(mtcars, aes(factor(cyl))) +
  geom_bar(fill = "#0093D5") +
  scale_y_dsi_col() +
  theme_dsi() +
  labs(title = "Cars by cylinder count", x = "Cylinders", y = NULL)
```

**`dsi_flextable_defaults()`** — one-line setup for `flextable` 
formatting (booktabs style, bold headers, paddings).

```r
dsi_flextable_defaults()
```

**`ggpie()`** — quick pie charts with automatic percentage labels.

```r
df <- data.frame(
  region = c("AFR", "AMR", "EUR", "WPR", "SEAR", "EMR"),
  countries = c(47, 35, 53, 28, 10, 21)
)
ggpie(df, "region", "countries", .offset = 1.2)
```

### Utilities

**`geomean()`** — geometric mean, with optional weights. Useful for 
aggregating ratio-based health indicators where the composite is 
multiplicative — e.g. UHC service-coverage tracers.

```r
geomean(c(0.6, 0.8, 0.95))                      # ~0.772
geomean(c(0.6, 0.8, 0.95), w = c(2, 1, 1))      # weighted version
```

**`aarr()`** — average annual rate of reduction, the standard 
WHO / UNICEF metric for tracking progress in declining indicators 
(maternal / child mortality, premature NCD mortality, stunting). 
Fits an OLS line to `log(value)` against `year` (the UNICEF-standard 
method; an endpoint-only method is also available). Positive values 
mean the indicator is declining; multiply by 100 to compare with 
published tables.

```r
years  <- 2000:2015
values <- 100 * (1 - 0.024) ^ (years - 2000)
aarr(years, values)                             # 0.024 = 2.4%/yr decline
aarr(years, values, method = "endpoint")        # first/last years only

# Slots into a grouped summarise() on cleaned indicator data
gho_data("NCDMORT3070", area = wpro_cty, dim1 = "SEX_BTSX") |>
  gho_clean() |>
  dplyr::summarise(aarr = aarr(year, value_num), .by = iso3)
```

### WHO GHO API

**Check availability before downloading.** GHO has thousands of 
indicators but any one of them may not cover the countries or years 
you need. Three lightweight helpers ask the server what is available 
without transferring observations:

```r
# Yes / no for a given indicator + filter (TRUE / FALSE / NA on failure)
gho_has_data("WHOSIS_000001", area = "FRA")

# Row count the same filter would return — useful for sizing a download
gho_count("WHOSIS_000001", area = wpro_cty)

# Per-country year coverage and observation counts
gho_coverage("WHOSIS_000001", area = c("FRA", "DEU", "JPN"))
#>   location year_min year_max n_obs
#> 1 DEU          2000     2021    66
#> 2 FRA          2000     2021    66
#> 3 JPN          2000     2021    66
```

**Fetch and clean.** The typical workflow is **search → fetch → clean**:

```r
# Search indicators by keyword
gho_indicators("mortality")

# Discover available dimensions for an indicator
gho_dimensions("NCDMORT3070")
gho_dimensions("NCDMORT3070", dimension = "Dim1")

# Fetch country-level data
gho_data("NCDMORT3070", spatial_type = "country")

# Fetch with area and year filters
gho_data(
  indicator = "WHOSIS_000001", 
  area      = wpro_cty,
  year_from = 2015
)

# Filter breakdown dimensions server-side — e.g. keep only the
# both-sexes rows instead of downloading every stratum. Use
# gho_dimensions() to discover the values available for an indicator.
gho_data("NCDMORT3070", spatial_type = "country", dim1 = "SEX_BTSX")

# Tidy the raw GHO response
raw <- gho_data("NCDMORT3070", spatial_type = "country", area = wpro_cty)
gho_clean(raw)
```

`gho_clean()` produces the **unified DSIR cleaned-indicator schema** —
the same 15-column shape produced by `sdg_clean()`, so GHO and SDG
output can be combined directly with `bind_indicators()` (see below).
Output columns: `source`, `id`, `indicator`, `location`, `iso3`,
`location_name`, `year`, `value`, `value_num`, `low`, `high`, `series`,
`dim1`–`dim3`. Source columns missing from the raw response (e.g.
`Low` / `High` for indicators without confidence intervals) are filled
with typed `NA`.

### UN SDG API

Same pattern as GHO: **search → fetch → clean**.

```r
# Browse goals, targets, indicators, and geographic areas
sdg_goals()
sdg_targets()
sdg_indicators()
sdg_areas()

# Search indicators by keyword — AND semantics, case-insensitive
# substring match on the indicator description (client-side filter)
sdg_indicators("mortality")
sdg_indicators("mortality cancer")
sdg_indicators(c("maternal", "mortality"))

# Fetch indicator data — `area` accepts ISO3 codes (converted internally)
# or UN M49 numeric codes. ISO3 lets DSIR's regional vectors be passed
# directly, the same way they work with the GHO client.
sdg_data("3.2.1", area = "PHL", year_from = 2015, year_to = 2023)
sdg_data("3.4.1", area = wpro_cty)

# M49 also works (e.g. when copy-pasting codes from sdg_areas())
sdg_data("3.2.1", area = "608", year_from = 2015, year_to = 2023)

# Tidy the SDG response
raw <- sdg_data("3.2.1", area = "PHL")
sdg_clean(raw)
```

`sdg_clean()` produces the same **unified 15-column schema** as
`gho_clean()`: `source`, `id`, `indicator`, `location`, `iso3`,
`location_name`, `year`, `value`, `value_num`, `low`, `high`, `series`,
`dim1`–`dim3`. SDG-side fields populate `id` (the indicator code, e.g.
`"3.4.1"`), `indicator` (the human-readable series description),
`location` (UN M49 numeric code), `iso3` (via `m49_to_iso3()` — `NA`
for region / world aggregates and non-Member areas), `location_name`,
and `series`. The GHO-only `dim1`–`dim3` columns are `NA` for SDG
rows. `value` is kept as character to preserve non-numeric entries
(`"<0.1"`, aggregate notes); `value_num` is the numeric coercion
(`NA` where coercion fails).

### Combining GHO and SDG output

Because both cleaners produce the same schema, you can stack their
output with `bind_indicators()` and use the `source` column to keep
track of where each row came from:

```r
gho <- gho_data("NCDMORT3070", area = wpro_cty) |> gho_clean()
sdg <- sdg_data("3.4.1",        area = wpro_cty) |> sdg_clean()
bind_indicators(gho, sdg)
```

**Exploring series.** A single SDG indicator often contains 
several series — for example different vaccines, sex strata, 
or causes of death — each with its own country / year coverage. 
`sdg_coverage()` summarises the year range and observation count 
per `(location, series)` so you can see what is available before 
deciding which series to analyse.

```r
# 3.b.1 (vaccine coverage) is published as four separate series
sdg_coverage("3.b.1", area = c("156", "608"))
#>   location series      year_min year_max n_obs
#> 1 156      SH_ACS_DTP3     2000     2023    24
#> 2 156      SH_ACS_HPV      2018     2023     6
#> 3 156      SH_ACS_MCV2     2000     2023    24
#> 4 156      SH_ACS_PCV3     2017     2023     7
#> 5 608      SH_ACS_DTP3     2000     2023    24
#> 6 608      SH_ACS_HPV      2017     2023     7
#> 7 608      SH_ACS_MCV2     2000     2023    24
#> 8 608      SH_ACS_PCV3     2014     2023    10
```

GHO-style `has_data()` / `count()` helpers are intentionally not 
provided for SDG because SDG data is generally complete enough 
that pre-flight checks add little value.

## License

MIT — © 2026 Shanlong Ding

# ============================================================
# DSIR Theme Demo
# ============================================================

library(DSIR)
library(ggplot2)
library(flextable)

has_patchwork <- requireNamespace("patchwork", quietly = TRUE)
has_officer   <- requireNamespace("officer",   quietly = TRUE)

# ------------------------------------------------------------
# 1. theme_dsi() \u2014 ggplot2 theme showcase
# ------------------------------------------------------------

# Scatter plot
p1 <- ggplot(msleep, aes(bodywt, brainwt, color = vore)) +
  geom_point(size = 2, alpha = 0.7, na.rm = TRUE) +
  scale_x_log10() +
  scale_y_log10() +
  labs(
    title    = "Brain vs Body Weight",
    subtitle = "Mammalian species from msleep dataset",
    x = "Body weight (kg, log scale)",
    y = "Brain weight (kg, log scale)",
    color = "Diet",
    caption = "Source: ggplot2::msleep"
  ) +
  theme_dsi()

# Bar chart
df_vore <- na.omit(msleep[, "vore", drop = FALSE])
df_vore <- as.data.frame(table(vore = df_vore$vore))

p2 <- ggplot(df_vore, aes(reorder(vore, -Freq), Freq, fill = vore)) +
  geom_col(show.legend = FALSE, width = 0.6) +
  geom_text(aes(label = Freq), vjust = -0.5, color = "steelblue", fontface = "bold") +
  scale_y_continuous(expand = expansion(mult = c(0, 0.15))) +
  labs(
    title    = "Number of Species by Diet Type",
    subtitle = "Counts from msleep dataset",
    x = NULL, y = "Count"
  ) +
  theme_dsi()

# Line chart
p3 <- ggplot(economics, aes(date, unemploy / 1000)) +
  geom_line(color = "steelblue", linewidth = 0.6) +
  geom_area(fill = "steelblue", alpha = 0.1) +
  labs(
    title    = "US Unemployment Over Time",
    subtitle = "In thousands",
    x = NULL, y = "Unemployed (thousands)",
    caption  = "Source: ggplot2::economics"
  ) +
  theme_dsi()

# Box plot
p4 <- ggplot(mpg, aes(reorder(class, hwy, FUN = median), hwy, fill = class)) +
  geom_boxplot(show.legend = FALSE, alpha = 0.7, outlier.color = "steelblue") +
  coord_flip() +
  labs(
    title    = "Highway MPG by Vehicle Class",
    subtitle = "Ordered by median",
    x = NULL, y = "Highway MPG"
  ) +
  theme_dsi()

if (has_patchwork) {
  combined <- (p1 + p2) / (p3 + p4) +
    patchwork::plot_annotation(
      title    = "theme_dsi() Gallery",
      subtitle = "Publication-ready charts with a consistent look",
      theme    = theme_dsi(base_size = 14)
    )
  print(combined)
} else {
  print(p1); print(p2); print(p3); print(p4)
  message("Install 'patchwork' to see the combined gallery.")
}


# ------------------------------------------------------------
# 2. dsi_flextable_defaults() \u2014 flextable theme showcase
# ------------------------------------------------------------

dsi_flextable_defaults()

ft1 <- data.frame(
  Region   = c("AFRO", "AMRO", "EMRO", "EURO", "SEARO", "WPRO"),
  Members  = c(47, 35, 22, 53, 10, 28),
  Example  = c("Nigeria", "Brazil", "Egypt", "Germany", "India", "China")
) |>
  flextable() |>
  set_caption("WHO Regional Offices Overview") |>
  autofit()

print(ft1)

ft2 <- head(mtcars[, c("mpg", "cyl", "hp", "wt", "qsec")], 8) |>
  flextable() |>
  set_caption("Motor Trend Car Data (Top 8)") |>
  colformat_double(digits = 1) |>
  autofit()

print(ft2)

if (has_officer) {
  doc <- officer::read_docx()
  doc <- flextable::body_add_flextable(doc, ft1)
  doc <- officer::body_add_par(doc, "")
  doc <- flextable::body_add_flextable(doc, ft2)
  out <- tempfile(fileext = ".docx")
  print(doc, target = out)
  message("Demo complete. Word file written to: ", out)
} else {
  message("Install 'officer' to export tables to Word.")
}

#' Convert UN M49 Numeric Codes to ISO3 Codes
#'
#' Maps UN M49 numeric area codes to ISO 3166-1 alpha-3 country codes
#' using the [`who_countries`] dataset shipped with DSIR. Counterpart
#' to [iso3_to_m49()] and used internally by [sdg_clean()] to populate
#' the `iso3` column on SDG output.
#'
#' M49 codes that do not correspond to a WHO Member State return `NA`.
#' This includes region / world aggregates (e.g. `"900"` for World,
#' `"001"` for World, `"419"` for Latin America and the Caribbean)
#' and codes for non-Member areas (e.g. Puerto Rico, Tokelau).
#'
#' Input accepts either the zero-padded form (`"076"`) or the bare
#' form (`"76"`); both are normalised before lookup. Non-numeric
#' input returns `NA` (with a single warning from the underlying
#' [as.integer()] coercion).
#'
#' @param m49 Character vector of M49 codes.
#'
#' @return A character vector the same length as `m49`. Non-Member
#'   codes (region aggregates, non-Member areas) return `NA`.
#' @seealso [`who_countries`], [iso3_to_m49()], [sdg_clean()].
#' @export
#'
#' @examples
#' m49_to_iso3(c("608", "250", "392"))
#' # "PHL" "FRA" "JPN"
#'
#' # Zero-padded and bare forms both accepted
#' m49_to_iso3(c("076", "76"))
#' # "BRA" "BRA"
#'
#' # Non-Member areas / aggregates return NA
#' m49_to_iso3(c("900", "608"))
#' # NA "PHL"
m49_to_iso3 <- function(m49) {
  if (!is.character(m49)) {
    cli::cli_abort(
      "{.arg m49} must be a character vector, not {.cls {class(m49)}}."
    )
  }
  padded <- formatC(suppressWarnings(as.integer(m49)),
                    width = 3, flag = "0")
  # formatC(NA_integer_) returns "NA" — turn it back into NA_character_
  # so match() returns NA, not a spurious match against a literal "NA".
  padded[padded == "NA"] <- NA_character_
  m <- match(padded, who_countries$m49_code)
  who_countries$iso3[m]
}

test_that("m49_to_iso3 maps known codes correctly", {
  expect_equal(m49_to_iso3(c("608", "250", "392")),
               c("PHL", "FRA", "JPN"))
})

test_that("m49_to_iso3 accepts zero-padded and bare forms", {
  # Brazil's M49 is 076; bare 76 should match the same row.
  expect_equal(m49_to_iso3("076"), "BRA")
  expect_equal(m49_to_iso3("76"),  "BRA")
})

test_that("m49_to_iso3 returns NA for aggregates and non-Member areas", {
  # 900/001 are SDG-style world aggregates; never in who_countries.
  expect_true(is.na(m49_to_iso3("900")))
  expect_true(is.na(m49_to_iso3("001")))
})

test_that("m49_to_iso3 preserves vector length and position", {
  res <- m49_to_iso3(c("608", "900", "250"))
  expect_length(res, 3L)
  expect_equal(res, c("PHL", NA_character_, "FRA"))
})

test_that("m49_to_iso3 handles non-numeric and NA input gracefully", {
  # as.integer() warns on non-numeric input — m49_to_iso3 swallows that
  # and returns NA.
  expect_true(is.na(m49_to_iso3("ABC")))
  expect_true(is.na(m49_to_iso3(NA_character_)))
})

test_that("m49_to_iso3 handles empty input", {
  out <- m49_to_iso3(character(0))
  expect_length(out, 0L)
  expect_type(out, "character")
})

test_that("m49_to_iso3 errors on non-character input", {
  expect_error(m49_to_iso3(608),  "must be a character")
  expect_error(m49_to_iso3(TRUE), "must be a character")
  expect_error(m49_to_iso3(NULL), "must be a character")
})

test_that("iso3_to_m49 and m49_to_iso3 round-trip on Member States", {
  iso <- c("PHL", "FRA", "JPN", "BRA", "USA")
  expect_equal(m49_to_iso3(iso3_to_m49(iso)), iso)
})

#' Explore Series Coverage of an SDG Indicator
#'
#' A single SDG indicator (for example `"3.4.1"`, NCD mortality) is
#' typically published as several **series** stratified by sex, age,
#' or cause. Different series may have different country and year
#' coverage. `sdg_coverage()` summarises year range and observation
#' count per `(location, series)` combination, so you can see which
#' series exist for an indicator and how each one is covered before
#' committing to a downstream analysis.
#'
#' Unlike the GHO availability helpers, this function is a
#' series-exploration tool rather than a payload-saving precheck:
#' SDG data is generally complete enough that GHO-style
#' `has_data()` / `count()` helpers add little value, so they are
#' intentionally not provided. The SDG API also offers no
#' payload-reduction option (no `$select` equivalent), so
#' `sdg_coverage()` calls [sdg_data()] internally and aggregates
#' the result client-side.
#'
#' @param indicator Character vector of SDG indicator codes
#'   (e.g. `"3.4.1"`).
#' @param area Character vector of country/area codes. Accepts either
#'   ISO3 codes (e.g. `c("PHL", "FRA")`) — converted automatically via
#'   [iso3_to_m49()] — or UN M49 numeric codes (e.g. `c("608", "250")`)
#'   as returned by [sdg_areas()]. Do not mix the two formats in a
#'   single call. Default `NULL` returns all areas. Unknown ISO3 codes
#'   are dropped with a warning before the network call.
#' @param year_from Numeric. Start year filter (inclusive).
#'   Default `NULL`.
#' @param year_to Numeric. End year filter (inclusive).
#'   Default `NULL`.
#'
#' @return A [tibble][tibble::tibble] with one row per
#'   `(location, series)` and columns:
#' * `location` (chr) — area code (`geoAreaCode`).
#' * `series` (chr) — SDG series code.
#' * `year_min` (int) — earliest year with data.
#' * `year_max` (int) — latest year with data.
#' * `n_obs` (int) — number of observations.
#'
#'   Sorted by `location` then `series`. Empty input or service
#'   failure returns an empty tibble with the same five columns.
#' @seealso [sdg_data()], [sdg_indicators()], [gho_coverage()].
#' @export
#'
#' @examples
#' \donttest{
#' # Series available for NCD mortality in China and Brazil
#' sdg_coverage("3.4.1", area = c("156", "076"))
#'
#' # Filter to a year range
#' sdg_coverage("3.4.1", area = "156", year_from = 2015)
#' }
sdg_coverage <- function(indicator, area = NULL,
                         year_from = NULL, year_to = NULL) {
  # Resolve up-front so the "unknown ISO3" warning surfaces from
  # sdg_coverage() rather than being swallowed by the
  # suppressWarnings() wrapper around sdg_data() below.
  area <- .resolve_area(area)

  empty <- tibble::tibble(
    location = character(),
    series   = character(),
    year_min = integer(),
    year_max = integer(),
    n_obs    = integer()
  )

  df <- suppressWarnings(
    sdg_data(indicator, area = area,
             year_from = year_from, year_to = year_to)
  )
  if (!is.data.frame(df) || nrow(df) == 0L) return(empty)
  if (!all(c("geoAreaCode", "series", "timePeriodStart") %in% names(df))) {
    return(empty)
  }

  loc <- as.character(df$geoAreaCode)
  ser <- as.character(df$series)
  yr  <- suppressWarnings(as.integer(df$timePeriodStart))

  # \x1f (US, unit separator) cannot appear in an M49 numeric code or
  # an SDG series code, so it separates the two parts unambiguously.
  key <- paste(loc, ser, sep = "\x1f")
  idx <- split(seq_along(key), key)

  yr_range <- function(x, fn) {
    x <- x[!is.na(x)]
    if (length(x) == 0L) NA_integer_ else as.integer(fn(x))
  }

  parts <- strsplit(names(idx), "\x1f", fixed = TRUE)
  out <- tibble::tibble(
    location = vapply(parts, `[[`, character(1), 1L),
    series   = vapply(parts, `[[`, character(1), 2L),
    year_min = vapply(idx, function(i) yr_range(yr[i], min), integer(1)),
    year_max = vapply(idx, function(i) yr_range(yr[i], max), integer(1)),
    n_obs    = vapply(idx, length, integer(1))
  )

  out[order(out$location, out$series), , drop = FALSE]
}

test_that("sdg_coverage returns the documented 5-column shape", {
  skip_on_cran()
  skip_if_offline()

  out <- sdg_coverage("3.4.1", area = c("156", "608"))
  expect_s3_class(out, "tbl_df")
  expect_named(out, c("location", "series", "year_min", "year_max", "n_obs"))
  expect_type(out$location, "character")
  expect_type(out$series, "character")
  expect_type(out$year_min, "integer")
  expect_type(out$year_max, "integer")
  expect_type(out$n_obs, "integer")
  expect_gt(nrow(out), 0L)
  expect_setequal(out$location, c("156", "608"))
  expect_true(all(out$year_min <= out$year_max))
  expect_true(all(out$n_obs >= 1L))
})

test_that("sdg_coverage exposes multiple series per location for a multi-series indicator", {
  skip_on_cran()
  skip_if_offline()

  # 3.b.1 (vaccine coverage) is published as multiple series:
  # SH_ACS_DTP3, SH_ACS_MCV2, SH_ACS_PCV3, SH_ACS_HPV. So one
  # location should appear with multiple rows.
  out <- sdg_coverage("3.b.1", area = "156")
  series_per_loc <- table(out$location)
  expect_true(any(series_per_loc > 1L))
})

test_that("sdg_coverage is sorted by location then series", {
  skip_on_cran()
  skip_if_offline()

  out <- sdg_coverage("3.b.1", area = c("608", "156"))
  expected <- out[order(out$location, out$series), , drop = FALSE]
  expect_identical(out, expected)
})

test_that("sdg_coverage returns an empty 5-col tibble on no match", {
  skip_on_cran()
  skip_if_offline()

  # "999" is not a valid M49 area code.
  out <- suppressWarnings(sdg_coverage("3.4.1", area = "999"))
  expect_s3_class(out, "tbl_df")
  expect_named(out, c("location", "series", "year_min", "year_max", "n_obs"))
  expect_equal(nrow(out), 0L)
  expect_type(out$location, "character")
  expect_type(out$series, "character")
  expect_type(out$year_min, "integer")
  expect_type(out$year_max, "integer")
  expect_type(out$n_obs, "integer")
})

# Year filtering for `sdg_data()` is implemented client-side as a
# workaround for a UN SDG API bug: sending `timePeriodStart` /
# `timePeriodEnd` as server-side parameters caused HTTP 500 and ~30x
# slowdowns on at least 3.2.1 / area = "608". The previously-broken
# call must now complete quickly and return rows; the filter
# semantics must still match the documented inclusive range.

test_that("year_from works on the previously broken case (3.2.1, 608)", {
  skip_on_cran()
  skip_if_offline()

  t <- system.time({
    out <- sdg_data("3.2.1", area = "608", year_from = 2010)
  })

  expect_s3_class(out, "tbl_df")
  expect_gt(nrow(out), 0L)
  expect_lt(as.numeric(t["elapsed"]), 30)

  yr <- suppressWarnings(as.integer(out$timePeriodStart))
  expect_true(all(yr >= 2010L, na.rm = TRUE))
})

test_that("year_from + year_to bracket the result inclusively", {
  skip_on_cran()
  skip_if_offline()

  out <- sdg_data("3.2.1", area = "608",
                  year_from = 2015, year_to = 2018)
  expect_s3_class(out, "tbl_df")
  expect_gt(nrow(out), 0L)

  yr <- suppressWarnings(as.integer(out$timePeriodStart))
  expect_true(all(yr >= 2015L & yr <= 2018L, na.rm = TRUE))
})

test_that("no year filter returns the full result", {
  skip_on_cran()
  skip_if_offline()

  full       <- sdg_data("3.2.1", area = "608")
  from_2010  <- sdg_data("3.2.1", area = "608", year_from = 2010)

  expect_s3_class(full, "tbl_df")
  expect_gt(nrow(full), 0L)

  # The from_2010 result must be the >= 2010 subset of the full result.
  yr_full <- suppressWarnings(as.integer(full$timePeriodStart))
  manual  <- full[!is.na(yr_full) & yr_full >= 2010L, , drop = FALSE]
  expect_equal(nrow(from_2010), nrow(manual))
})

test_that("sdg_coverage inherits the year-filter fix", {
  skip_on_cran()
  skip_if_offline()

  t <- system.time({
    out <- sdg_coverage("3.2.1", area = "608", year_from = 2010)
  })

  expect_s3_class(out, "tbl_df")
  expect_named(out, c("location", "series", "year_min", "year_max", "n_obs"))
  expect_gt(nrow(out), 0L)
  expect_lt(as.numeric(t["elapsed"]), 30)
  expect_true(all(out$year_min >= 2010L))
})

#' Average Annual Rate of Reduction (AARR)
#'
#' Computes the average annual rate of reduction of an indicator over
#' time — the standard WHO / UNICEF metric for tracking progress in
#' declining indicators such as maternal, neonatal, and under-five
#' mortality, stunting prevalence, or premature NCD mortality.
#'
#' @details
#' Two estimation methods are offered:
#'
#' * `"regression"` (default; the UNICEF-recommended approach): an
#'   ordinary least-squares line is fitted to `log(value)` against
#'   `year`, and the AARR is `1 - exp(b)`, where `b` is the fitted
#'   slope. All observations contribute, so the estimate is robust to
#'   noise in individual years.
#' * `"endpoint"`: only the earliest and latest years are used:
#'   `1 - (v1 / v0) ^ (1 / (y1 - y0))`, where `v0` and `v1` are the
#'   values at the earliest year `y0` and the latest year `y1`.
#'   Intermediate observations are ignored. If several observations
#'   share the earliest or latest year, their mean is used (but see
#'   the note on duplicated years below).
#'
#' **Sign convention.** A *positive* AARR means the indicator is
#' *declining* (progress, for a mortality-type indicator): `0.024`
#' means an average decline of 2.4% per year. A *negative* AARR means
#' the indicator is increasing. Note this is the reverse of a growth
#' rate.
#'
#' **Duplicated years** usually mean the data still mix several strata
#' — for example both sexes plus male / female in `dim1`, or several
#' `series` codes in data cleaned by [gho_clean()] / [sdg_clean()].
#' `aarr()` warns and proceeds (the regression pools the strata), but
#' you almost always want to filter to a single stratum first.
#'
#' **Relation to published figures.** Published WHO / UNICEF tables
#' print the AARR as a percentage (`4.4` meaning 4.4% per year);
#' multiply the value returned by this function by 100 to compare.
#' Note also that WHO's *Trends in Maternal Mortality* reports print a
#' continuous-time rate, `-log(v1 / v0) / (y1 - y0)` (there called
#' ARR), which agrees closely but not exactly with the discrete AARR
#' computed here — small differences from those tables are expected.
#'
#' @param year Numeric vector of years.
#' @param value Numeric vector of indicator values, the same length as
#'   `year`. Values must be positive (the computation is on the log
#'   scale); any zero or negative value yields `NA_real_` with a
#'   warning.
#' @param method Character. `"regression"` (default) or `"endpoint"`.
#'   See Details.
#' @param na.rm Logical. Should pairs with a missing `year` or `value`
#'   be removed before computation? Default `TRUE`. When `FALSE`, any
#'   missing element makes the result `NA_real_`.
#'
#' @return A numeric scalar: the average annual rate of reduction as a
#'   *fraction* (`0.024` = 2.4% per year). Multiply by 100 to compare
#'   with published WHO / UNICEF tables, which print percentages.
#'   Returns `NA_real_` (with a warning) when fewer than two distinct
#'   years remain after `NA` handling, when any value is zero or
#'   negative, or when `year` or `value` contains non-finite values.
#' @seealso [geomean()] for ratio-based indicator aggregation.
#' @export
#'
#' @examples
#' # A perfectly exponential 2.4%/yr decline recovers exactly 0.024
#' years  <- 2000:2015
#' values <- 100 * (1 - 0.024) ^ (years - 2000)
#' aarr(years, values)                       # 0.024
#' 100 * aarr(years, values)                 # 2.4 — as printed in reports
#'
#' # Endpoint method uses only the earliest and latest years
#' aarr(years, values, method = "endpoint")  # also 0.024 here
#'
#' # An increasing indicator gives a negative AARR
#' aarr(2010:2020, 50 * 1.01 ^ (0:10))       # about -0.01
#'
#' # Back-of-envelope projection to 2030 at the observed AARR
#' r <- aarr(years, values)
#' values[length(values)] * (1 - r) ^ (2030 - 2015)
aarr <- function(year, value, method = c("regression", "endpoint"),
                 na.rm = TRUE) {
  # ── Input validation ──────────────────────────────────────────
  method <- match.arg(method)
  if (!is.numeric(year)) {
    cli::cli_abort("{.arg year} must be numeric, not {.cls {class(year)}}.")
  }
  if (!is.numeric(value)) {
    cli::cli_abort("{.arg value} must be numeric, not {.cls {class(value)}}.")
  }
  if (length(year) != length(value)) {
    cli::cli_abort(
      "{.arg year} and {.arg value} must have the same length \\
      ({length(year)} vs {length(value)})."
    )
  }
  if (!isTRUE(na.rm) && !isFALSE(na.rm)) {
    cli::cli_abort("{.arg na.rm} must be a single logical value.")
  }

  # ── NA filtering ──────────────────────────────────────────────
  if (na.rm) {
    ok <- !is.na(year) & !is.na(value)
    year  <- year[ok]
    value <- value[ok]
  }
  if (anyNA(year) || anyNA(value)) return(NA_real_)

  # ── Domain checks ─────────────────────────────────────────────
  # NA and NaN are handled above, so this catches infinite values,
  # which would otherwise abort inside stats::lm().
  if (!all(is.finite(year)) || !all(is.finite(value))) {
    cli::cli_warn(
      "Non-finite values in {.arg year} or {.arg value}; the AARR is \\
      undefined."
    )
    return(NA_real_)
  }
  if (length(unique(year)) < 2L) {
    cli::cli_warn(
      "At least two distinct years are needed to compute the AARR."
    )
    return(NA_real_)
  }
  if (any(value <= 0)) {
    cli::cli_warn(
      "Zero or negative values in {.arg value}; the AARR is undefined \\
      on the log scale."
    )
    return(NA_real_)
  }
  if (anyDuplicated(year)) {
    cli::cli_warn(c(
      "Duplicated years in {.arg year}.",
      "i" = "This usually means the data still mix several strata \\
      (e.g. {.field dim1} or {.field series}); filter to a single \\
      stratum first."
    ))
  }

  # ── Compute ───────────────────────────────────────────────────
  if (method == "regression") {
    b <- stats::coef(stats::lm(log(value) ~ year))[["year"]]
    return(1 - exp(b))
  }

  # method == "endpoint"
  y0 <- min(year)
  y1 <- max(year)
  v0 <- mean(value[year == y0])
  v1 <- mean(value[year == y1])
  1 - (v1 / v0) ^ (1 / (y1 - y0))
}

# Unit tests for aarr(). All offline — pure computation.

test_that("aarr recovers the exact rate from a perfect exponential decline", {
  years  <- 2000:2015
  values <- 100 * (1 - 0.024) ^ (years - 2000)
  # On perfectly exponential data both methods are exact, which locks
  # the formula (1 - exp(b), NOT -b) and the sign convention forever.
  expect_equal(aarr(years, values), 0.024, tolerance = 1e-10)
  expect_equal(aarr(years, values, method = "endpoint"), 0.024,
               tolerance = 1e-10)
})

test_that("aarr regression matches a hand-computed OLS slope on noisy data", {
  years  <- c(2000, 2005, 2010, 2015)
  values <- c(120, 100, 70, 55)
  b <- stats::coef(stats::lm(log(values) ~ years))[["years"]]
  expect_equal(aarr(years, values), 1 - exp(b))
})

test_that("aarr endpoint uses earliest/latest year regardless of input order", {
  years  <- c(2010, 2000, 2005)
  values <- c(80, 100, 95)
  expect_equal(aarr(years, values, method = "endpoint"),
               1 - (80 / 100) ^ (1 / 10))
})

test_that("an increasing indicator yields a negative aarr", {
  years  <- 2010:2020
  values <- 50 * 1.01 ^ (years - 2010)
  expect_equal(aarr(years, values), 1 - 1.01, tolerance = 1e-10)
  expect_lt(aarr(years, values, method = "endpoint"), 0)
})

test_that("aarr handles missing values per na.rm", {
  years  <- c(2000, 2005, 2010, 2015)
  values <- c(100, NA, 70, 55)
  expect_equal(aarr(years, values),
               aarr(years[-2], values[-2]))
  expect_identical(aarr(years, values, na.rm = FALSE), NA_real_)

  # NA in year is removed pairwise too
  years_na <- c(2000, NA, 2010, 2015)
  expect_equal(aarr(years_na, c(100, 90, 70, 55)),
               aarr(c(2000, 2010, 2015), c(100, 70, 55)))
})

test_that("aarr warns and returns NA on non-finite inputs", {
  # Inf passes both the NA filter and the value <= 0 check; without an
  # explicit guard it would abort inside stats::lm() with an opaque
  # error — fatal in grouped summarise() over many countries.
  expect_warning(out <- aarr(2000:2002, c(100, Inf, 80)), "Non-finite")
  expect_identical(out, NA_real_)
  expect_warning(out <- aarr(c(2000, Inf), c(100, 80)), "Non-finite")
  expect_identical(out, NA_real_)
  expect_warning(
    out <- aarr(2000:2002, c(100, Inf, 80), method = "endpoint"),
    "Non-finite"
  )
  expect_identical(out, NA_real_)
})

test_that("aarr warns and returns NA for zero or negative values", {
  expect_warning(out <- aarr(2000:2002, c(10, 0, 5)), "undefined")
  expect_identical(out, NA_real_)
  expect_warning(out <- aarr(2000:2002, c(10, -1, 5)), "undefined")
  expect_identical(out, NA_real_)
})

test_that("aarr warns and returns NA with fewer than two distinct years", {
  expect_warning(out <- aarr(2020, 55), "two distinct years")
  expect_identical(out, NA_real_)
  expect_warning(out <- aarr(c(2020, 2020), c(55, 60)), "two distinct years")
  expect_identical(out, NA_real_)
  # Empty after NA removal
  expect_warning(out <- aarr(c(NA_real_, NA_real_), c(1, 2)),
                 "two distinct years")
  expect_identical(out, NA_real_)
})

test_that("aarr warns on duplicated years but still computes", {
  # Two strata accidentally pooled: the classic unfiltered-dim1 mistake.
  years  <- c(2000, 2000, 2010, 2010)
  values <- c(90, 110, 60, 80)
  expect_warning(out <- aarr(years, values), "Duplicated years")
  expect_type(out, "double")
  expect_false(is.na(out))

  # Endpoint method averages the values at each endpoint year
  expect_warning(
    out_ep <- aarr(years, values, method = "endpoint"),
    "Duplicated years"
  )
  expect_equal(out_ep, 1 - (70 / 100) ^ (1 / 10))
})

test_that("aarr validates its inputs", {
  expect_error(aarr("2000", 1), "numeric")
  expect_error(aarr(2000:2001, "a"), "numeric")
  expect_error(aarr(2000:2002, c(1, 2)), "length")
  expect_error(aarr(2000:2001, c(1, 2), na.rm = NA), "logical")
  expect_error(aarr(2000:2001, c(1, 2), method = "banana"))
})

#' Create a Pie Chart with ggplot2
#'
#' Builds a pie chart from a data frame using one categorical column
#' and one numeric column. Slices are labeled with the category
#' name and percentage share.
#'
#' @param df A data frame.
#' @param .x Column name (string) of the categorical variable used
#'   for the slices.
#' @param .y Column name (string) of the numeric variable used for
#'   the slice values.
#' @param .offset Numeric scalar (> 0). Controls label position
#'   along the slice radius. Default `1` places the label at the
#'   middle of the slice. Smaller values (e.g. `0.5`) move the
#'   label inward toward the centre; larger values (e.g. `2`)
#'   move it outward toward the edge or beyond, useful for
#'   donut-style layouts.
#' @param .color Border color between slices. Default `"white"`.
#' @param .legend Logical. Show the legend? Default `FALSE`.
#' @param .label Logical. Draw `"name\n(pct%)"` labels on the
#'   slices? Default `TRUE`.
#' @param .label_size Label text size in mm. Default `3.5`.
#'
#' @return A `ggplot` object.
#' @export
#'
#' @examples
#' df <- data.frame(
#'   category = c("A", "B", "C"),
#'   value = c(40, 35, 25)
#' )
#' ggpie(df, "category", "value")
ggpie <- function(df, .x, .y, .offset = 1, .color = "white",
                  .legend = FALSE, .label = TRUE, .label_size = 3.5) {
  stopifnot(
    is.data.frame(df),
    is.character(.x), length(.x) == 1L, .x %in% names(df),
    is.character(.y), length(.y) == 1L, .y %in% names(df),
    is.numeric(df[[.y]]), 
    is.numeric(.offset), length(.offset) == 1L, .offset > 0
  )
  
  df_plot <- df
  vals    <- df_plot[[.y]]
  total   <- sum(vals)
  df_plot[[".dsir_label"]] <- paste0(
    df_plot[[.x]], "\n(", round(vals / total * 100, 1), "%)"
  )

  p <- ggplot2::ggplot(df_plot, ggplot2::aes(
    x    = 1,
    y    = .data[[.y]],
    fill = .data[[.x]]
  )) +
    ggplot2::geom_col(color = .color, linewidth = 0.5) +
    ggplot2::coord_polar("y") +
    ggplot2::labs(x = NULL, y = NULL, fill = NULL) +
    ggplot2::theme_void() +
    ggplot2::theme(
      plot.title = ggplot2::element_text(hjust = 0.5, face = "bold")
    )
  
  if (.label) {
    p <- p + ggplot2::geom_text(
      ggplot2::aes(label = .data[[".dsir_label"]], x = .offset),
      position = ggplot2::position_stack(vjust = 0.5),
      size     = .label_size
    )
  }
  
  if (!.legend) {
    p <- p + ggplot2::theme(legend.position = "none")
  }
  
  p
}

test_that("ggpie returns a ggplot object", {
  df <- data.frame(
    category = c("A", "B", "C"),
    value = c(40, 35, 25)
  )
  p <- ggpie(df, "category", "value")
  expect_s3_class(p, "ggplot")
})

test_that("ggpie validates its inputs", {
  df <- data.frame(a = 1, b = 2)
  expect_error(ggpie(df, "missing", "b"))
  expect_error(ggpie(df, "a", "missing"))
  expect_error(ggpie(list(a = 1, b = 2), "a", "b"))
})

test_that("ggpie .legend and .label toggles work", {
  df <- data.frame(x = c("a", "b"), y = c(1, 2))
  p1 <- ggpie(df, "x", "y", .label = FALSE, .legend = TRUE)
  expect_s3_class(p1, "ggplot")
})

test_that("ggpie rejects invalid .offset values", {
  df <- data.frame(x = c("a", "b"), y = c(1, 2))
  expect_error(ggpie(df, "x", "y", .offset = 0))      # must be > 0
  expect_error(ggpie(df, "x", "y", .offset = -1))     # must be > 0
  expect_error(ggpie(df, "x", "y", .offset = "1"))    # must be numeric
  expect_error(ggpie(df, "x", "y", .offset = c(1, 2)))# must be length 1
})

test_that("ggpie rejects non-numeric .y column", {
  df <- data.frame(x = c("a", "b"), y = c("1", "2"))
  expect_error(ggpie(df, "x", "y"))
})

#' Continuous Scales for DSIR Bar / Column Charts
#'
#' Thin wrappers around [ggplot2::scale_y_continuous()] and
#' [ggplot2::scale_x_continuous()] that remove the default lower expansion
#' so that columns sit flush with the axis — the convention for WHO and
#' most publication-style bar charts. The upper expansion is preserved at
#' 5% so the tallest column has breathing room above (or to the right of) it.
#'
#' Use `scale_y_dsi_col()` for vertical bars (`geom_col()` / `geom_bar()`)
#' and `scale_x_dsi_col()` when bars are horizontal (via `coord_flip()` or
#' `geom_col(orientation = "y")`).
#'
#' Pass any other `scale_*_continuous()` argument
#' (`labels`, `breaks`, `limits`, ...) through `...`.
#'
#' @param ... Arguments forwarded to the underlying
#'   [ggplot2::scale_y_continuous()] / [ggplot2::scale_x_continuous()].
#'
#' @return A `ggplot2` Scale object, to be added to a plot with `+`.
#'
#' @examples
#' library(ggplot2)
#'
#' # Vertical bars
#' ggplot(mtcars, aes(factor(cyl))) +
#'   geom_bar(fill = "#0093D5") +
#'   scale_y_dsi_col() +
#'   theme_dsi()
#'
#' @name scale_dsi_col
NULL

#' @rdname scale_dsi_col
#' @export
scale_y_dsi_col <- function(...) {
  ggplot2::scale_y_continuous(
    expand = ggplot2::expansion(mult = c(0, 0.05)),
    ...
  )
}

#' @rdname scale_dsi_col
#' @export
scale_x_dsi_col <- function(...) {
  ggplot2::scale_x_continuous(
    expand = ggplot2::expansion(mult = c(0, 0.05)),
    ...
  )
}

test_that("scale_y_dsi_col returns a continuous Y scale", {
  s <- scale_y_dsi_col()
  expect_s3_class(s, "ScaleContinuous")
  expect_true("y" %in% s$aesthetics)
})

test_that("scale_x_dsi_col returns a continuous X scale", {
  s <- scale_x_dsi_col()
  expect_s3_class(s, "ScaleContinuous")
  expect_true("x" %in% s$aesthetics)
})

test_that("scale_*_dsi_col use c(0, 0.05) expansion to flush columns to the axis", {
  # The whole point: lower-side multiplicative expansion is 0.
  s_y <- scale_y_dsi_col()
  s_x <- scale_x_dsi_col()
  expect_equal(s_y$expand, ggplot2::expansion(mult = c(0, 0.05)))
  expect_equal(s_x$expand, ggplot2::expansion(mult = c(0, 0.05)))
})

test_that("scale_*_dsi_col forward extra arguments to scale_*_continuous", {
  # Passing `limits` should be forwarded so users can override the
  # default scale range without losing the flush expansion.
  s <- scale_y_dsi_col(limits = c(0, 100))
  expect_equal(s$limits, c(0, 100))
})

#' Normalise an SDG `area` argument
#'
#' Detects whether `area` contains ISO3 codes or UN M49 codes and
#' returns M49 in either case. Rejects mixed formats and inputs that
#' are neither ISO3 nor M49. Unknown ISO3 codes are dropped with a
#' warning.
#'
#' @noRd
.resolve_area <- function(area) {
  if (is.null(area)) return(NULL)
  if (!is.character(area)) {
    cli::cli_abort(
      "{.arg area} must be a character vector, not {.cls {class(area)}}."
    )
  }
  if (length(area) == 0L) return(area)

  is_iso3 <- grepl("^[A-Za-z]{3}$", area)
  is_m49  <- grepl("^[0-9]+$", area)

  if (!all(is_iso3 | is_m49)) {
    bad <- area[!is_iso3 & !is_m49]
    cli::cli_abort(c(
      "Invalid {.arg area} value{?s}: {.val {bad}}",
      "i" = "Use ISO3 codes (e.g. {.val PHL}) or UN M49 numeric codes (e.g. {.val 608})."
    ))
  }

  if (any(is_iso3) && any(is_m49)) {
    cli::cli_abort(c(
      "{.arg area} mixes ISO3 and M49 codes.",
      "i" = "Use one format consistently within a single call."
    ))
  }

  if (all(is_m49)) return(area)

  converted <- iso3_to_m49(area)
  bad <- area[is.na(converted)]
  if (length(bad) > 0L) {
    cli::cli_warn(
      "{length(bad)} ISO3 code{?s} did not match a WHO Member State and {?was/were} dropped: {.val {bad}}"
    )
    converted <- converted[!is.na(converted)]
  }
  if (length(converted) == 0L) {
    cli::cli_abort("No valid area codes remain after conversion.")
  }
  converted
}


#' List SDG Goals
#'
#' Fetches the list of Sustainable Development Goals from the UN
#' SDG API.
#'
#' @param include_children Logical. Include targets and indicators
#'   nested under each goal? Default `FALSE`.
#'
#' @return A list (or [tibble][tibble::tibble]) of SDG goals, or
#'   `NULL` when the service is unreachable.
#' @seealso [sdg_targets()], [sdg_indicators()], [sdg_data()].
#' @export
#'
#' @examples
#' \donttest{
#' sdg_goals()
#' sdg_goals(include_children = TRUE)
#' }
sdg_goals <- function(include_children = FALSE) {
  url <- paste0(
    "https://unstats.un.org/sdgs/UNSDGAPIV5/v1/sdg/Goal/List",
    "?includechildren=", tolower(as.character(include_children))
  )
  .sdg_get(url)
}


#' List SDG Targets
#'
#' Fetches the list of SDG targets from the UN SDG API.
#'
#' @param include_children Logical. Include indicators nested under
#'   each target? Default `FALSE`.
#'
#' @return A list (or [tibble][tibble::tibble]) of SDG targets, or
#'   `NULL` when the service is unreachable.
#' @seealso [sdg_goals()], [sdg_indicators()].
#' @export
#'
#' @examples
#' \donttest{
#' sdg_targets()
#' }
sdg_targets <- function(include_children = FALSE) {
  url <- paste0(
    "https://unstats.un.org/sdgs/UNSDGAPIV5/v1/sdg/Target/List",
    "?includechildren=", tolower(as.character(include_children))
  )
  .sdg_get(url)
}


#' List SDG Indicators
#'
#' Fetches the list of SDG indicators from the UN SDG API, with
#' optional keyword filtering on the indicator description.
#'
#' @param search Optional character. Search keywords matched against
#'   the `description` column (case-insensitive). All terms must
#'   match (AND semantics). Accepts either:
#'   * a single string, which is split on whitespace into terms
#'     (e.g. `"mortality cancer"` keeps rows whose description
#'     contains both "mortality" and "cancer"), or
#'   * a character vector, whose elements are used as terms verbatim
#'     (so a term may itself contain whitespace, e.g.
#'     `c("mortality rate", "attributed")`).
#'
#'   The filter is applied client-side using
#'   [grepl()] with `fixed = TRUE` because the UN SDG
#'   `/Indicator/List` endpoint is not OData and exposes no
#'   server-side search parameter; the full list is small
#'   (~250 rows) so this is cheap.
#'
#' @return A list (or [tibble][tibble::tibble]) of SDG indicators,
#'   or `NULL` when the service is unreachable. When `search` matches
#'   no rows, an empty tibble with the same columns as the unfiltered
#'   response is returned.
#' @seealso [sdg_targets()], [sdg_data()].
#' @export
#'
#' @examples
#' \donttest{
#' # Full list
#' sdg_indicators()
#'
#' # Single keyword
#' sdg_indicators("mortality")
#'
#' # Multi-keyword — AND semantics
#' sdg_indicators("mortality cancer")
#' sdg_indicators(c("maternal", "mortality"))
#' }
sdg_indicators <- function(search = NULL) {
  terms <- if (is.null(search)) NULL else .sdg_indicators_parse_terms(search)

  res <- .sdg_get(
    "https://unstats.un.org/sdgs/UNSDGAPIV5/v1/sdg/Indicator/List"
  )
  if (is.null(terms) || is.null(res)) return(res)
  if (!"description" %in% names(res)) return(res[integer(0), , drop = FALSE])

  haystack <- tolower(res$description)
  keep <- rep(TRUE, length(haystack))
  for (term in tolower(terms)) {
    keep <- keep & grepl(term, haystack, fixed = TRUE)
  }
  res[which(keep), , drop = FALSE]
}


#' @noRd
.sdg_indicators_parse_terms <- function(search) {
  stopifnot(
    is.character(search),
    length(search) >= 1L,
    !anyNA(search),
    all(nzchar(search))
  )

  terms <- if (length(search) == 1L) {
    strsplit(search, "\\s+")[[1]]
  } else {
    search
  }
  terms <- terms[nzchar(terms)]
  stopifnot(length(terms) >= 1L)
  terms
}


#' List SDG Geographic Areas
#'
#' Fetches the list of geographic areas available from the UN SDG
#' database.
#'
#' @return A [tibble][tibble::tibble] with area codes and names, or
#'   `NULL` when the service is unreachable.
#' @seealso [sdg_data()].
#' @export
#'
#' @examples
#' \donttest{
#' sdg_areas()
#' }
sdg_areas <- function() {
  .sdg_get("https://unstats.un.org/sdgs/UNSDGAPIV5/v1/sdg/GeoArea/List")
}


#' Fetch SDG Data
#'
#' Retrieves data for one or more SDG indicators from the UN SDG
#' API, with optional filters by area and year.
#'
#' @param indicator Character vector of indicator codes
#'   (e.g. `"1.1.1"`). Use [sdg_indicators()] to find codes.
#' @param area Character vector of country/area codes. Accepts either
#'   ISO3 codes (e.g. `c("PHL", "FRA")`) — converted automatically via
#'   [iso3_to_m49()] — or UN M49 numeric codes (e.g. `c("608", "250")`)
#'   as returned by [sdg_areas()]. Do not mix the two formats in a
#'   single call. Default `NULL` returns all areas.
#' @param year_from Numeric. Start year filter (inclusive).
#'   Default `NULL`.
#' @param year_to Numeric. End year filter (inclusive).
#'   Default `NULL`.
#' @param page_size Integer. Number of records per page.
#'   Default `1000`, maximum `10000`.
#'
#' @return A [tibble][tibble::tibble] of indicator observations, or
#'   an empty tibble when the service is unreachable or there are no
#'   matching rows.
#' @seealso [sdg_indicators()], [sdg_areas()], [iso3_to_m49()].
#' @export
#'
#' @examples
#' \donttest{
#' # One indicator, one country — the typical entry point
#' sdg_data("1.1.1", area = "PHL")
#'
#' # Specific area and year range (M49 code)
#' sdg_data("3.2.1", area = "156", year_from = 2015, year_to = 2023)
#'
#' # ISO3 codes work directly — DSIR's regional vectors can be passed in
#' sdg_data("3.4.1", area = c("PHL", "FRA", "JPN"))
#' }
sdg_data <- function(indicator, area = NULL,
                     year_from = NULL, year_to = NULL,
                     page_size = 1000L) {
  stopifnot(is.character(indicator), length(indicator) >= 1L)
  area <- .resolve_area(area)

  # The SDG API expects multi-value parameters as repeated keys
  # (`indicator=3.4.1&indicator=3.4.2`) rather than comma-separated.
  # A comma-separated value is silently dropped and the filter is
  # ignored, returning all rows for the indicator.
  parts <- c(
    paste0("indicator=", indicator),
    paste0("pageSize=", page_size)
  )
  if (!is.null(area))      parts <- c(parts, paste0("areaCode=", area))

  # Year filtering (`year_from` / `year_to`) is applied client-side,
  # not as `timePeriodStart` / `timePeriodEnd` server-side parameters.
  # The UN SDG API returns HTTP 500 and stalls (~30x slower) when
  # those parameters are sent for some indicator/area combinations.

  base_url <- "https://unstats.un.org/sdgs/UNSDGAPIV5/v1/sdg/Indicator/Data"
  url <- paste0(base_url, "?", paste(parts, collapse = "&"))

  all_data <- list()
  page <- 1

  repeat {
    page_url <- paste0(url, "&page=", page)
    body <- .sdg_get(page_url)
    if (is.null(body)) return(tibble::tibble())
    if (is.null(body$data) || length(body$data) == 0) break

    all_data <- c(all_data, list(tibble::as_tibble(body$data)))

    total_pages <- body$totalPages %||% 1
    if (page >= total_pages) break
    page <- page + 1
  }

  if (length(all_data) == 0) {
    cli::cli_warn("No data returned for indicator {.val {indicator}}.")
    return(tibble::tibble())
  }

  # `make.row.names = FALSE` is essential: with the default TRUE, rbind
  # tries to combine each tibble's "1".."pageSize" row names, which
  # collide across pages and trip `duplicate 'row.names' are not allowed`
  # for indicators that span more than one full page.
  out <- do.call(rbind, c(all_data, list(make.row.names = FALSE)))
  out <- tibble::as_tibble(out)

  # Client-side year filter — workaround for UN SDG API bug where
  # timePeriodStart / timePeriodEnd cause HTTP 500 server-side.
  if (!is.null(year_from) || !is.null(year_to)) {
    if (!"timePeriodStart" %in% names(out)) {
      cli::cli_warn(c(
        "Cannot apply year filter: {.field timePeriodStart} column missing.",
        "i" = "Returning unfiltered data."
      ))
    } else {
      yr <- suppressWarnings(as.integer(out$timePeriodStart))
      keep <- !is.na(yr)
      if (!is.null(year_from)) keep <- keep & yr >= as.integer(year_from)
      if (!is.null(year_to))   keep <- keep & yr <= as.integer(year_to)
      out <- out[keep, , drop = FALSE]
    }
  }

  out
}


#' @noRd
.sdg_resolve_location_name <- function(iso3, raw_geo_area_name) {
  if (length(iso3) == 0L) return(character(0))

  out <- rep(NA_character_, length(iso3))

  iso3_match <- match(iso3, who_countries$iso3)
  has_iso3 <- !is.na(iso3_match)
  out[has_iso3] <- who_countries$name_short[iso3_match[has_iso3]]

  if (!is.null(raw_geo_area_name)) {
    still_na <- is.na(out)
    out[still_na] <- as.character(raw_geo_area_name[still_na])
  }

  out
}


#' Tidy an SDG Data Frame
#'
#' Selects, renames, and type-casts the most useful columns from an
#' SDG observation table returned by [sdg_data()], producing a compact
#' tibble in the **unified DSIR cleaned-indicator schema** — the same
#' schema produced by [gho_clean()], so the two outputs can be combined
#' directly with [bind_indicators()].
#'
#' The mapping (SDG source → unified column) is:
#' * `indicator` (list-column, flattened) → `id` (e.g. `"3.4.1"`)
#' * `seriesDescription`                  → `indicator` (human-readable
#'   label; `NA` if the API response does not include it)
#' * `geoAreaCode`                        → `location` (UN M49 numeric,
#'   as character); also `iso3` via [m49_to_iso3()] for WHO Member
#'   States — region / world aggregates and non-Member areas get
#'   `iso3 = NA`
#' * `location_name` is resolved by looking up `iso3` against
#'   [`who_countries`] (so a WHO Member State has the same
#'   `location_name` here and in [gho_clean()] output), with a
#'   fallback to the SDG API's raw `geoAreaName` for non-Member-State
#'   rows (e.g. regional / world aggregates)
#' * `timePeriodStart`                    → `year` (integer)
#' * `value`                              → `value` (character; raw)
#'   and `value_num` (numeric; `NA` for non-numeric entries like
#'   `"<0.1"` or aggregate notes)
#' * `lowerBound`, `upperBound`           → `low`, `high` (numeric)
#' * `series`                             → `series`
#'
#' Three columns are always present but never populated for SDG
#' output: `dim1`, `dim2`, `dim3` (GHO-only concepts).
#'
#' @param df A data frame returned by [sdg_data()].
#'
#' @return A [tibble][tibble::tibble] with 15 columns: `source` (always
#'   `"sdg"`), `id`, `indicator`, `location`, `iso3`, `location_name`,
#'   `year`, `value`, `value_num`, `low`, `high`, `series`, `dim1`
#'   (`NA`), `dim2` (`NA`), `dim3` (`NA`). Sorted by `location` then
#'   `year`. Empty input returns an empty tibble with the same columns
#'   and types.
#' @seealso [sdg_data()], [gho_clean()], [bind_indicators()],
#'   [m49_to_iso3()].
#' @export
#'
#' @examples
#' \donttest{
#' sdg_data("3.2.1", area = "156", year_from = 2015) |>
#'   sdg_clean()
#' }
sdg_clean <- function(df) {
  if (!is.data.frame(df)) {
    cli::cli_abort("{.arg df} must be a data frame.")
  }

  n <- nrow(df)
  if (n == 0L) return(.dsi_empty_clean())

  flatten_chr <- function(src) {
    if (!src %in% names(df)) return(.fill_na(n, "chr"))
    x <- df[[src]]
    if (is.list(x)) {
      vapply(x, function(v) {
        if (length(v) == 0L) NA_character_ else as.character(v[[1]])
      }, character(1))
    } else {
      as.character(x)
    }
  }
  pick_chr <- function(src) {
    if (src %in% names(df)) as.character(df[[src]]) else .fill_na(n, "chr")
  }
  pick_num <- function(src) {
    if (src %in% names(df)) {
      suppressWarnings(as.numeric(df[[src]]))
    } else {
      .fill_na(n, "num")
    }
  }
  pick_int <- function(src) {
    if (src %in% names(df)) {
      suppressWarnings(as.integer(df[[src]]))
    } else {
      .fill_na(n, "int")
    }
  }

  location <- pick_chr("geoAreaCode")
  iso3 <- m49_to_iso3(location)
  value_chr <- pick_chr("value")

  out <- tibble::tibble(
    source        = rep("sdg", n),
    id            = flatten_chr("indicator"),
    indicator     = pick_chr("seriesDescription"),
    location      = location,
    iso3          = iso3,
    location_name = .sdg_resolve_location_name(
      iso3,
      if ("geoAreaName" %in% names(df)) df[["geoAreaName"]] else NULL
    ),
    year          = pick_int("timePeriodStart"),
    value         = value_chr,
    value_num     = suppressWarnings(as.numeric(value_chr)),
    low           = pick_num("lowerBound"),
    high          = pick_num("upperBound"),
    series        = pick_chr("series"),
    dim1          = .fill_na(n, "chr"),
    dim2          = .fill_na(n, "chr"),
    dim3          = .fill_na(n, "chr")
  )

  out[order(out$location, out$year), , drop = FALSE]
}


#' @noRd
.sdg_get <- function(url) {
  cli::cli_inform("Fetching: {.url {url}}")

  resp <- tryCatch(
    httr2::request(url) |>
      httr2::req_headers(Accept = "application/json") |>
      httr2::req_timeout(20) |>
      httr2::req_retry(
        max_tries = 3,
        backoff   = ~ min(2 ^ .x, 30)
      ) |>
      httr2::req_perform(),
    error = function(e) {
      # Reference the message via a variable so cli does not glue-interpret
      # any literal braces the error message may carry (see body parse below).
      msg <- conditionMessage(e)
      cli::cli_warn(c(
        "SDG request failed.",
        "i" = "URL: {.url {url}}",
        "x" = "{msg}"
      ))
      NULL
    }
  )
  if (is.null(resp)) return(NULL)

  # Body parse must also fail soft: the UN SDG endpoint occasionally
  # returns a truncated response body (premature EOF), which jsonlite
  # surfaces as a parse error. Without this tryCatch the error would
  # propagate and break R CMD check examples (CRAN-blocking).
  out <- tryCatch(
    httr2::resp_body_json(resp, simplifyVector = TRUE),
    error = function(e) {
      # Reference the message via a variable: a jsonlite parse error
      # carries literal `{`/`}` from the offending JSON, which cli would
      # otherwise try to interpret as glue expressions and re-error.
      msg <- conditionMessage(e)
      cli::cli_warn(c(
        "SDG response could not be parsed as JSON.",
        "i" = "URL: {.url {url}}",
        "x" = "{msg}"
      ))
      NULL
    }
  )
  if (is.null(out)) return(NULL)
  if (is.data.frame(out)) tibble::as_tibble(out) else out
}

# Tests for bind_indicators(). Inputs are constructed via gho_clean() /
# sdg_clean() on small fake frames so the tests run offline.

make_gho <- function() {
  gho_clean(tibble::tibble(
    IndicatorCode = "WHOSIS_000001",
    IndicatorName = "Life expectancy",
    SpatialDim    = "FRA",
    TimeDim       = 2019L,
    Value         = "82.5",
    NumericValue  = 82.5
  ))
}

make_sdg <- function() {
  sdg_clean(tibble::tibble(
    indicator       = list("3.4.1"),
    series          = "SH_DTH_NCD",
    seriesDescription = "NCD mortality",
    geoAreaCode     = "608",
    geoAreaName     = "Philippines",
    timePeriodStart = 2019,
    value           = "17.3"
  ))
}

unified_cols <- c(
  "source", "id", "indicator", "location", "iso3", "location_name",
  "year", "value", "value_num", "low", "high", "series",
  "dim1", "dim2", "dim3"
)

test_that("bind_indicators returns the unified 15-column schema", {
  out <- bind_indicators(make_gho(), make_sdg())
  expect_s3_class(out, "tbl_df")
  expect_named(out, unified_cols)
  expect_equal(nrow(out), 2L)
})

test_that("bind_indicators preserves source column", {
  out <- bind_indicators(make_gho(), make_sdg())
  expect_setequal(out$source, c("gho", "sdg"))
})

test_that("bind_indicators preserves input order", {
  out <- bind_indicators(make_gho(), make_sdg())
  expect_equal(out$source, c("gho", "sdg"))
  out2 <- bind_indicators(make_sdg(), make_gho())
  expect_equal(out2$source, c("sdg", "gho"))
})

test_that("bind_indicators accepts a single input", {
  out <- bind_indicators(make_gho())
  expect_equal(nrow(out), 1L)
  expect_equal(out$source, "gho")
})

test_that("bind_indicators ignores NULL arguments", {
  out <- bind_indicators(make_gho(), NULL, make_sdg(), NULL)
  expect_equal(nrow(out), 2L)
  expect_equal(out$source, c("gho", "sdg"))
})

test_that("bind_indicators returns empty schema on no inputs", {
  out <- bind_indicators()
  expect_named(out, unified_cols)
  expect_equal(nrow(out), 0L)
  expect_type(out$year,      "integer")
  expect_type(out$value_num, "double")

  out_null <- bind_indicators(NULL, NULL)
  expect_equal(nrow(out_null), 0L)
})

test_that("bind_indicators rejects non-data-frame input", {
  expect_error(bind_indicators(make_gho(), "not a df"),
               "must be a data frame")
  expect_error(bind_indicators(list(a = 1)),
               "must be a data frame")
})

test_that("bind_indicators rejects unclean input (missing schema cols)", {
  bad <- tibble::tibble(foo = 1, bar = 2)
  expect_error(bind_indicators(make_gho(), bad),
               "missing required column")
})

test_that("bind_indicators tolerates column re-ordering in inputs", {
  reordered <- make_gho()[, rev(names(make_gho()))]
  out <- bind_indicators(reordered, make_sdg())
  expect_named(out, unified_cols)
  expect_equal(nrow(out), 2L)
})

test_that("sdg_indicators(NULL) returns the full unfiltered list", {
  skip_on_cran()
  skip_if_offline()

  out <- sdg_indicators()
  expect_s3_class(out, "tbl_df")
  expect_true("description" %in% names(out))
  expect_gt(nrow(out), 200L)
})

test_that("sdg_indicators filters on a single term (case-insensitive substring)", {
  skip_on_cran()
  skip_if_offline()

  out <- sdg_indicators("mortality")
  expect_s3_class(out, "tbl_df")
  expect_gt(nrow(out), 0L)
  expect_true(all(grepl("mortality", tolower(out$description), fixed = TRUE)))

  # Same result regardless of input case
  out_upper <- sdg_indicators("MORTALITY")
  expect_identical(out, out_upper)
})

test_that("sdg_indicators applies AND semantics on a whitespace-split string", {
  skip_on_cran()
  skip_if_offline()

  out <- sdg_indicators("mortality cancer")
  expect_s3_class(out, "tbl_df")
  expect_gt(nrow(out), 0L)
  expect_true(all(grepl("mortality", tolower(out$description), fixed = TRUE)))
  expect_true(all(grepl("cancer",    tolower(out$description), fixed = TRUE)))

  # Stricter than the single-term result
  single <- sdg_indicators("mortality")
  expect_lte(nrow(out), nrow(single))
})

test_that("sdg_indicators accepts a character vector as terms (AND)", {
  skip_on_cran()
  skip_if_offline()

  out <- sdg_indicators(c("maternal", "mortality"))
  expect_s3_class(out, "tbl_df")
  expect_gt(nrow(out), 0L)
  expect_true(all(grepl("maternal",
                        tolower(out$description), fixed = TRUE)))
  expect_true(all(grepl("mortality",
                        tolower(out$description), fixed = TRUE)))
})

test_that("sdg_indicators treats whitespace inside a vector element as part of the term", {
  skip_on_cran()
  skip_if_offline()

  # An element with internal whitespace is matched verbatim — not split
  # into separate terms — so this is stricter than the single-term call.
  out <- sdg_indicators(c("mortality rate", "attributed"))
  expect_s3_class(out, "tbl_df")
  expect_gt(nrow(out), 0L)
  expect_true(all(grepl("mortality rate",
                        tolower(out$description), fixed = TRUE)))
  expect_true(all(grepl("attributed",
                        tolower(out$description), fixed = TRUE)))
})

test_that("sdg_indicators returns an empty tibble (correct shape) on no match", {
  skip_on_cran()
  skip_if_offline()

  full <- sdg_indicators()
  out  <- sdg_indicators("zzzzzzzzz-no-such-thing")
  expect_s3_class(out, "tbl_df")
  expect_equal(nrow(out), 0L)
  expect_identical(names(out), names(full))
})

test_that("sdg_indicators handles search terms containing a single quote", {
  skip_on_cran()
  skip_if_offline()

  # Should not error — filter is client-side, fixed = TRUE
  out <- sdg_indicators("women's")
  expect_s3_class(out, "tbl_df")
})

test_that("sdg_indicators validates the search argument", {
  expect_error(sdg_indicators(search = 1L))
  expect_error(sdg_indicators(search = NA_character_))
  expect_error(sdg_indicators(search = ""))
  expect_error(sdg_indicators(search = character()))
})

# Offline regression tests for .sdg_get(), sdg_data(), and the SDG list
# endpoints. Uses httr2 mocked responses so these run on CRAN and on
# machines without internet. Each test guards an explicit fix described
# in NEWS.md.

skip_if_not_installed("httptest2")  # Suggests dependency

mock_json <- function(body, status = 200L) {
  list(httr2::response(
    status_code = status,
    headers     = list(`content-type` = "application/json"),
    body        = charToRaw(body)
  ))
}

test_that(".sdg_get returns NULL with a warning on HTTP failure", {
  httr2::with_mocked_responses(
    mock_json('{"error":"boom"}', status = 404L),
    {
      expect_warning(
        out <- DSIR:::.sdg_get("https://example.test/v1/sdg/Goal/List"),
        "SDG request failed"
      )
      expect_null(out)
    }
  )
})

test_that("sdg_goals returns NULL when the service is unreachable", {
  httr2::with_mocked_responses(
    mock_json('{"error":"boom"}', status = 404L),
    {
      out <- suppressWarnings(sdg_goals())
      expect_null(out)
    }
  )
})

test_that("sdg_data assembles rows across multiple pages without errors", {
  # NEWS 0.7.0: per-page row.names previously clashed when do.call(rbind)
  # ran across pages ("duplicate 'row.names' are not allowed"). To
  # actually trip the collision in a regression test, each mock page
  # must contain >1 row so that auto-generated row names overlap
  # ("1","2" on both pages).
  make_row <- function(year) {
    paste0(
      '{"goal":"3","target":"3.4","indicator":["3.4.1"],',
      '"series":"SH_DTH_NCD","geoAreaCode":"608","geoAreaName":"PHL",',
      '"timePeriodStart":', year, ',"value":"17.3"}'
    )
  }
  page1 <- paste0('{"data":[', paste(make_row(2017), make_row(2018), sep = ","),
                  '],"totalPages":2}')
  page2 <- paste0('{"data":[', paste(make_row(2019), make_row(2020), sep = ","),
                  '],"totalPages":2}')
  httr2::with_mocked_responses(
    c(mock_json(page1), mock_json(page2)),
    {
      out <- sdg_data("3.4.1", area = "608")
      expect_s3_class(out, "tbl_df")
      expect_equal(nrow(out), 4L)
      expect_setequal(out$timePeriodStart, c(2017, 2018, 2019, 2020))
    }
  )
})

# Helper for the URL-capturing tests below. `mock` is a function rather
# than a list because we need to inspect each outgoing request before
# returning the canned response.
capture_url_mock <- function(body) {
  captured <- character()
  fn <- function(req) {
    captured[[length(captured) + 1L]] <<- req$url
    httr2::response(
      status_code = 200L,
      headers     = list(`content-type` = "application/json"),
      body        = charToRaw(body)
    )
  }
  list(fn = fn, urls = function() captured)
}

test_that("sdg_data sends multi-value indicator as repeated keys, not comma-joined", {
  # NEWS 0.6.0: previously joined as `indicator=A,B`, which the SDG API
  # silently drops. Must serialise as repeated `indicator=A&indicator=B`.
  m <- capture_url_mock('{"data": [], "totalPages": 1}')
  httr2::with_mocked_responses(m$fn, {
    suppressWarnings(sdg_data(c("3.4.1", "3.4.2"), area = "608"))
  })
  urls <- m$urls()
  expect_gte(length(urls), 1L)
  expect_match(urls[1], "indicator=3.4.1", fixed = TRUE)
  expect_match(urls[1], "indicator=3.4.2", fixed = TRUE)
  expect_false(grepl("indicator=3.4.1,3.4.2", urls[1], fixed = TRUE))
})

test_that("sdg_data sends multi-value area as repeated areaCode keys", {
  m <- capture_url_mock('{"data": [], "totalPages": 1}')
  httr2::with_mocked_responses(m$fn, {
    suppressWarnings(sdg_data("3.4.1", area = c("608", "076")))
  })
  urls <- m$urls()
  expect_gte(length(urls), 1L)
  expect_match(urls[1], "areaCode=608", fixed = TRUE)
  expect_match(urls[1], "areaCode=076", fixed = TRUE)
  expect_false(grepl("areaCode=608,076", urls[1], fixed = TRUE))
})

test_that("sdg_data applies year filter client-side", {
  # NEWS 0.6.0: timePeriodStart / timePeriodEnd cause HTTP 500 on the
  # real API for some combinations, so year filtering must run on the
  # client. Verify (a) no year params in the outgoing URL, (b) rows
  # outside the requested range are stripped from the result.
  page <- paste0(
    '{"data":[',
    '{"indicator":["3.4.1"],"geoAreaCode":"608","timePeriodStart":2005,"value":"1"},',
    '{"indicator":["3.4.1"],"geoAreaCode":"608","timePeriodStart":2015,"value":"2"},',
    '{"indicator":["3.4.1"],"geoAreaCode":"608","timePeriodStart":2020,"value":"3"}',
    '],"totalPages":1}'
  )
  m <- capture_url_mock(page)
  httr2::with_mocked_responses(m$fn, {
    out <<- sdg_data("3.4.1", area = "608", year_from = 2010, year_to = 2017)
  })
  urls <- m$urls()
  expect_gte(length(urls), 1L)
  expect_false(grepl("timePeriodStart", urls[1], fixed = TRUE))
  expect_false(grepl("timePeriodEnd",   urls[1], fixed = TRUE))
  expect_equal(nrow(out), 1L)
  expect_equal(out$timePeriodStart, 2015)
})

test_that(".sdg_get returns NULL with a warning on a malformed JSON body", {
  # NEWS 0.7.0: the UN SDG endpoint occasionally returns a truncated
  # response body; resp_body_json() surfaces that as a parse error.
  # The body-parse tryCatch must downgrade it to a warning + NULL so a
  # flaky endpoint cannot break an R CMD check example run.
  httr2::with_mocked_responses(
    mock_json('{"data": [{"series":"SH_DTH_NCD"'),  # truncated, unparseable
    {
      expect_warning(
        out <- DSIR:::.sdg_get("https://example.test/v1/sdg/Goal/List"),
        "could not be parsed as JSON"
      )
      expect_null(out)
    }
  )
})

# Most of these test the internal .resolve_area() directly so they run
# offline. One integration test at the bottom confirms end-to-end ISO3
# → SDG plumbing via the live API.

test_that("M49 codes pass through unchanged", {
  expect_equal(DSIR:::.resolve_area("608"), "608")
  expect_equal(DSIR:::.resolve_area(c("608", "250")), c("608", "250"))
  # Zero-padded forms also pass through (Brazil, Afghanistan, …)
  expect_equal(DSIR:::.resolve_area("076"), "076")
})

test_that("ISO3 codes are converted to M49", {
  result <- DSIR:::.resolve_area("PHL")
  expect_length(result, 1L)
  expect_match(result, "^[0-9]+$")
})

test_that("Case-insensitive ISO3 input works", {
  expect_equal(DSIR:::.resolve_area("phl"), DSIR:::.resolve_area("PHL"))
  expect_equal(DSIR:::.resolve_area("Fra"), DSIR:::.resolve_area("FRA"))
})

test_that("Mixed ISO3 and M49 errors", {
  expect_error(DSIR:::.resolve_area(c("PHL", "608")), "mixes")
})

test_that("Invalid format errors", {
  expect_error(DSIR:::.resolve_area("ZZ"),    "Invalid")
  expect_error(DSIR:::.resolve_area("PHLX"),  "Invalid")
  expect_error(DSIR:::.resolve_area("PHL-1"), "Invalid")
})

test_that("Unknown ISO3 warns and drops", {
  expect_warning(
    result <- DSIR:::.resolve_area(c("PHL", "XYZ")),
    "did not match"
  )
  expect_length(result, 1L)
})

test_that("All-unknown ISO3 errors", {
  expect_error(
    suppressWarnings(DSIR:::.resolve_area("XYZ")),
    "No valid area codes"
  )
})

test_that("NULL passes through", {
  expect_null(DSIR:::.resolve_area(NULL))
})

test_that("Empty character vector passes through", {
  result <- DSIR:::.resolve_area(character(0))
  expect_type(result, "character")
  expect_length(result, 0L)
})

test_that("Non-character input errors", {
  expect_error(DSIR:::.resolve_area(123),  "must be a character")
  expect_error(DSIR:::.resolve_area(TRUE), "must be a character")
})

test_that("sdg_data accepts ISO3 directly (network test)", {
  skip_on_cran()
  skip_if_offline()

  result <- sdg_data("3.4.1", area = "PHL")
  expect_s3_class(result, "tbl_df")
  expect_gt(nrow(result), 0L)
  # The server-side area code should be the M49 equivalent
  expect_true(all(result$geoAreaCode == iso3_to_m49("PHL")))
})

#' Look Up the WHO Region for ISO3 Codes
#'
#' Maps ISO 3166-1 alpha-3 country codes to WHO region codes using
#' the [`who_countries`] dataset shipped with DSIR. Stays in sync with
#' WHO governance changes reflected in DSIR — for example, Indonesia's
#' reassignment from SEAR to WPR following EB156 (May 2025).
#'
#' Codes that do not correspond to a WHO Member State return `NA`.
#' This includes Associate Members (Puerto Rico, Tokelau) and other
#' non-Member areas that some indicator data sets cover.
#'
#' @param iso3 Character vector of ISO3 codes. Case-sensitive
#'   (uppercase, as in [`who_countries`]).
#' @param long Logical. If `TRUE`, return long-form region names
#'   (e.g. `"Western Pacific"`). If `FALSE` (default), return the
#'   short codes used elsewhere in DSIR: `"AFR"`, `"AMR"`, `"SEAR"`,
#'   `"EUR"`, `"EMR"`, `"WPR"`.
#'
#' @return A character vector the same length as `iso3`.
#' @seealso [`who_countries`], [`wpro_cty`].
#' @export
#'
#' @examples
#' iso3_to_region(c("PHL", "FRA", "USA", "COK"))
#' # "WPR" "EUR" "AMR" "WPR"
#'
#' iso3_to_region(c("IDN", "JPN"), long = TRUE)
#' # "Western Pacific" "Western Pacific"  (Indonesia in WPR since May 2025)
#'
#' # Non-Member areas return NA
#' iso3_to_region(c("PRI", "TKL", "PHL"))
#' # NA NA "WPR"
iso3_to_region <- function(iso3, long = FALSE) {
  if (!is.character(iso3)) {
    cli::cli_abort("{.arg iso3} must be a character vector, not {.cls {class(iso3)}}.")
  }
  if (!isTRUE(long) && !isFALSE(long)) {
    cli::cli_abort("{.arg long} must be a single logical value.")
  }

  m <- match(iso3, who_countries$iso3)
  codes <- who_countries$who_region[m]

  if (!long) return(codes)

  long_map <- c(
    AFR  = "Africa",
    AMR  = "Americas",
    SEAR = "South-East Asia",
    EUR  = "Europe",
    EMR  = "Eastern Mediterranean",
    WPR  = "Western Pacific"
  )
  unname(long_map[codes])
}

test_that("iso3_to_region returns short codes", {
  expect_equal(iso3_to_region("PHL"), "WPR")
  expect_equal(iso3_to_region("IDN"), "WPR")  # post-EB156, May 2025
  expect_equal(iso3_to_region(c("FRA", "USA", "JPN")), c("EUR", "AMR", "WPR"))
})

test_that("iso3_to_region returns long names for all six regions", {
  # One country per region — ensures the short → long map has full coverage.
  expect_equal(iso3_to_region("ZAF", long = TRUE), "Africa")
  expect_equal(iso3_to_region("USA", long = TRUE), "Americas")
  expect_equal(iso3_to_region("IND", long = TRUE), "South-East Asia")
  expect_equal(iso3_to_region("FRA", long = TRUE), "Europe")
  expect_equal(iso3_to_region("EGY", long = TRUE), "Eastern Mediterranean")
  expect_equal(iso3_to_region("PHL", long = TRUE), "Western Pacific")
})

test_that("iso3_to_region returns NA for non-Member codes", {
  expect_true(is.na(iso3_to_region("XXX")))
  expect_true(is.na(iso3_to_region("PRI")))  # Associate Member
})

test_that("iso3_to_region preserves vector length and order", {
  res <- iso3_to_region(c("PHL", "XXX", "FRA"))
  expect_length(res, 3)
  expect_equal(res, c("WPR", NA_character_, "EUR"))
})

test_that("iso3_to_region rejects bad input", {
  expect_error(iso3_to_region(123), "must be a character vector")
  expect_error(iso3_to_region("PHL", long = "yes"), "single logical")
})

#' Convert ISO3 Codes to UN M49 Numeric Codes
#'
#' Maps ISO 3166-1 alpha-3 country codes to UN M49 numeric area codes
#' using the [`who_countries`] dataset shipped with DSIR. Useful when
#' moving from data sources keyed by ISO3 (e.g. the WHO GHO API) to
#' sources keyed by M49 (e.g. the UN SDG API).
#'
#' Codes that do not correspond to a WHO Member State return `NA`.
#' This includes Associate Members (e.g. Puerto Rico) and other
#' non-Member areas that some indicator data sets cover.
#'
#' Most users will not need to call this function directly:
#' [sdg_data()] and [sdg_coverage()] accept ISO3 codes for their
#' `area` argument and convert internally. This helper is exported
#' for cases where you want to inspect or manipulate the conversion
#' yourself.
#'
#' @param iso3 Character vector of ISO3 codes. Case-insensitive;
#'   values are upper-cased before lookup.
#'
#' @return A character vector the same length as `iso3`, with M49
#'   codes in the same format as `who_countries$m49_code` (three-
#'   character zero-padded strings, e.g. `"076"`). Non-Member areas
#'   return `NA`.
#' @seealso [`who_countries`], [iso3_to_region()], [sdg_data()].
#' @export
#'
#' @examples
#' iso3_to_m49(c("PHL", "FRA", "JPN"))
#' # "608" "250" "392"
#'
#' # Case-insensitive
#' iso3_to_m49("phl")
#' # "608"
#'
#' # Non-Member areas return NA
#' iso3_to_m49(c("PRI", "PHL"))
#' # NA "608"
iso3_to_m49 <- function(iso3) {
  if (!is.character(iso3)) {
    cli::cli_abort(
      "{.arg iso3} must be a character vector, not {.cls {class(iso3)}}."
    )
  }
  m <- match(toupper(iso3), who_countries$iso3)
  who_countries$m49_code[m]
}

test_that("iso3_to_m49 maps known codes correctly", {
  result <- iso3_to_m49(c("PHL", "FRA", "JPN"))
  expect_equal(length(result), 3L)
  expect_false(any(is.na(result)))
  expect_type(result, "character")
})

test_that("iso3_to_m49 returns NA for non-Members and unknown codes", {
  expect_true(is.na(iso3_to_m49("PRI")))   # Associate Member
  expect_true(is.na(iso3_to_m49("XYZ")))   # never assigned
})

test_that("iso3_to_m49 is case-insensitive", {
  expect_equal(iso3_to_m49("phl"), iso3_to_m49("PHL"))
  expect_equal(iso3_to_m49("Fra"), iso3_to_m49("FRA"))
})

test_that("iso3_to_m49 preserves position (NAs in place, length matches)", {
  result <- iso3_to_m49(c("PHL", "XYZ", "FRA"))
  expect_equal(length(result), 3L)
  expect_false(is.na(result[1]))
  expect_true(is.na(result[2]))
  expect_false(is.na(result[3]))
})

test_that("iso3_to_m49 handles empty input", {
  result <- iso3_to_m49(character(0))
  expect_equal(length(result), 0L)
  expect_type(result, "character")
})

test_that("iso3_to_m49 errors on non-character input", {
  expect_error(iso3_to_m49(123),  "must be a character")
  expect_error(iso3_to_m49(NULL), "must be a character")
  expect_error(iso3_to_m49(TRUE), "must be a character")
})

#' Bind Cleaned Indicator Tibbles
#'
#' Combines two or more tibbles produced by [gho_clean()] or
#' [sdg_clean()] into a single tibble. Because both cleaners output the
#' same 15-column schema, the result is a uniform table that can be
#' filtered, joined, or visualised without source-specific code paths;
#' use the `source` column to tell GHO rows apart from SDG rows.
#'
#' Inputs do not need to be in any particular order. `NULL` inputs are
#' silently dropped, which makes it ergonomic to write code like
#' `bind_indicators(maybe_gho, maybe_sdg)` where some sources may not
#' have been fetched.
#'
#' @param ... Two or more tibbles returned by [gho_clean()] or
#'   [sdg_clean()] (or any data frame with the same column set). `NULL`
#'   arguments are dropped. Calling with no inputs — or only `NULL`
#'   inputs — returns the empty 15-column tibble.
#'
#' @return A single [tibble][tibble::tibble] with the unified cleaned-
#'   indicator schema (15 columns). Row order is `c(input_1, input_2,
#'   ...)`, preserving within-input order.
#' @seealso [gho_clean()], [sdg_clean()].
#' @export
#'
#' @examples
#' \donttest{
#' gho <- gho_data("NCDMORT3070", area = wpro_cty) |> gho_clean()
#' sdg <- sdg_data("3.4.1",        area = wpro_cty) |> sdg_clean()
#' bind_indicators(gho, sdg)
#' }
bind_indicators <- function(...) {
  dfs <- list(...)
  dfs <- dfs[!vapply(dfs, is.null, logical(1))]
  if (length(dfs) == 0L) return(.dsi_empty_clean())

  for (i in seq_along(dfs)) {
    if (!is.data.frame(dfs[[i]])) {
      cli::cli_abort(c(
        "Argument {i} must be a data frame, not {.cls {class(dfs[[i]])[1]}}.",
        "i" = "All inputs should come from {.fn gho_clean} or {.fn sdg_clean}."
      ))
    }
  }

  schema_cols <- names(.dsi_clean_schema())
  for (i in seq_along(dfs)) {
    missing_cols <- setdiff(schema_cols, names(dfs[[i]]))
    if (length(missing_cols) > 0L) {
      cli::cli_abort(c(
        "Argument {i} is missing required column{?s}: {.val {missing_cols}}.",
        "i" = "Did you forget to call {.fn gho_clean} or {.fn sdg_clean} first?"
      ))
    }
    # Re-order columns so rbind aligns by position without surprises.
    dfs[[i]] <- dfs[[i]][, schema_cols, drop = FALSE]
  }

  out <- do.call(rbind, c(dfs, list(make.row.names = FALSE)))
  tibble::as_tibble(out)
}

#' DSIR ggplot2 theme — WHO publication-style
#'
#' A clean, modern theme tuned for WHO and global-health publications.
#' Removes the panel border, draws light grid lines, and uses a muted
#' text colour so that the data — not the chart chrome — is the visual
#' focus. The `grid` argument controls which direction(s) the grid
#' lines run, so the theme works equally well for vertical bars,
#' horizontal bars (via `coord_flip()`), scatter plots, and line charts.
#'
#' @param base_size Base font size in points. Default `12`.
#' @param base_family Base font family. Default `""` (system default).
#'   Set to `"sans"`, `"Arial"`, `"Helvetica"`, or `"Calibri"` for a
#'   specific look. Empty default keeps the package portable on CRAN's
#'   Linux check machines, where Calibri is unavailable.
#' @param accent Accent colour used for axis lines and as a default for
#'   highlight elements. Default `"#0093D5"` (WHO blue). Pass any colour
#'   string.
#' @param grid_color Colour of the major grid. Default
#'   `"grey92"` — a very light grey, close to bbplot and OECD style.
#' @param grid Which direction(s) to draw major grid lines. One of
#'   `"both"` (default — draws both horizontal and vertical, works
#'   correctly under `coord_flip()`), `"y"` (only horizontal grid lines,
#'   the look used in DSIR <= 0.5.x), `"x"` (only vertical grid lines),
#'   or `"none"`.
#' @param legend_position Position of the legend. Default `"bottom"`.
#'   Pass `"none"`, `"top"`, `"right"`, or a numeric vector `c(x, y)`.
#'
#' @return A `ggplot2` theme object.
#'
#' @seealso [theme_dsi_facet()] for a sibling theme tuned for faceted plots.
#'
#' @examples
#' library(ggplot2)
#'
#' # Default — grid in both directions, works under coord_flip()
#' ggplot(mtcars, aes(wt, mpg, color = factor(cyl))) +
#'   geom_point(size = 3) +
#'   theme_dsi() +
#'   labs(title = "Fuel efficiency by weight",
#'        x = "Weight (1000 lbs)", y = "Miles per gallon",
#'        color = "Cylinders")
#'
#' # Minimal look — only horizontal grid lines
#' ggplot(mtcars, aes(wt, mpg)) +
#'   geom_point(size = 3, color = "#0093D5") +
#'   theme_dsi(grid = "y") +
#'   labs(x = "Weight (1000 lbs)", y = "Miles per gallon")
#'
#' @export
theme_dsi <- function(base_size = 12,
                      base_family = "",
                      accent = "#0093D5",
                      grid_color = "grey92",
                      grid = c("both", "x", "y", "none"),
                      legend_position = "bottom") {

  grid <- match.arg(grid)

  .theme_dsi_base(base_size       = base_size,
                  base_family     = base_family,
                  accent          = accent,
                  legend_position = legend_position) +
    ggplot2::theme(
      # ── axes (line, no border) ─────────────────────────────────
      panel.border = ggplot2::element_blank(),
      axis.line.x  = ggplot2::element_line(color = accent,
                                           linewidth = 0.3),
      axis.line.y  = ggplot2::element_line(color = accent,
                                           linewidth = 0.3),
      axis.ticks   = ggplot2::element_line(color = accent,
                                           linewidth = 0.3),

      # ── grid ───────────────────────────────────────────────────
      panel.grid.major.x = if (grid %in% c("both", "x")) {
        ggplot2::element_line(color = grid_color, linewidth = 0.3)
      } else {
        ggplot2::element_blank()
      },
      panel.grid.major.y = if (grid %in% c("both", "y")) {
        ggplot2::element_line(color = grid_color, linewidth = 0.3)
      } else {
        ggplot2::element_blank()
      },

      # ── strip (for facet_wrap) ─────────────────────────────────
      strip.text       = ggplot2::element_text(face = "bold",
                                               size = base_size * 0.9,
                                               color = "grey20",
                                               margin = ggplot2::margin(t = 4, b = 4)),
      strip.background = ggplot2::element_blank()
    )
}


#' DSIR ggplot2 theme for faceted plots
#'
#' A sibling of [theme_dsi()] tuned for faceted plots. Where `theme_dsi()`
#' uses half-frame axis lines and only horizontal grid lines — a look that
#' suits a single panel — repeating those across every facet looks heavy
#' and the panels run together. `theme_dsi_facet()` replaces the axis
#' lines with a light panel border, draws grid lines on both axes, gives
#' the strip a soft background, and inserts whitespace between panels.
#'
#' Use `theme_dsi()` for single-panel plots and `theme_dsi_facet()` for
#' plots with `facet_wrap()` or `facet_grid()`. Shared elements (text
#' styles, title block, legend, plot margins) match `theme_dsi()` exactly,
#' so the two themes feel like the same family.
#'
#' @param base_size Base font size in points. Default `12`.
#' @param base_family Base font family. Default `""` (system default).
#'   Set to `"sans"`, `"Arial"`, `"Helvetica"`, or `"Calibri"` for a
#'   specific look. Empty default keeps the package portable on CRAN's
#'   Linux check machines, where Calibri is unavailable.
#' @param accent Accent colour, kept for parity with [theme_dsi()] so
#'   that the argument set is interchangeable. Not currently used in the
#'   facet variant — the panel border replaces the accent-coloured axis
#'   lines. Default `"#0093D5"` (WHO blue).
#' @param grid_color Colour of the major grid (both axes). Default
#'   `"grey92"`.
#' @param strip_fill Background fill colour for facet strips. Default
#'   `"grey95"` — a light neutral grey. Avoid blues, which clash with
#'   the WHO-blue `accent`.
#' @param strip_color Text colour inside facet strips. Default `"grey20"`.
#' @param grid Which direction(s) to draw major grid lines. One of
#'   `"both"` (default — both horizontal and vertical), `"y"` (only
#'   horizontal), `"x"` (only vertical), or `"none"`. Matches the
#'   identical argument on [theme_dsi()] so the two themes can be
#'   swapped without changing other settings.
#' @param legend_position Position of the legend. Default `"bottom"`.
#'   Pass `"none"`, `"top"`, `"right"`, or a numeric vector `c(x, y)`.
#'
#' @return A `ggplot2` theme object.
#'
#' @seealso [theme_dsi()] for the single-panel sibling.
#'
#' @examples
#' library(ggplot2)
#' ggplot(mtcars, aes(wt, mpg)) +
#'   geom_point(size = 2, color = "#0093D5") +
#'   facet_wrap(~ cyl, labeller = label_both) +
#'   theme_dsi_facet() +
#'   labs(title = "Fuel efficiency by cylinder count",
#'        x = "Weight (1000 lbs)", y = "Miles per gallon")
#'
#' @export
theme_dsi_facet <- function(base_size = 12,
                            base_family = "",
                            accent = "#0093D5",
                            grid_color = "grey92",
                            strip_fill = "grey95",
                            strip_color = "grey20",
                            grid = c("both", "x", "y", "none"), 
                            legend_position = "bottom") {
  grid <- match.arg(grid)

  .theme_dsi_base(base_size       = base_size,
                  base_family     = base_family,
                  accent          = accent,
                  legend_position = legend_position) +
    ggplot2::theme(
      # ── panel border replaces axis lines ───────────────────────
      panel.border = ggplot2::element_rect(color = "grey80",
                                           fill = NA,
                                           linewidth = 0.4),
      axis.line    = ggplot2::element_blank(),
      axis.ticks   = ggplot2::element_line(color = "grey60",
                                           linewidth = 0.3),

      # ── grid (configurable; default "both" matches facet conventions) ──
      panel.grid.major.x = if (grid %in% c("both", "x")) {
        ggplot2::element_line(color = grid_color, linewidth = 0.3)
      } else {
        ggplot2::element_blank()
      },
      panel.grid.major.y = if (grid %in% c("both", "y")) {
        ggplot2::element_line(color = grid_color, linewidth = 0.3)
      } else {
        ggplot2::element_blank()
      },

      # ── strip ──────────────────────────────────────────────────
      strip.background = ggplot2::element_rect(fill = strip_fill,
                                               color = NA),
      strip.text       = ggplot2::element_text(face = "bold",
                                               size = base_size * 0.9,
                                               color = strip_color,
                                               margin = ggplot2::margin(t = 6, r = 6,
                                                                        b = 6, l = 6)),

      # ── panel spacing (so panels don't run together) ───────────
      panel.spacing = grid::unit(8, "pt")
    )
}


# Internal helper. Holds every theme element that is identical between
# theme_dsi() and theme_dsi_facet() — text styles, title block, legend,
# plot margins, plot background. The two wrappers then add the elements
# that differ (axis line vs panel border, one-axis vs two-axis grid,
# transparent vs filled strip background).
#
# `accent` is plumbed through for forward compatibility even though this
# helper does not currently use it directly.
.theme_dsi_base <- function(base_size,
                            base_family,
                            accent,
                            legend_position) {

  ggplot2::theme_minimal(
    base_size   = base_size,
    base_family = base_family
  ) +
    ggplot2::theme(
      # ── text ────────────────────────────────────────────────────
      text         = ggplot2::element_text(color = "grey20"),
      axis.text    = ggplot2::element_text(color = "grey30",
                                           size = base_size * 0.85),
      axis.title   = ggplot2::element_text(color = "grey20",
                                           face = "bold",
                                           size = base_size * 0.9),
      axis.title.x = ggplot2::element_text(margin = ggplot2::margin(t = 10)),
      axis.title.y = ggplot2::element_text(margin = ggplot2::margin(r = 10)),

      # ── title block ─────────────────────────────────────────────
      plot.title       = ggplot2::element_text(face   = "bold",
                                               size   = base_size * 1.4,
                                               color  = "grey10",
                                               margin = ggplot2::margin(b = 4)),
      plot.subtitle    = ggplot2::element_text(size   = base_size * 1.0,
                                               color  = "grey40",
                                               margin = ggplot2::margin(b = 12)),
      plot.caption     = ggplot2::element_text(size   = base_size * 0.75,
                                               color  = "grey50",
                                               hjust  = 1,
                                               margin = ggplot2::margin(t = 12)),
      plot.title.position   = "plot",
      plot.caption.position = "plot",

      # ── shared axis spacing / minor grid ───────────────────────
      axis.ticks.length = grid::unit(3, "pt"),
      panel.grid.minor  = ggplot2::element_blank(),

      # ── legend ─────────────────────────────────────────────────
      legend.position  = legend_position,
      legend.title     = ggplot2::element_text(face = "bold",
                                               size = base_size * 0.85),
      legend.text      = ggplot2::element_text(size = base_size * 0.85),
      legend.key.size  = grid::unit(12, "pt"),
      legend.margin    = ggplot2::margin(t = 6),

      # ── plot area ──────────────────────────────────────────────
      plot.background  = ggplot2::element_blank(),
      panel.background = ggplot2::element_blank(),
      plot.margin      = ggplot2::margin(t = 12, r = 16, b = 12, l = 12)
    )
}

test_that("theme_dsi returns a ggplot theme", {
  th <- theme_dsi()
  expect_s3_class(th, "theme")
  expect_s3_class(th, "gg")
})

test_that("theme_dsi respects base_size and accent args", {
  th <- theme_dsi(base_size = 16, accent = "darkred")

  expect_equal(th$text$size, 16)

  expect_equal(th$axis.line.x$colour, "darkred")
  expect_equal(th$axis.line.y$colour, "darkred")

  expect_equal(th$text$colour, "grey20")
})

test_that("theme_dsi default draws grid in both directions", {
  th <- theme_dsi()
  expect_s3_class(th$panel.grid.major.x, "element_line")
  expect_s3_class(th$panel.grid.major.y, "element_line")
})

test_that("theme_dsi(grid = 'y') draws only horizontal grid", {
  th <- theme_dsi(grid = "y")
  expect_s3_class(th$panel.grid.major.x, "element_blank")
  expect_s3_class(th$panel.grid.major.y, "element_line")
})

test_that("theme_dsi(grid = 'x') draws only vertical grid", {
  th <- theme_dsi(grid = "x")
  expect_s3_class(th$panel.grid.major.x, "element_line")
  expect_s3_class(th$panel.grid.major.y, "element_blank")
})

test_that("theme_dsi(grid = 'none') draws no major grid", {
  th <- theme_dsi(grid = "none")
  expect_s3_class(th$panel.grid.major.x, "element_blank")
  expect_s3_class(th$panel.grid.major.y, "element_blank")
})

test_that("theme_dsi rejects invalid grid value", {
  expect_error(theme_dsi(grid = "diagonal"))
})

test_that("theme_dsi_facet returns a ggplot2 theme", {
  expect_s3_class(theme_dsi_facet(), "theme")
})

test_that("theme_dsi_facet respects strip_fill parameter", {
  th <- theme_dsi_facet(strip_fill = "#E5F4FB")
  expect_equal(th$strip.background$fill, "#E5F4FB")
})

test_that("theme_dsi_facet has panel border (unlike theme_dsi)", {
  expect_s3_class(theme_dsi_facet()$panel.border, "element_rect")
  expect_s3_class(theme_dsi()$panel.border, "element_blank")
})

test_that("theme_dsi axis lines still use the accent colour after refactor", {
  th <- theme_dsi()
  expect_s3_class(th$axis.line.x, "element_line")
  expect_equal(th$axis.line.x$colour, "#0093D5")
})

test_that("theme_dsi_facet default draws grid in both directions", {
  th <- theme_dsi_facet()
  expect_s3_class(th$panel.grid.major.x, "element_line")
  expect_s3_class(th$panel.grid.major.y, "element_line")
})

test_that("theme_dsi_facet(grid = 'y') draws only horizontal grid", {
  th <- theme_dsi_facet(grid = "y")
  expect_s3_class(th$panel.grid.major.x, "element_blank")
  expect_s3_class(th$panel.grid.major.y, "element_line")
})

test_that("theme_dsi_facet(grid = 'none') draws no major grid", {
  th <- theme_dsi_facet(grid = "none")
  expect_s3_class(th$panel.grid.major.x, "element_blank")
  expect_s3_class(th$panel.grid.major.y, "element_blank")
})

test_that("theme_dsi_facet rejects invalid grid value", {
  expect_error(theme_dsi_facet(grid = "diagonal"))
})

# =============================================================================
# R/data.R
#
# Roxygen documentation for datasets shipped with DSIR.
# Datasets themselves are generated by data-raw/who_countries.R.
# =============================================================================


#' WHO Member States with regional and Pacific classifications
#'
#' A tibble of the 194 World Health Organization (WHO) Member States, with
#' standard country identifiers and WHO/Pacific groupings used across DSIR
#' analytical workflows.
#'
#' @format A tibble with 194 rows and 7 columns:
#' \describe{
#'   \item{iso3}{ISO 3166-1 alpha-3 code (3-letter, e.g. `"PHL"`).}
#'   \item{iso2}{ISO 3166-1 alpha-2 code (2-letter, e.g. `"PH"`).}
#'   \item{m49_code}{UN M49 numeric code, stored as a 3-character string with
#'   leading zeros where present (e.g. `"008"` for Albania). Stored as
#'   character because some downstream APIs (notably the UN SDG API) expect
#'   the leading-zero form.}
#'   \item{name_official}{WHO official English name (e.g.
#'   `"Iran (Islamic Republic of)"`). Use for formal documents and reports.}
#'   \item{name_short}{A shorter form suitable for charts and tables (e.g.
#'   `"Iran"`). Equal to `name_official` for countries whose official name
#'   is already concise. See Details.}
#'   \item{who_region}{WHO region code: one of `"AFR"`, `"AMR"`, `"SEAR"`,
#'   `"EUR"`, `"EMR"`, `"WPR"`.}
#'   \item{is_pic}{Logical. `TRUE` for the 14 Pacific Island Country (PIC)
#'   Member States in WPR, `FALSE` otherwise.}
#' }
#'
#' @details
#' **Scope.** WHO has 194 Member States plus 2 Associate Members
#' (Puerto Rico, Tokelau) and reports on additional non-Member areas
#' (e.g. West Bank and Gaza Strip). This dataset includes only the 194
#' Member States. Cook Islands and Niue are full WHO Member States and
#' are included even though they are not UN member states.
#'
#' **Region coverage** (as of May 2025, after WHO EB156 reassignment of
#' Indonesia from SEAR to WPR):
#' \itemize{
#'   \item AFR (African Region): 47
#'   \item AMR (Region of the Americas): 35
#'   \item SEAR (South-East Asia Region): 10
#'   \item EUR (European Region): 53
#'   \item EMR (Eastern Mediterranean Region): 21
#'   \item WPR (Western Pacific Region): 28
#' }
#'
#' **Short names.** `name_short` differs from `name_official` for 13
#' countries where the official name is a parenthetical descriptor or is
#' otherwise long. Examples: `"DPR Korea"`, `"DR Congo"`, `"Lao PDR"`,
#' `"United Kingdom"`. These short forms follow conventions used in WHO
#' regional reports and OECD Health at a Glance: Asia/Pacific.
#'
#' **PICs.** The Pacific Island Countries flag (`is_pic`) marks the 14
#' WHO Member States in the Pacific sub-region: Cook Islands, Fiji,
#' Kiribati, Marshall Islands, Micronesia (Federated States of), Nauru,
#' Niue, Palau, Papua New Guinea, Samoa, Solomon Islands, Tonga, Tuvalu,
#' and Vanuatu. Non-Member Pacific areas (e.g. New Caledonia, French
#' Polynesia, American Samoa) are not included in this dataset.
#'
#' @source
#' \itemize{
#'   \item WHO official region listing:
#'   \url{https://www.who.int/countries}
#'   \item ISO 3166-1 codes and UN M49 numeric codes: UN Statistics Division
#'   \url{https://unstats.un.org/unsd/methodology/m49/}
#' }
#'
#' @examples
#' # All WPR Member States, sorted alphabetically by short name
#' wpr <- who_countries[who_countries$who_region == "WPR", ]
#' wpr <- wpr[order(wpr$name_short), c("iso3", "name_short")]
#' head(wpr)
#'
#' # Filter a data frame to PIC member states
#' # df_pic <- subset(my_data, country_iso3 %in% who_countries$iso3[who_countries$is_pic])
"who_countries"


#' WHO regional Member State ISO3 vectors
#'
#' Convenience character vectors of ISO3 codes for the WHO Member States
#' in each region. Each vector is the regional subset of
#' [`who_countries`]'s `iso3` column, sorted alphabetically.
#'
#' @details
#' These are derived directly from [`who_countries`] and exist for
#' ergonomic filtering, e.g.
#' `dplyr::filter(data, iso3 %in% wpro_cty)`.
#' If you need to update group membership, edit
#' `data-raw/who_countries.R` and re-run that script — the vectors
#' regenerate from the master table.
#'
#' \describe{
#'   \item{`afro_cty`}{47 Member States in the WHO African Region.}
#'   \item{`amro_cty`}{35 Member States in the WHO Region of the Americas.}
#'   \item{`searo_cty`}{10 Member States in the WHO South-East Asia Region.}
#'   \item{`euro_cty`}{53 Member States in the WHO European Region.}
#'   \item{`emro_cty`}{21 Member States in the WHO Eastern Mediterranean
#'   Region.}
#'   \item{`wpro_cty`}{28 Member States in the WHO Western Pacific Region.
#'   Includes Indonesia from May 2025 (per WHO EB156).}
#' }
#'
#' @format Character vectors of ISO 3166-1 alpha-3 codes.
#'
#' @seealso [`who_countries`], [`pic_cty`].
#'
#' @examples
#' length(wpro_cty)        # 28
#' "IDN" %in% wpro_cty     # TRUE — Indonesia in WPR since May 2025
#'
#' @name who_region_vectors
NULL

#' @rdname who_region_vectors
"afro_cty"

#' @rdname who_region_vectors
"amro_cty"

#' @rdname who_region_vectors
"searo_cty"

#' @rdname who_region_vectors
"euro_cty"

#' @rdname who_region_vectors
"emro_cty"

#' @rdname who_region_vectors
"wpro_cty"


#' Pacific Island Country ISO3 codes
#'
#' Character vector of ISO3 codes for the 14 WHO Member States classified
#' as Pacific Island Countries (PICs). Sorted alphabetically.
#'
#' @details
#' All 14 PICs are within the Western Pacific Region, so
#' `all(pic_cty %in% wpro_cty)` is `TRUE`. Non-Member Pacific areas
#' (e.g. New Caledonia, French Polynesia, American Samoa, Tokelau) are
#' not included.
#'
#' The 14 PIC Member States are: Cook Islands, Fiji, Kiribati, Marshall
#' Islands, Micronesia (Federated States of), Nauru, Niue, Palau, Papua
#' New Guinea, Samoa, Solomon Islands, Tonga, Tuvalu, and Vanuatu.
#'
#' @format A character vector of 14 ISO 3166-1 alpha-3 codes.
#'
#' @seealso [`who_countries`], [`wpro_cty`].
#'
#' @examples
#' length(pic_cty)              # 14
#' all(pic_cty %in% wpro_cty)   # TRUE
"pic_cty"

#' DSIR: Data Science Infrastructure for Global Health in R
#'
#' A small, opinionated toolkit for global health data analysis.
#' It bundles a publication-ready `ggplot2` theme ([theme_dsi()],
#' [theme_dsi_facet()]) with flush-axis bar-chart scales
#' ([scale_y_dsi_col()], [scale_x_dsi_col()]),
#' sensible `flextable` defaults ([dsi_flextable_defaults()]),
#' a quick pie chart helper ([ggpie()]), regional country-code
#' datasets ([wpro_cty]) with WHO region and UN M49 lookups
#' ([iso3_to_region()], [iso3_to_m49()], [m49_to_iso3()]), a
#' geometric mean helper for indicator aggregation ([geomean()]),
#' thin clients for the WHO Global Health Observatory API
#' ([gho_indicators()], [gho_data()], [gho_dimensions()],
#' [gho_has_data()], [gho_count()], [gho_coverage()]) and the UN
#' Sustainable Development Goals API ([sdg_goals()], [sdg_targets()],
#' [sdg_indicators()], [sdg_areas()], [sdg_data()], [sdg_coverage()]),
#' plus a unified cleaning / binding pipeline ([gho_clean()],
#' [sdg_clean()], [bind_indicators()]) that puts GHO and SDG output
#' into the same 15-column schema.
#'
#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @importFrom rlang .data %||%
#' @importFrom tibble as_tibble tibble
## usethis namespace: end
NULL

# Silence R CMD check NOTE for the lazy-loaded `who_countries` dataset
# referenced unqualified inside package functions (e.g. iso3_to_region()).
utils::globalVariables("who_countries")

#' Set DSIR Flextable Defaults
#'
#' Applies a consistent set of `flextable` formatting defaults for
#' publication-ready tables (booktabs theme, bold headers, modest
#' padding). Pick any font you like — the default `""` leaves the
#' `flextable` default in place so the call is safe on systems where
#' Cambria is not installed.
#'
#' @param font_size Font size in points. Default `12`.
#' @param font_family Font family name. Default `""` keeps the
#'   existing `flextable` default; try `"Cambria"` for the original
#'   DSIR look on Windows.
#' @param font_color Font color. Default `"#333333"`.
#' @param border_color Border color. Default `"black"`.
#' @param padding Numeric vector of length 1 (applied to all sides)
#'   or length 4 (`top`, `bottom`, `left`, `right`).
#'   Default `c(3, 3, 4, 4)`.
#'
#' @return Invisibly returns `NULL`. Called for its side effect of
#'   mutating the `flextable` global defaults via
#'   [flextable::set_flextable_defaults()].
#' @export
#'
#' @examples
#' dsi_flextable_defaults()
dsi_flextable_defaults <- function(font_size = 12,
                                   font_family = "",
                                   font_color = "#333333",
                                   border_color = "black",
                                   padding = c(3, 3, 4, 4)) {
  if (length(padding) == 1) padding <- rep(padding, 4)

  args <- list(
    font.size      = font_size,
    font.color     = font_color,
    border.color   = border_color,
    padding.top    = padding[1],
    padding.bottom = padding[2],
    padding.left   = padding[3],
    padding.right  = padding[4],
    theme_fun      = function(x) flextable::theme_booktabs(x, bold_header = TRUE)
  )
  if (nzchar(font_family)) args$font.family <- font_family

  do.call(flextable::set_flextable_defaults, args)
  invisible(NULL)
}

test_that("dsi_flextable_defaults runs without error", {
  old <- flextable::get_flextable_defaults()
  on.exit(do.call(flextable::set_flextable_defaults, old), add = TRUE)
  expect_null(dsi_flextable_defaults())
  expect_null(dsi_flextable_defaults(padding = 2))
})

test_that("dsi_flextable_defaults accepts length-4 padding (top/bottom/left/right)", {
  old <- flextable::get_flextable_defaults()
  on.exit(do.call(flextable::set_flextable_defaults, old), add = TRUE)

  expect_null(dsi_flextable_defaults(padding = c(1, 2, 3, 4)))
  d <- flextable::get_flextable_defaults()
  expect_equal(d$padding.top,    1)
  expect_equal(d$padding.bottom, 2)
  expect_equal(d$padding.left,   3)
  expect_equal(d$padding.right,  4)
})

test_that("dsi_flextable_defaults length-1 padding broadcasts to all four sides", {
  old <- flextable::get_flextable_defaults()
  on.exit(do.call(flextable::set_flextable_defaults, old), add = TRUE)

  expect_null(dsi_flextable_defaults(padding = 5))
  d <- flextable::get_flextable_defaults()
  expect_equal(d$padding.top,    5)
  expect_equal(d$padding.bottom, 5)
  expect_equal(d$padding.left,   5)
  expect_equal(d$padding.right,  5)
})

# Package-level cache for session-scoped data. Currently used to memoise
# the GHO indicator catalog (see .gho_indicator_catalog()).
.dsi_cache <- new.env(parent = emptyenv())


#' List GHO Indicators
#'
#' Fetches the catalog of indicators from the WHO Global Health
#' Observatory (GHO) OData API.
#'
#' @param search Optional character. Search keywords matched against
#'   `IndicatorName` (case-insensitive). All terms must match
#'   (AND semantics). Accepts either:
#'   * a single string, which is split on whitespace into terms
#'     (e.g. `"child mortality"` matches indicators containing both
#'     "child" and "mortality"), or
#'   * a character vector, whose elements are used as terms verbatim
#'     (whitespace inside an element is treated as part of the term).
#'
#'   Single quotes in any term are escaped for the OData filter.
#'
#' @return A [tibble][tibble::tibble] with columns `IndicatorCode`,
#'   `IndicatorName` and `Language`. Returns an empty tibble (with
#'   a message) when the service is unreachable.
#' @seealso [gho_data()], [gho_dimensions()].
#' @export
#'
#' @examples
#' \donttest{
#' # All indicators
#' inds <- gho_indicators()
#'
#' # Single keyword
#' gho_indicators("mortality")
#'
#' # Multiple keywords from one string (AND): both terms must appear
#' gho_indicators("child mortality")
#'
#' # Or pass terms as a vector
#' gho_indicators(c("child", "mortality"))
#' }
gho_indicators <- function(search = NULL) {
  url <- .gho_indicators_build_url(search)

  res <- .gho_get(url)
  if (is.null(res) || nrow(res) == 0L) {
    return(tibble::tibble(IndicatorCode = character(),
                          IndicatorName = character(),
                          Language = character()))
  }
  res[, c("IndicatorCode", "IndicatorName", "Language")]
}


#' @noRd
.gho_indicators_build_url <- function(search = NULL) {
  base_url <- "https://ghoapi.azureedge.net/api/Indicator"
  if (is.null(search)) return(base_url)

  stopifnot(
    is.character(search),
    length(search) >= 1L,
    !anyNA(search),
    all(nzchar(search))
  )

  terms <- if (length(search) == 1L) {
    strsplit(search, "\\s+")[[1]]
  } else {
    search
  }
  terms <- terms[nzchar(terms)]
  stopifnot(length(terms) >= 1L)

  terms_escaped <- gsub("'", "''", tolower(terms), fixed = TRUE)
  clauses <- paste0("contains(tolower(IndicatorName),'", terms_escaped, "')")
  filter <- paste(clauses, collapse = " and ")
  paste0(base_url, "?$filter=", utils::URLencode(filter, reserved = TRUE))
}


#' Fetch GHO Data
#'
#' Retrieves observations for a specific indicator from the WHO GHO
#' OData API, with optional filters by spatial level, country /
#' region and year range.
#'
#' @param indicator Character scalar. The indicator code
#'   (e.g. `"NCDMORT3070"`). Use [gho_indicators()] to find codes.
#' @param spatial_type Character. Spatial dimension to filter on:
#'   one of `"country"`, `"region"`, `"global"`, or `NULL` (all
#'   levels, the default).
#' @param area Character vector of country or region codes
#'   (e.g. `c("FRA", "DEU")`). Default `NULL` returns all areas.
#' @param year_from Numeric. Start year filter (inclusive).
#'   Default `NULL`.
#' @param year_to Numeric. End year filter (inclusive).
#'   Default `NULL`.
#'
#' @return A [tibble][tibble::tibble] of indicator observations, or
#'   an empty tibble when the service is unreachable.
#' @seealso [gho_indicators()], [gho_dimensions()].
#' @export
#'
#' @examples
#' \donttest{
#' # Country-level data for one indicator
#' gho_data("NCDMORT3070", spatial_type = "country")
#'
#' # Specific countries and years
#' gho_data("WHOSIS_000001", area = c("FRA", "DEU"), year_from = 2015)
#' }
gho_data <- function(indicator, spatial_type = NULL, area = NULL,
                     year_from = NULL, year_to = NULL) {
  url <- .gho_build_url(indicator,
                        spatial_type = spatial_type, area = area,
                        year_from = year_from, year_to = year_to)
  res <- .gho_get(url)
  if (is.null(res)) tibble::tibble() else res
}


#' @noRd
.gho_build_url <- function(indicator, spatial_type = NULL, area = NULL,
                           year_from = NULL, year_to = NULL,
                           top = NULL, select = NULL, count = FALSE) {
  stopifnot(is.character(indicator), length(indicator) == 1L, nzchar(indicator))
  base_url <- paste0("https://ghoapi.azureedge.net/api/", indicator)

  filters <- character(0)

  if (!is.null(area) && is.null(spatial_type)) {
    cli::cli_inform(c(
      "Assuming {.arg spatial_type} = {.val country} since {.arg area} was given.",
      "i" = "Pass {.arg spatial_type} explicitly to silence this message."
    ))
    spatial_type <- "country"
  }

  if (!is.null(spatial_type)) {
    spatial_type <- tolower(spatial_type)
    st <- switch(spatial_type,
                 country = "COUNTRY",
                 region  = "REGION",
                 global  = "GLOBAL",
                 cli::cli_abort("Unknown {.arg spatial_type}: {.val {spatial_type}}. Use \"country\", \"region\", or \"global\".")
    )
    filters <- c(filters, paste0("SpatialDimType eq '", st, "'"))
  }

  if (!is.null(area)) {
    stopifnot(
      is.character(area),
      length(area) >= 1L,
      !anyNA(area),
      all(nzchar(area))
    )
    area_filter <- paste0(
      "SpatialDim in ('",
      paste(area, collapse = "','"),
      "')"
    )
    filters <- c(filters, area_filter)
  }

  if (!is.null(year_from)) {
    filters <- c(filters, paste0("TimeDim ge ", year_from))
  }
  if (!is.null(year_to)) {
    filters <- c(filters, paste0("TimeDim le ", year_to))
  }

  query_parts <- character(0)
  if (length(filters) > 0) {
    filter_str <- paste(filters, collapse = " and ")
    query_parts <- c(query_parts,
                     paste0("$filter=", utils::URLencode(filter_str, reserved = TRUE)))
  }
  if (!is.null(top)) {
    query_parts <- c(query_parts, paste0("$top=", top))
  }
  if (!is.null(select)) {
    query_parts <- c(query_parts, paste0("$select=", paste(select, collapse = ",")))
  }
  if (isTRUE(count)) {
    query_parts <- c(query_parts, "$count=true")
  }

  if (length(query_parts) == 0L) return(base_url)
  paste0(base_url, "?", paste(query_parts, collapse = "&"))
}


#' Check Whether a GHO Indicator Has Data for a Filter
#'
#' Sends a minimal request (`$top=1&$select=Id`) to the WHO GHO OData
#' API to find out whether any observations exist for the given
#' indicator and filter combination, without downloading the full
#' result set. Useful as a quick precheck before [gho_data()].
#'
#' @inheritParams gho_data
#'
#' @return A logical scalar:
#' * `TRUE` if at least one observation exists for the filter.
#' * `FALSE` if the server returns an empty result.
#' * `NA` if the request fails (network failure, unreachable host,
#'   or the indicator code does not exist and the server returns an
#'   HTTP error). A warning is emitted in the failure case.
#' @seealso [gho_data()], [gho_count()], [gho_coverage()].
#' @export
#'
#' @examples
#' \donttest{
#' # Does WHO have life-expectancy data for France?
#' gho_has_data("WHOSIS_000001", area = "FRA")
#'
#' # Quickly screen a list of indicators before downloading any data
#' inds <- c("WHOSIS_000001", "NCDMORT3070")
#' vapply(inds, gho_has_data, logical(1), area = "FRA")
#' }
gho_has_data <- function(indicator, spatial_type = NULL, area = NULL,
                         year_from = NULL, year_to = NULL) {
  url <- .gho_build_url(indicator,
                        spatial_type = spatial_type, area = area,
                        year_from = year_from, year_to = year_to,
                        top = 1, select = "Id")
  res <- .gho_get(url)
  if (is.null(res)) return(NA)
  nrow(res) > 0L
}


#' Count Observations for a GHO Indicator Filter
#'
#' Sends a `$top=0&$count=true` request to the WHO GHO OData API,
#' which returns the matching row count without transferring any
#' observations. Useful for sizing a download before issuing it.
#'
#' @inheritParams gho_data
#'
#' @return An integer scalar — the number of observations the
#'   server would return for the same filter via [gho_data()].
#'   Returns `NA_integer_` (with a warning) if the request fails.
#' @seealso [gho_data()], [gho_has_data()], [gho_coverage()].
#' @export
#'
#' @examples
#' \donttest{
#' # How many rows would gho_data() pull for France?
#' gho_count("WHOSIS_000001", area = "FRA")
#'
#' # Compare coverage across regions
#' gho_count("NCDMORT3070", spatial_type = "country")
#' gho_count("NCDMORT3070", spatial_type = "region")
#' }
gho_count <- function(indicator, spatial_type = NULL, area = NULL,
                      year_from = NULL, year_to = NULL) {
  url <- .gho_build_url(indicator,
                        spatial_type = spatial_type, area = area,
                        year_from = year_from, year_to = year_to,
                        top = 0, count = TRUE)

  cli::cli_inform("Fetching: {.url {url}}")
  resp <- tryCatch(
    httr2::request(url) |>
      httr2::req_headers(Accept = "application/json") |>
      httr2::req_timeout(20) |>
      httr2::req_retry(
        max_tries = 3,
        backoff   = ~ min(2 ^ .x, 30)
      ) |>
      httr2::req_perform(),
    error = function(e) {
      # Reference the message via a variable so cli does not glue-interpret
      # any literal braces the error message may carry (see body parse below).
      msg <- conditionMessage(e)
      cli::cli_warn(c(
        "GHO request failed.",
        "i" = "URL: {.url {url}}",
        "x" = "{msg}"
      ))
      NULL
    }
  )
  if (is.null(resp)) return(NA_integer_)

  # Body parse must also fail soft: a truncated response body (premature
  # EOF) would otherwise propagate a jsonlite parse error and break
  # R CMD check examples (CRAN-blocking).
  body <- tryCatch(
    httr2::resp_body_json(resp, simplifyVector = TRUE),
    error = function(e) {
      # Reference the message via a variable: a jsonlite parse error
      # carries literal `{`/`}` from the offending JSON, which cli would
      # otherwise try to interpret as glue expressions and re-error.
      msg <- conditionMessage(e)
      cli::cli_warn(c(
        "GHO response could not be parsed as JSON.",
        "i" = "URL: {.url {url}}",
        "x" = "{msg}"
      ))
      NULL
    }
  )
  if (is.null(body)) return(NA_integer_)
  cnt <- body[["@odata.count"]]
  if (is.null(cnt)) return(NA_integer_)
  as.integer(cnt)
}


#' Summarise Per-Location Data Coverage of a GHO Indicator
#'
#' Fetches only the `SpatialDim` and `TimeDim` columns for a GHO
#' indicator (much lighter than [gho_data()]) and summarises the
#' year range and observation count per location. Useful for
#' answering "which countries have data, and for what years?"
#' before committing to a full download.
#'
#' @param indicator Character scalar. The indicator code
#'   (e.g. `"WHOSIS_000001"`).
#' @param spatial_type Character. Spatial dimension to filter on:
#'   one of `"country"`, `"region"`, `"global"`. Defaults to
#'   `"country"` since per-country coverage is the typical use
#'   case. Pass `NULL` for all spatial levels.
#' @param area Character vector of country or region codes
#'   (e.g. `c("FRA", "DEU")`). Default `NULL` returns all areas
#'   for the chosen `spatial_type`.
#' @param year_from Numeric. Start year filter (inclusive).
#'   Default `NULL`.
#' @param year_to Numeric. End year filter (inclusive).
#'   Default `NULL`.
#'
#' @return A [tibble][tibble::tibble] with one row per location and
#'   columns:
#' * `location` (chr) — the `SpatialDim` value (typically ISO3).
#' * `year_min` (int) — earliest year with data.
#' * `year_max` (int) — latest year with data.
#' * `n_obs` (int) — number of observations.
#'
#'   Sorted by `location`. Empty input or service failure returns
#'   an empty tibble with the same four columns.
#' @seealso [gho_data()], [gho_has_data()], [gho_count()].
#' @export
#'
#' @examples
#' \donttest{
#' # Year coverage of life expectancy for three countries
#' gho_coverage("WHOSIS_000001", area = c("FRA", "DEU", "JPN"))
#'
#' # All countries with any life-expectancy data, since 2010
#' gho_coverage("WHOSIS_000001", year_from = 2010)
#' }
gho_coverage <- function(indicator, spatial_type = "country", area = NULL,
                         year_from = NULL, year_to = NULL) {
  empty <- tibble::tibble(
    location = character(),
    year_min = integer(),
    year_max = integer(),
    n_obs    = integer()
  )

  url <- .gho_build_url(indicator,
                        spatial_type = spatial_type, area = area,
                        year_from = year_from, year_to = year_to,
                        select = c("SpatialDim", "TimeDim"))
  res <- .gho_get(url)
  if (is.null(res) || nrow(res) == 0L) return(empty)
  if (!all(c("SpatialDim", "TimeDim") %in% names(res))) return(empty)

  loc <- as.character(res$SpatialDim)
  yr  <- suppressWarnings(as.integer(res$TimeDim))

  by_loc <- split(yr, loc)
  by_loc <- by_loc[order(names(by_loc))]

  yr_range <- function(x, fn) {
    x <- x[!is.na(x)]
    if (length(x) == 0L) NA_integer_ else as.integer(fn(x))
  }

  tibble::tibble(
    location = names(by_loc),
    year_min = vapply(by_loc, yr_range, integer(1), fn = min),
    year_max = vapply(by_loc, yr_range, integer(1), fn = max),
    n_obs    = vapply(by_loc, length, integer(1))
  )
}


#' List Dimensions of a GHO Indicator
#'
#' Returns the unique values of a given dimension across all
#' observations of a GHO indicator. Useful for discovering which
#' ages, sexes, regions, or other breakdowns are available before
#' calling [gho_data()].
#'
#' @param indicator Character scalar. The indicator code
#'   (e.g. `"NCDMORT3070"`).
#' @param dimension Character. Name of the dimension column in the
#'   indicator data. Common values include `"SpatialDim"`,
#'   `"SpatialDimType"`, `"TimeDim"`, `"Dim1"`, `"Dim2"`, and
#'   `"Dim3"`. Default `"SpatialDimType"`.
#'
#' @return A character vector of unique, sorted dimension values,
#'   or an empty character vector when the service is unreachable
#'   or the dimension is missing.
#' @seealso [gho_data()], [gho_indicators()].
#' @export
#'
#' @examples
#' \donttest{
#' gho_dimensions("NCDMORT3070")
#' gho_dimensions("NCDMORT3070", dimension = "Dim1")
#' }
gho_dimensions <- function(indicator, dimension = "SpatialDimType") {
  stopifnot(is.character(indicator), length(indicator) == 1L, nzchar(indicator))
  stopifnot(is.character(dimension), length(dimension) == 1L)

  res <- .gho_get(paste0("https://ghoapi.azureedge.net/api/", indicator))
  if (is.null(res) || !dimension %in% names(res)) return(character())
  vals <- unique(res[[dimension]])
  sort(vals[!is.na(vals)])
}


#' @noRd
.gho_indicator_catalog <- function() {
  if (is.null(.dsi_cache$gho_indicator_catalog)) {
    .dsi_cache$gho_indicator_catalog <- gho_indicators()
  }
  .dsi_cache$gho_indicator_catalog
}


#' @noRd
.gho_resolve_location_name <- function(location) {
  if (length(location) == 0L) return(character(0))

  # WHO regional and aggregate codes. Hardcoded because they are stable
  # and not part of who_countries (which lists Member States only).
  region_names <- c(
    AFR    = "Africa",
    AMR    = "Americas",
    SEAR   = "South-East Asia",
    EUR    = "Europe",
    EMR    = "Eastern Mediterranean",
    WPR    = "Western Pacific",
    GLOBAL = "Global"
  )

  out <- rep(NA_character_, length(location))

  iso3_match <- match(location, who_countries$iso3)
  has_iso3 <- !is.na(iso3_match)
  out[has_iso3] <- who_countries$name_short[iso3_match[has_iso3]]

  region_match <- match(location, names(region_names))
  has_region <- !is.na(region_match)
  out[has_region] <- region_names[region_match[has_region]]

  out
}


#' @noRd
.gho_resolve_indicator_name <- function(codes) {
  if (all(is.na(codes))) return(.fill_na(length(codes), "chr"))

  catalog <- .gho_indicator_catalog()

  if (nrow(catalog) == 0L) return(.fill_na(length(codes), "chr"))

  catalog$IndicatorName[match(codes, catalog$IndicatorCode)]
}


#' Tidy a GHO Data Frame
#'
#' Selects, renames, and type-casts the most useful columns from a GHO
#' observation table returned by [gho_data()], producing a compact
#' tibble in the **unified DSIR cleaned-indicator schema** — the same
#' schema produced by [sdg_clean()], so the two outputs can be combined
#' directly with [bind_indicators()].
#'
#' The mapping (GHO source → unified column) is:
#' * `IndicatorCode` → `id`
#' * `IndicatorCode` resolved against the GHO indicator catalog →
#'   `indicator` (the human-readable name; cached at session level
#'   after the first call)
#' * `SpatialDim`    → `location`; also `iso3` when it matches a WHO
#'   Member State, otherwise `iso3 = NA`
#' * `TimeDim`       → `year` (integer)
#' * `Value`         → `value` (character; raw)
#' * `NumericValue`  → `value_num` (numeric)
#' * `Low`, `High`   → `low`, `high` (numeric)
#' * `Dim1`, `Dim2`, `Dim3` → `dim1`, `dim2`, `dim3` (character)
#'
#' The `series` column is always `NA` for GHO output (it is an SDG-only
#' concept). The `location_name` column is populated by looking up
#' `location` (an ISO3 code or a WHO region code) against the
#' [`who_countries`] dataset and a hardcoded set of WHO regional names;
#' locations that match neither (e.g. non-Member State areas) are left
#' as `NA`.
#'
#' Source columns absent from `df` (e.g. `Low` / `High` for indicators
#' without confidence intervals) are filled with typed `NA`, so the
#' output always has the same 15 columns with the same column types.
#'
#' The GHO data endpoint (`/api/{IndicatorCode}`) does not return
#' `IndicatorName`; that field lives on the catalog endpoint queried by
#' [gho_indicators()]. On the first call within an R session,
#' `gho_clean()` fetches the catalog once and caches it for the rest of
#' the session, so the `indicator` column carries the full
#' human-readable indicator name. If the catalog cannot be fetched
#' (e.g. no network), [gho_indicators()] emits a warning and the
#' `indicator` column falls back to `NA`.
#'
#' @param df A data frame returned by [gho_data()].
#'
#' @return A [tibble][tibble::tibble] with 15 columns: `source` (always
#'   `"gho"`), `id`, `indicator`, `location`, `iso3`, `location_name`,
#'   `year`, `value`, `value_num`, `low`, `high`, `series` (`NA`),
#'   `dim1`, `dim2`, `dim3`. Sorted by `location` then `year`.
#'   Empty input returns an empty tibble with the same columns and
#'   types.
#' @seealso [gho_data()], [sdg_clean()], [bind_indicators()].
#' @export
#'
#' @examples
#' \donttest{
#' gho_data("NCDMORT3070", spatial_type = "country") |>
#'   gho_clean()
#' }
gho_clean <- function(df) {
  if (!is.data.frame(df)) {
    cli::cli_abort("{.arg df} must be a data frame.")
  }

  n <- nrow(df)
  if (n == 0L) return(.dsi_empty_clean())

  pick_chr <- function(src) {
    if (src %in% names(df)) as.character(df[[src]]) else .fill_na(n, "chr")
  }
  pick_num <- function(src) {
    if (src %in% names(df)) {
      suppressWarnings(as.numeric(df[[src]]))
    } else {
      .fill_na(n, "num")
    }
  }
  pick_int <- function(src) {
    if (src %in% names(df)) {
      suppressWarnings(as.integer(df[[src]]))
    } else {
      .fill_na(n, "int")
    }
  }

  location <- pick_chr("SpatialDim")
  iso3     <- ifelse(location %in% who_countries$iso3,
                     location, NA_character_)

  out <- tibble::tibble(
    source        = rep("gho", n),
    id            = pick_chr("IndicatorCode"),
    indicator     = .gho_resolve_indicator_name(pick_chr("IndicatorCode")),
    location      = location,
    iso3          = iso3,
    location_name = .gho_resolve_location_name(location),
    year          = pick_int("TimeDim"),
    value         = pick_chr("Value"),
    value_num     = pick_num("NumericValue"),
    low           = pick_num("Low"),
    high          = pick_num("High"),
    series        = .fill_na(n, "chr"),
    dim1          = pick_chr("Dim1"),
    dim2          = pick_chr("Dim2"),
    dim3          = pick_chr("Dim3")
  )

  out[order(out$location, out$year), , drop = FALSE]
}


#' @noRd
.gho_get <- function(url) {
  all_data <- list()
  next_url <- url

  repeat {
    cli::cli_inform("Fetching: {.url {next_url}}")

    resp <- tryCatch(
      httr2::request(next_url) |>
        httr2::req_headers(Accept = "application/json") |>
        httr2::req_timeout(20) |>
        httr2::req_retry(
          max_tries = 3,
          backoff   = ~ min(2 ^ .x, 30)
        ) |>
        httr2::req_perform(),
      error = function(e) {
        # Reference the message via a variable so cli does not glue-interpret
        # any literal braces the error message may carry (see body parse below).
        msg <- conditionMessage(e)
        cli::cli_warn(c(
          "GHO request failed.",
          "i" = "URL: {.url {next_url}}",
          "x" = "{msg}"
        ))
        NULL
      }
    )
    if (is.null(resp)) return(NULL)

    # Body parse must also fail soft: a truncated response body (premature
    # EOF) would otherwise propagate a jsonlite parse error and break
    # R CMD check examples (CRAN-blocking).
    body <- tryCatch(
      httr2::resp_body_json(resp, simplifyVector = TRUE),
      error = function(e) {
        # Reference the message via a variable: a jsonlite parse error
        # carries literal `{`/`}` from the offending JSON, which cli would
        # otherwise try to interpret as glue expressions and re-error.
        msg <- conditionMessage(e)
        cli::cli_warn(c(
          "GHO response could not be parsed as JSON.",
          "i" = "URL: {.url {next_url}}",
          "x" = "{msg}"
        ))
        NULL
      }
    )
    if (is.null(body)) return(NULL)
    # Skip empty `value` chunks. GHO returns `value = []` (an empty list,
    # not an empty data frame) when a filter matches no rows; rbind-ing
    # it would produce a spurious 1x1 result.
    val <- body$value
    if (is.data.frame(val) && nrow(val) > 0L) {
      all_data <- c(all_data, list(val))
    }

    next_url <- body[["@odata.nextLink"]]
    if (is.null(next_url)) break
  }

  if (length(all_data) == 0L) return(tibble::tibble())
  out <- do.call(rbind, c(all_data, list(make.row.names = FALSE)))
  rownames(out) <- NULL
  tibble::as_tibble(out)
}

test_that("gho_has_data returns TRUE when data exist for the filter", {
  skip_on_cran()
  skip_if_offline()

  expect_true(gho_has_data("WHOSIS_000001", area = "FRA"))
})

test_that("gho_has_data returns FALSE when the server returns no rows", {
  skip_on_cran()
  skip_if_offline()

  # "ZZZ" is a reserved ISO3 code never assigned to a country, so the
  # GHO server returns an empty result (200 OK, value = []).
  expect_false(gho_has_data("WHOSIS_000001", area = "ZZZ"))
})

test_that("gho_has_data returns NA on request failure", {
  skip_on_cran()
  skip_if_offline()

  # A non-existent indicator code triggers HTTP 404 from the GHO server,
  # which we treat as a request failure -> NA.
  res <- suppressWarnings(gho_has_data("THIS_INDICATOR_DOES_NOT_EXIST_XYZ"))
  expect_true(is.na(res))
})

test_that("gho_has_data infers country when area given without spatial_type", {
  skip_on_cran()
  skip_if_offline()

  expect_message(
    gho_has_data("WHOSIS_000001", area = "FRA"),
    regexp = "country"
  )
})

test_that("gho_count returns an integer for valid filters", {
  skip_on_cran()
  skip_if_offline()

  n <- gho_count("WHOSIS_000001", area = "FRA")
  expect_type(n, "integer")
  expect_length(n, 1L)
  expect_gt(n, 0L)
})

test_that("gho_count returns 0L when no rows match", {
  skip_on_cran()
  skip_if_offline()

  expect_identical(gho_count("WHOSIS_000001", area = "ZZZ"), 0L)
})

test_that("gho_coverage returns the documented 4-column shape", {
  skip_on_cran()
  skip_if_offline()

  out <- gho_coverage("WHOSIS_000001", area = c("FRA", "DEU", "JPN"))
  expect_s3_class(out, "tbl_df")
  expect_named(out, c("location", "year_min", "year_max", "n_obs"))
  expect_equal(nrow(out), 3L)
  expect_setequal(out$location, c("FRA", "DEU", "JPN"))
  expect_type(out$year_min, "integer")
  expect_type(out$year_max, "integer")
  expect_type(out$n_obs, "integer")
  expect_true(all(out$year_min <= out$year_max))
  expect_true(all(out$n_obs >= 1L))
  # Sorted by location
  expect_equal(out$location, sort(out$location))
})

test_that("gho_coverage returns an empty tibble with correct columns on no match", {
  skip_on_cran()
  skip_if_offline()

  out <- gho_coverage("WHOSIS_000001", area = "ZZZ")
  expect_s3_class(out, "tbl_df")
  expect_named(out, c("location", "year_min", "year_max", "n_obs"))
  expect_equal(nrow(out), 0L)
})

# Unit tests for gho_dimensions(). Uses httr2 mocked responses so the
# tests run offline (and on CRAN).

skip_if_not_installed("httptest2")

mock_json <- function(body, status = 200L) {
  list(httr2::response(
    status_code = status,
    headers     = list(`content-type` = "application/json"),
    body        = charToRaw(body)
  ))
}

# ── Input validation (runs without any HTTP call) ────────────────────

test_that("gho_dimensions validates the indicator argument", {
  expect_error(gho_dimensions(123),         "is.character")
  expect_error(gho_dimensions(c("A", "B")), "length")
  expect_error(gho_dimensions(""),          "nzchar")
})

test_that("gho_dimensions validates the dimension argument", {
  expect_error(gho_dimensions("X", dimension = 1L),
               "is.character")
  expect_error(gho_dimensions("X", dimension = c("a", "b")),
               "length")
})

# ── Behaviour under mocked GHO responses ─────────────────────────────

test_that("gho_dimensions returns sorted unique values of the requested column", {
  # Three rows; SpatialDimType has two distinct values plus an NA. The
  # NA must be dropped and the result must be sorted.
  body <- paste0(
    '{"value":[',
    '{"SpatialDimType":"REGION","Dim1":"BTSX"},',
    '{"SpatialDimType":"COUNTRY","Dim1":"MLE"},',
    '{"SpatialDimType":"COUNTRY","Dim1":null},',
    '{"SpatialDimType":null,"Dim1":"FMLE"}',
    ']}'
  )
  httr2::with_mocked_responses(mock_json(body), {
    out <- gho_dimensions("X")  # default dimension = SpatialDimType
    expect_type(out, "character")
    expect_equal(out, c("COUNTRY", "REGION"))
  })
})

test_that("gho_dimensions picks the requested dimension column", {
  body <- paste0(
    '{"value":[',
    '{"SpatialDimType":"COUNTRY","Dim1":"BTSX"},',
    '{"SpatialDimType":"COUNTRY","Dim1":"MLE"},',
    '{"SpatialDimType":"COUNTRY","Dim1":"FMLE"}',
    ']}'
  )
  httr2::with_mocked_responses(mock_json(body), {
    out <- gho_dimensions("X", dimension = "Dim1")
    expect_equal(out, c("BTSX", "FMLE", "MLE"))  # sorted
  })
})

test_that("gho_dimensions returns empty character when dimension column is missing", {
  # SpatialDimType absent from the response — function should return
  # character(0), not error.
  body <- '{"value":[{"Dim1":"BTSX"},{"Dim1":"MLE"}]}'
  httr2::with_mocked_responses(mock_json(body), {
    out <- gho_dimensions("X")  # default dimension = SpatialDimType
    expect_type(out, "character")
    expect_length(out, 0L)
  })
})

test_that("gho_dimensions requests only the dimension column via $select", {
  # NEWS 0.8.0: gho_dimensions() no longer downloads the full
  # observation table; it sends $select=<dimension> so only one column
  # travels over the wire. Capture the outgoing URL to lock this in.
  captured <- character()
  mock_fn <- function(req) {
    captured[[length(captured) + 1L]] <<- req$url
    httr2::response(
      status_code = 200L,
      headers     = list(`content-type` = "application/json"),
      body        = charToRaw(
        '{"value":[{"Dim1":"SEX_BTSX"},{"Dim1":"SEX_MLE"}]}'
      )
    )
  }
  httr2::with_mocked_responses(mock_fn, {
    out <- gho_dimensions("X", dimension = "Dim1")
    expect_equal(out, c("SEX_BTSX", "SEX_MLE"))
  })
  expect_length(captured, 1L)
  expect_match(captured[1], "$select=Dim1", fixed = TRUE)
})

test_that("gho_dimensions returns empty character on HTTP failure", {
  httr2::with_mocked_responses(
    mock_json('{"error":"not found"}', status = 404L),
    {
      out <- suppressWarnings(gho_dimensions("MISSING"))
      expect_type(out, "character")
      expect_length(out, 0L)
    }
  )
})

# Regression tests for .gho_indicator_catalog(): a failed catalog
# fetch (gho_indicators() fails soft with an EMPTY tibble, never NULL)
# must not be cached, so the next call can retry once the network is
# back. Before the 0.8.0 fix, one offline first call poisoned the
# session cache and gho_clean()'s indicator column stayed NA for the
# rest of the session.

skip_if_not_installed("httptest2")

mock_json <- function(body, status = 200L) {
  list(httr2::response(
    status_code = status,
    headers     = list(`content-type` = "application/json"),
    body        = charToRaw(body)
  ))
}

catalog_body <- paste0(
  '{"value":[{"IndicatorCode":"X1",',
  '"IndicatorName":"Test indicator","Language":"EN"}]}'
)

test_that("a failed catalog fetch is not cached and is retried on the next call", {
  old <- .dsi_cache$gho_indicator_catalog
  on.exit(.dsi_cache$gho_indicator_catalog <- old)
  .dsi_cache$gho_indicator_catalog <- NULL

  # First call: HTTP failure -> empty catalog, which must NOT be cached.
  # 404 is not retried by req_retry, so a single mocked response is enough.
  suppressWarnings(
    httr2::with_mocked_responses(
      mock_json('{"error":"down"}', status = 404L),
      out1 <- DSIR:::.gho_indicator_catalog()
    )
  )
  expect_equal(nrow(out1), 0L)
  expect_null(.dsi_cache$gho_indicator_catalog)

  # Second call: the fetch succeeds -> cached and returned.
  httr2::with_mocked_responses(
    mock_json(catalog_body),
    out2 <- DSIR:::.gho_indicator_catalog()
  )
  expect_equal(out2$IndicatorName, "Test indicator")
  expect_equal(nrow(.dsi_cache$gho_indicator_catalog), 1L)
})

test_that("gho_clean resolves indicator names on retry after an offline first call", {
  old <- .dsi_cache$gho_indicator_catalog
  on.exit(.dsi_cache$gho_indicator_catalog <- old)
  .dsi_cache$gho_indicator_catalog <- NULL

  df <- data.frame(
    IndicatorCode = "X1",
    SpatialDim    = "PHL",
    TimeDim       = 2020L,
    Value         = "1",
    NumericValue  = 1
  )

  suppressWarnings(
    httr2::with_mocked_responses(
      mock_json('{"error":"down"}', status = 404L),
      out1 <- gho_clean(df)
    )
  )
  expect_true(is.na(out1$indicator))

  httr2::with_mocked_responses(
    mock_json(catalog_body),
    out2 <- gho_clean(df)
  )
  expect_equal(out2$indicator, "Test indicator")
})

# Tests for the unified 15-column schema produced by gho_clean() and
# sdg_clean(). Uses constructed input frames so the tests run offline.

unified_cols <- c(
  "source", "id", "indicator", "location", "iso3", "location_name",
  "year", "value", "value_num", "low", "high", "series",
  "dim1", "dim2", "dim3"
)

# ── empty / shape ────────────────────────────────────────────────────

test_that("gho_clean returns the 15-col schema on empty input", {
  out <- gho_clean(tibble::tibble())
  expect_s3_class(out, "tbl_df")
  expect_named(out, unified_cols)
  expect_equal(nrow(out), 0L)
  expect_type(out$year,      "integer")
  expect_type(out$value_num, "double")
  expect_type(out$low,       "double")
  expect_type(out$high,      "double")
  expect_type(out$source,    "character")
})

test_that("sdg_clean returns the 15-col schema on empty input", {
  out <- sdg_clean(tibble::tibble())
  expect_s3_class(out, "tbl_df")
  expect_named(out, unified_cols)
  expect_equal(nrow(out), 0L)
})

test_that("gho_clean errors on non-data-frame input", {
  expect_error(gho_clean(NULL),       "must be a data frame")
  expect_error(gho_clean(list(a=1)),  "must be a data frame")
})

test_that("sdg_clean errors on non-data-frame input", {
  expect_error(sdg_clean(NULL),       "must be a data frame")
  expect_error(sdg_clean(list(a=1)),  "must be a data frame")
})

# ── gho_clean: realistic row ─────────────────────────────────────────

test_that("gho_clean maps GHO columns to the unified schema", {
  raw <- tibble::tibble(
    IndicatorCode = c("WHOSIS_000001", "WHOSIS_000001"),
    IndicatorName = c("Life expectancy at birth (years)",
                      "Life expectancy at birth (years)"),
    SpatialDim    = c("FRA", "DEU"),
    SpatialDimType = c("COUNTRY", "COUNTRY"),
    TimeDim       = c(2019, 2019),
    Dim1          = c("BTSX", "MLE"),
    Dim2          = c(NA, NA),
    Dim3          = c(NA, NA),
    Value         = c("82.5", "78.9"),
    NumericValue  = c(82.5, 78.9),
    Low           = c(NA, NA),
    High          = c(NA, NA)
  )
  out <- gho_clean(raw)

  expect_named(out, unified_cols)
  expect_equal(nrow(out), 2L)
  expect_true(all(out$source == "gho"))
  expect_equal(out$id[1],         "WHOSIS_000001")
  expect_equal(out$indicator[1],  "Life expectancy at birth (years)")
  expect_setequal(out$location,   c("FRA", "DEU"))
  expect_equal(out$iso3,          out$location)  # both are valid ISO3
  expect_setequal(out$location_name, c("France", "Germany"))
  expect_type(out$year,           "integer")
  expect_equal(out$value_num,
               c(78.9, 82.5))                    # sorted by location: DEU, FRA
  expect_true(all(is.na(out$series)))
  expect_setequal(out$dim1,       c("BTSX", "MLE"))
})

test_that("gho_clean sets iso3 to NA for non-Member spatial codes", {
  # Region-level GHO data uses codes like 'EUR', 'AMR', 'GLOBAL'
  raw <- tibble::tibble(
    IndicatorCode = c("X", "X"),
    IndicatorName = c("Y", "Y"),
    SpatialDim    = c("EUR", "GLOBAL"),
    TimeDim       = c(2020, 2020),
    Value         = c("1", "2"),
    NumericValue  = c(1, 2)
  )
  out <- gho_clean(raw)
  expect_true(all(is.na(out$iso3)))
  expect_setequal(out$location, c("EUR", "GLOBAL"))
})

# ── gho_clean: location_name resolution ──────────────────────────────

test_that("gho_clean populates location_name for WHO Member States", {
  raw <- tibble::tibble(
    IndicatorCode = c("FAKE_001", "FAKE_001", "FAKE_001"),
    SpatialDim    = c("PHL", "FRA", "USA"),
    TimeDim       = c(2020L, 2020L, 2020L),
    Value         = c("10", "10", "10"),
    NumericValue  = c(10, 10, 10)
  )
  out <- gho_clean(raw)
  expect_false(any(is.na(out$location_name)))
  expect_true("Philippines" %in% out$location_name)
  expect_true("France"      %in% out$location_name)
})

test_that("gho_clean populates location_name for WHO regions and GLOBAL", {
  raw <- tibble::tibble(
    IndicatorCode = c("FAKE_001", "FAKE_001", "FAKE_001"),
    SpatialDim    = c("WPR", "AFR", "GLOBAL"),
    TimeDim       = c(2020L, 2020L, 2020L),
    Value         = c("10", "10", "10"),
    NumericValue  = c(10, 10, 10)
  )
  out <- gho_clean(raw)
  expect_equal(
    sort(out$location_name),
    sort(c("Africa", "Global", "Western Pacific"))
  )
})

test_that("gho_clean returns NA location_name for unknown spatial codes", {
  raw <- tibble::tibble(
    IndicatorCode = c("FAKE_001", "FAKE_001"),
    SpatialDim    = c("XYZ", "ZZ"),
    TimeDim       = c(2020L, 2020L),
    Value         = c("10", "10"),
    NumericValue  = c(10, 10)
  )
  out <- gho_clean(raw)
  expect_true(all(is.na(out$location_name)))
})

test_that("gho_clean fills missing source columns with typed NA", {
  # "NOT_IN_CATALOG" misses the helper-injected catalog, so `indicator`
  # falls back to NA via catalog-miss rather than via a missing source
  # column (the old IndicatorName route — no longer used).
  raw <- tibble::tibble(
    IndicatorCode = "NOT_IN_CATALOG",
    SpatialDim    = "FRA",
    TimeDim       = 2019L
    # Value, NumericValue, Low, High, Dim1-3 all absent
  )
  out <- gho_clean(raw)
  expect_named(out, unified_cols)
  expect_equal(nrow(out), 1L)
  expect_true(is.na(out$indicator))
  expect_true(is.na(out$value))
  expect_true(is.na(out$value_num))
  expect_true(is.na(out$low))
  expect_true(is.na(out$high))
  expect_type(out$value_num, "double")
  expect_type(out$year,      "integer")
})

# ── gho_clean: indicator column resolved from catalog ────────────────

test_that("gho_clean populates indicator from the GHO catalog", {
  # Helper pre-populates the cache with WHOSIS_000001 → "Life
  # expectancy at birth (years)". gho_clean() should pick that up
  # from IndicatorCode alone, even when the source frame does not
  # carry IndicatorName (which the data endpoint never returns).
  raw <- tibble::tibble(
    IndicatorCode = c("WHOSIS_000001", "WHOSIS_000001"),
    SpatialDim    = c("FRA", "DEU"),
    TimeDim       = c(2019L, 2019L),
    Value         = c("82.5", "81.0"),
    NumericValue  = c(82.5, 81.0)
  )
  out <- gho_clean(raw)
  expect_true(all(out$indicator == "Life expectancy at birth (years)"))
})

test_that(".gho_indicator_catalog() reads from the cache deterministically", {
  old <- .dsi_cache$gho_indicator_catalog
  on.exit(.dsi_cache$gho_indicator_catalog <- old, add = TRUE)

  marker <- tibble::tibble(
    IndicatorCode = "MARKER_XYZ",
    IndicatorName = "test marker",
    Language      = "EN"
  )
  .dsi_cache$gho_indicator_catalog <- marker

  expect_identical(.gho_indicator_catalog(), marker)
  # A second call must not replace the cached value.
  invisible(.gho_indicator_catalog())
  expect_identical(.dsi_cache$gho_indicator_catalog, marker)
})

test_that("gho_clean falls back to NA indicator when the catalog is empty", {
  old <- .dsi_cache$gho_indicator_catalog
  on.exit(.dsi_cache$gho_indicator_catalog <- old, add = TRUE)
  .dsi_cache$gho_indicator_catalog <- tibble::tibble(
    IndicatorCode = character(),
    IndicatorName = character(),
    Language      = character()
  )

  raw <- tibble::tibble(
    IndicatorCode = "WHOSIS_000001",
    SpatialDim    = "FRA",
    TimeDim       = 2019L,
    Value         = "82.5",
    NumericValue  = 82.5
  )
  out <- gho_clean(raw)
  expect_equal(nrow(out), 1L)
  expect_true(is.na(out$indicator))
})

test_that("gho_clean populates indicator from live catalog (integration)", {
  skip_on_cran()
  skip_if_offline()

  old <- .dsi_cache$gho_indicator_catalog
  on.exit(.dsi_cache$gho_indicator_catalog <- old, add = TRUE)
  rm("gho_indicator_catalog", envir = .dsi_cache)

  raw <- gho_data("WHOSIS_000001", area = "FRA")
  skip_if(nrow(raw) == 0L, "GHO returned no rows for WHOSIS_000001/FRA")

  clean <- gho_clean(raw)
  expect_true(any(!is.na(clean$indicator)))
  expect_match(
    clean$indicator[!is.na(clean$indicator)][1],
    "life expectancy",
    ignore.case = TRUE
  )
})

# ── sdg_clean: realistic row ─────────────────────────────────────────

test_that("sdg_clean maps SDG columns to the unified schema", {
  raw <- tibble::tibble(
    goal              = c("3", "3"),
    target            = c("3.4", "3.4"),
    indicator         = list("3.4.1", "3.4.1"),
    series            = c("SH_DTH_NCD", "SH_DTH_NCD"),
    seriesDescription = c("NCD mortality, both sexes",
                          "NCD mortality, both sexes"),
    geoAreaCode       = c("608", "076"),
    geoAreaName       = c("Philippines", "Brazil"),
    timePeriodStart   = c(2019, 2019),
    value             = c("17.3", "16.2"),
    lowerBound        = c("16.1", "15.0"),
    upperBound        = c("18.6", "17.5")
  )
  out <- sdg_clean(raw)

  expect_named(out, unified_cols)
  expect_equal(nrow(out), 2L)
  expect_true(all(out$source == "sdg"))
  expect_equal(out$id[1], "3.4.1")
  expect_equal(out$indicator[1], "NCD mortality, both sexes")
  expect_setequal(out$location, c("608", "076"))
  expect_setequal(out$iso3,     c("PHL", "BRA"))
  expect_setequal(out$location_name, c("Philippines", "Brazil"))
  expect_type(out$year, "integer")
  expect_type(out$value_num, "double")
  expect_type(out$low,       "double")
  expect_type(out$high,      "double")
  expect_true(all(is.na(out$dim1)))
  expect_true(all(is.na(out$dim2)))
  expect_true(all(is.na(out$dim3)))
})

test_that("sdg_clean tolerates non-numeric value entries (<0.1, aggregates)", {
  raw <- tibble::tibble(
    indicator       = list("3.4.1"),
    series          = "SH_DTH_NCD",
    geoAreaCode     = "608",
    geoAreaName     = "Philippines",
    timePeriodStart = 2019,
    value           = "<0.1",
    lowerBound      = NA_character_,
    upperBound      = NA_character_
  )
  out <- sdg_clean(raw)
  expect_equal(out$value,      "<0.1")
  expect_true(is.na(out$value_num))
})

test_that("sdg_clean iso3 is NA for region/world M49 aggregates", {
  raw <- tibble::tibble(
    indicator       = list("3.4.1"),
    geoAreaCode     = c("001", "900"),
    timePeriodStart = c(2019, 2019),
    value           = c("1", "2")
  )
  out <- sdg_clean(raw)
  expect_true(all(is.na(out$iso3)))
})

test_that("sdg_clean prefers who_countries names over SDG API names", {
  # Pass a deliberately different geoAreaName ("Philippine Islands") to
  # verify the cleaner uses who_countries$name_short ("Philippines")
  # instead of the SDG API's raw label for WHO Member States.
  raw <- tibble::tibble(
    indicator       = list("3.4.1"),
    geoAreaCode     = "608",
    geoAreaName     = "Philippine Islands",
    timePeriodStart = 2020L,
    value           = "10"
  )
  out <- sdg_clean(raw)
  expect_equal(
    out$location_name[1],
    who_countries$name_short[who_countries$iso3 == "PHL"]
  )
  expect_false(out$location_name[1] == "Philippine Islands")
})

test_that("sdg_clean falls back to geoAreaName for non-Member-State rows", {
  raw <- tibble::tibble(
    indicator       = list("3.4.1"),
    geoAreaCode     = "001",          # World aggregate
    geoAreaName     = "World",
    timePeriodStart = 2020L,
    value           = "10"
  )
  out <- sdg_clean(raw)
  expect_true(is.na(out$iso3[1]))
  expect_equal(out$location_name[1], "World")
})

test_that("sdg_clean location_name matches gho_clean for the same iso3", {
  # The point of the change: cross-API consistency. The same country
  # must have the same `location_name` whether routed via gho_clean
  # (ISO3) or sdg_clean (M49).
  gho_raw <- tibble::tibble(
    IndicatorCode = "X",
    SpatialDim    = c("PHL", "FRA"),
    TimeDim       = c(2020L, 2020L),
    Value         = c("1", "2"),
    NumericValue  = c(1, 2)
  )
  sdg_raw <- tibble::tibble(
    indicator       = list("3.4.1", "3.4.1"),
    geoAreaCode     = c("608", "250"),      # PHL, FRA
    geoAreaName     = c("Philippines", "France"),
    timePeriodStart = c(2020L, 2020L),
    value           = c("1", "2")
  )
  g <- gho_clean(gho_raw)
  s <- sdg_clean(sdg_raw)
  expect_equal(
    g$location_name[order(g$iso3)],
    s$location_name[order(s$iso3)]
  )
})

test_that("sdg_clean indicator column accepts atomic or list source", {
  # Some endpoints return `indicator` as plain chr; sdg_clean should
  # handle that too.
  raw <- tibble::tibble(
    indicator       = c("3.4.1", "3.4.1"),
    geoAreaCode     = c("608", "076"),
    timePeriodStart = c(2019, 2019),
    value           = c("1", "2")
  )
  out <- sdg_clean(raw)
  expect_equal(unique(out$id), "3.4.1")
})

# ── gho_clean + sdg_clean share schema ───────────────────────────────

test_that("gho_clean and sdg_clean produce identical column types", {
  empty_g <- gho_clean(tibble::tibble())
  empty_s <- sdg_clean(tibble::tibble())
  expect_named(empty_g, names(empty_s))
  expect_equal(vapply(empty_g, typeof, character(1)),
               vapply(empty_s, typeof, character(1)))
})

# Offline regression tests for .gho_get() and the gho_*() callers that
# wrap it. Uses httr2 mocked responses so these run on CRAN and on
# machines without internet. Each test guards an explicit fix that is
# described in NEWS.md.

skip_if_not_installed("httptest2")  # Suggests dependency

# Build a canned JSON response wrapped in a length-1 list, which is the
# form httr2::with_mocked_responses() expects for "return this single
# response to every request inside the block".
mock_json <- function(body, status = 200L) {
  list(httr2::response(
    status_code = status,
    headers     = list(`content-type` = "application/json"),
    body        = charToRaw(body)
  ))
}

test_that(".gho_get returns an empty tibble when GHO sends value = []", {
  # NEWS 0.6.0: the previous implementation produced a spurious 1x1
  # list-column tibble for an empty `value` field. We assert the fix.
  httr2::with_mocked_responses(
    mock_json('{"value": []}'),
    {
      out <- DSIR:::.gho_get("https://example.test/api/X")
      expect_s3_class(out, "tbl_df")
      expect_equal(nrow(out), 0L)
    }
  )
})

test_that(".gho_get follows @odata.nextLink and concatenates pages", {
  page1 <- paste0(
    '{"value":[',
    '{"IndicatorCode":"X","SpatialDim":"FRA","TimeDim":2019,"NumericValue":1}',
    '],"@odata.nextLink":"https://example.test/api/X?$skip=1"}'
  )
  page2 <- paste0(
    '{"value":[',
    '{"IndicatorCode":"X","SpatialDim":"DEU","TimeDim":2019,"NumericValue":2}',
    ']}'
  )
  # Two-page response — flatten the two single-element response lists
  # into one list of two responses, consumed in order.
  httr2::with_mocked_responses(
    c(mock_json(page1), mock_json(page2)),
    {
      out <- DSIR:::.gho_get("https://example.test/api/X")
      expect_equal(nrow(out), 2L)
      expect_setequal(out$SpatialDim, c("FRA", "DEU"))
    }
  )
})

test_that(".gho_get returns NULL with a warning when GHO errors out", {
  # 404 is not retried (req_retry only retries on 5xx / 429 / network),
  # so a single mocked 404 lands in the tryCatch and triggers the
  # cli_warn + NULL return that .gho_get advertises.
  httr2::with_mocked_responses(
    mock_json('{"error":"not found"}', status = 404L),
    {
      expect_warning(
        out <- DSIR:::.gho_get("https://example.test/api/X"),
        "GHO request failed"
      )
      expect_null(out)
    }
  )
})

test_that("gho_data returns an empty tibble on request failure", {
  httr2::with_mocked_responses(
    mock_json('{"error":"not found"}', status = 404L),
    {
      out <- suppressWarnings(
        gho_data("X", spatial_type = "country", area = "FRA")
      )
      expect_s3_class(out, "tbl_df")
      expect_equal(nrow(out), 0L)
    }
  )
})

test_that("gho_indicators returns an empty 3-col tibble on request failure", {
  httr2::with_mocked_responses(
    mock_json('{"error":"oops"}', status = 404L),
    {
      out <- suppressWarnings(gho_indicators("mortality"))
      expect_s3_class(out, "tbl_df")
      expect_named(out, c("IndicatorCode", "IndicatorName", "Language"))
      expect_equal(nrow(out), 0L)
    }
  )
})

test_that("gho_data() sends area filter using the OData 'in' operator", {
  # NEWS 0.5.0: switched from chained OR to `SpatialDim in (...)` to
  # avoid HTTP 400 on long area vectors. Verify the URL form.
  captured <- character()
  mock_fn <- function(req) {
    captured[[length(captured) + 1L]] <<- req$url
    httr2::response(
      status_code = 200L,
      headers     = list(`content-type` = "application/json"),
      body        = charToRaw('{"value": []}')
    )
  }
  httr2::with_mocked_responses(mock_fn, {
    gho_data("X", spatial_type = "country",
             area = c("FRA", "DEU", "JPN"))
  })
  expect_length(captured, 1L)
  decoded <- utils::URLdecode(captured[1])
  expect_match(decoded, "SpatialDim in ('FRA','DEU','JPN')", fixed = TRUE)
})

test_that("gho_data() sends dim1/dim2/dim3 filters server-side", {
  # NEWS 0.8.0: dimension breakdowns (sex, age group, ...) can be
  # filtered server-side instead of downloading the full table.
  captured <- character()
  mock_fn <- function(req) {
    captured[[length(captured) + 1L]] <<- req$url
    httr2::response(
      status_code = 200L,
      headers     = list(`content-type` = "application/json"),
      body        = charToRaw('{"value": []}')
    )
  }
  httr2::with_mocked_responses(mock_fn, {
    gho_data("X", spatial_type = "country",
             dim1 = "SEX_BTSX", dim3 = c("A", "B"))
  })
  expect_length(captured, 1L)
  decoded <- utils::URLdecode(captured[1])
  expect_match(decoded, "Dim1 in ('SEX_BTSX')", fixed = TRUE)
  expect_match(decoded, "Dim3 in ('A','B')", fixed = TRUE)
})

test_that("gho_has_data, gho_count, and gho_coverage pass dim filters through", {
  # All four public wrappers route through .gho_build_url; deleting the
  # dim passthrough in any one of them must fail a test, not just the
  # gho_data() one above.
  captured <- character()
  mock_fn <- function(req) {
    captured[[length(captured) + 1L]] <<- req$url
    httr2::response(
      status_code = 200L,
      headers     = list(`content-type` = "application/json"),
      body        = charToRaw('{"@odata.count": 0, "value": []}')
    )
  }
  httr2::with_mocked_responses(mock_fn, {
    gho_has_data("X", spatial_type = "country", dim1 = "SEX_FMLE")
    gho_count("X", spatial_type = "country", dim2 = "AGEGROUP_YEARS15-19")
    gho_coverage("X", dim3 = c("URB", "RUR"))
  })
  expect_length(captured, 3L)
  decoded <- vapply(captured, utils::URLdecode, character(1),
                    USE.NAMES = FALSE)
  expect_match(decoded[1], "Dim1 in ('SEX_FMLE')", fixed = TRUE)
  expect_match(decoded[2], "Dim2 in ('AGEGROUP_YEARS15-19')", fixed = TRUE)
  expect_match(decoded[3], "Dim3 in ('URB','RUR')", fixed = TRUE)
})

test_that(".gho_get returns NULL with a warning on a malformed JSON body", {
  # NEWS 0.7.0: a truncated upstream body (premature EOF) reaches
  # resp_body_json() as unparseable JSON. The body-parse tryCatch must
  # downgrade the jsonlite error to a warning + NULL, the same way an
  # HTTP failure is handled. Without it the parse error would propagate
  # and break R CMD check examples.
  httr2::with_mocked_responses(
    mock_json('{"value": [{"SpatialDim":"FRA"'),  # truncated, unparseable
    {
      expect_warning(
        out <- DSIR:::.gho_get("https://example.test/api/X"),
        "could not be parsed as JSON"
      )
      expect_null(out)
    }
  )
})

test_that("gho_count returns NA on a malformed JSON body", {
  # NEWS 0.7.0: gho_count() has its own HTTP call site (it needs
  # @odata.count, not value), so it carries its own body-parse
  # tryCatch. A truncated body must yield NA_integer_, not an error.
  httr2::with_mocked_responses(
    mock_json('{"@odata.count": 4'),  # truncated, unparseable
    {
      expect_warning(
        n <- gho_count("X", spatial_type = "country", area = "FRA"),
        "could not be parsed as JSON"
      )
      expect_identical(n, NA_integer_)
    }
  )
})

test_that("gho_data handles long area vectors via in operator", {
  skip_on_cran()
  skip_if_offline()
  
  result <- gho_data("MDG_0000000026", 
                     spatial_type = "country",
                     area = wpro_cty)
  expect_s3_class(result, "tbl_df")
  expect_gt(nrow(result), 0)
  expect_true(all(result$SpatialDim %in% wpro_cty))
})

test_that("gho_data validates area input", {
  expect_error(gho_data("X", spatial_type = "country", area = c("PHL", NA)))
  expect_error(gho_data("X", spatial_type = "country", area = c("PHL", "")))
  expect_error(gho_data("X", spatial_type = "country", area = 123))
})

test_that("gho_data infers country when area given without spatial_type", {
  skip_on_cran()
  skip_if_offline()

  expect_message(
    gho_data("MDG_0000000026", area = "PHL"),
    regexp = "country"
  )
})

# Filter extracted from a built URL — decoded so the assertions read clearly.
gho_indicators_filter <- function(url) {
  utils::URLdecode(sub(".*\\$filter=", "", url))
}

test_that(".gho_build_url adds Dim1/Dim2/Dim3 'in' filters", {
  url <- .gho_build_url("X", dim1 = c("SEX_MLE", "SEX_FMLE"),
                        dim2 = "AGEGROUP_YEARS15-49")
  expect_equal(
    gho_indicators_filter(url),
    "Dim1 in ('SEX_MLE','SEX_FMLE') and Dim2 in ('AGEGROUP_YEARS15-49')"
  )

  url <- .gho_build_url("X", dim3 = "URB")
  expect_equal(gho_indicators_filter(url), "Dim3 in ('URB')")
})

test_that(".gho_build_url combines dim filters with spatial and year filters", {
  url <- .gho_build_url("X", spatial_type = "country", area = "PHL",
                        year_from = 2015, dim1 = "SEX_BTSX")
  expect_equal(
    gho_indicators_filter(url),
    paste0("SpatialDimType eq 'COUNTRY' and SpatialDim in ('PHL') ",
           "and TimeDim ge 2015 and Dim1 in ('SEX_BTSX')")
  )
})

test_that(".gho_build_url escapes single quotes in dim values", {
  url <- .gho_build_url("X", dim1 = "IT'S")
  expect_equal(gho_indicators_filter(url), "Dim1 in ('IT''S')")
})

test_that(".gho_build_url validates dim arguments", {
  expect_error(.gho_build_url("X", dim1 = 1L),           "is.character")
  expect_error(.gho_build_url("X", dim1 = c("A", NA)),   "anyNA")
  expect_error(.gho_build_url("X", dim2 = ""),           "nzchar")
  expect_error(.gho_build_url("X", dim3 = character(0)), "length")
})

test_that("gho_indicators builds a single-term filter (backward compatible)", {
  url <- .gho_indicators_build_url("mortality")
  expect_equal(
    gho_indicators_filter(url),
    "contains(tolower(IndicatorName),'mortality')"
  )
})

test_that("gho_indicators splits a string on whitespace into AND-ed terms", {
  url <- .gho_indicators_build_url("child mortality")
  expect_equal(
    gho_indicators_filter(url),
    "contains(tolower(IndicatorName),'child') and contains(tolower(IndicatorName),'mortality')"
  )
})

test_that("gho_indicators uses a character vector as terms directly", {
  url <- .gho_indicators_build_url(c("child", "mortality"))
  expect_equal(
    gho_indicators_filter(url),
    "contains(tolower(IndicatorName),'child') and contains(tolower(IndicatorName),'mortality')"
  )
})

test_that("gho_indicators escapes single quotes in search terms", {
  url <- .gho_indicators_build_url("alzheimer's")
  expect_equal(
    gho_indicators_filter(url),
    "contains(tolower(IndicatorName),'alzheimer''s')"
  )
})

# =============================================================================
# R/clean_schema.R
#
# Internal helpers for the unified 15-column schema produced by
# gho_clean() and sdg_clean() (and consumed by bind_indicators()).
# Centralises column names and types so all three functions stay in sync.
# =============================================================================


#' Column names and types for the unified cleaned-indicator schema.
#'
#' Names are returned in canonical output order. Types use short codes —
#' `"chr"` for character, `"int"` for integer, `"num"` for numeric — so a
#' single small lookup table covers both the empty-tibble template
#' (`.dsi_empty_clean()`) and the typed NA filler (`.fill_na()`).
#'
#' @noRd
.dsi_clean_schema <- function() {
  c(
    source        = "chr",
    id            = "chr",
    indicator     = "chr",
    location      = "chr",
    iso3          = "chr",
    location_name = "chr",
    year          = "int",
    value         = "chr",
    value_num     = "num",
    low           = "num",
    high          = "num",
    series        = "chr",
    dim1          = "chr",
    dim2          = "chr",
    dim3          = "chr"
  )
}


#' A 0-row tibble with the cleaned-indicator schema and correct
#' column types. Used as the fallback for empty input and for service
#' failure.
#'
#' @noRd
.dsi_empty_clean <- function() {
  schema <- .dsi_clean_schema()
  cols <- lapply(schema, function(t) .fill_na(0L, t))
  names(cols) <- names(schema)
  tibble::as_tibble(cols)
}


#' Typed NA filler of length n.
#'
#' @noRd
.fill_na <- function(n, type) {
  switch(
    type,
    chr = rep(NA_character_, n),
    int = rep(NA_integer_,   n),
    num = rep(NA_real_,      n),
    cli::cli_abort("Unknown column type: {.val {type}}.")
  )
}

#' Geometric Mean
#'
#' Computes the geometric mean of a numeric vector, with optional
#' weights. Useful for aggregating multiplicative quantities such as
#' ratio-based health indicators (e.g. UHC service-coverage tracers,
#' where the composite index is the geometric mean of component
#' coverage values).
#'
#' Pass `w` to compute a weighted geometric mean, defined as
#' `exp(weighted.mean(log(x), w))`.
#'
#' @param x A numeric vector. Zeros produce a result of `0`.
#'   Negative values produce `NaN` with a warning, since the
#'   geometric mean is undefined for negative numbers.
#' @param w Optional numeric vector of weights, the same length as
#'   `x`. Must be non-negative. If `NULL` (default), the unweighted
#'   geometric mean is returned.
#' @param na.rm Logical. Should missing values in `x` (and `w`, if
#'   provided) be removed before computation? Default `TRUE`.
#'
#' @return A numeric scalar. Returns `NA_real_` when the input is
#'   empty, when it is entirely `NA`, or when `na.rm = FALSE` and
#'   any element is `NA`. Returns `NaN` with a warning when `x`
#'   contains negative values, or when all weights are zero.
#' @export
#'
#' @examples
#' # Unweighted
#' geomean(c(1, 4, 16))                # 4
#' geomean(c(0.6, 0.8, 0.95))          # ~0.772 — typical UHC tracer aggregation
#' geomean(c(1, NA, 4))                # 2
#' geomean(c(1, NA, 4), na.rm = FALSE) # NA_real_
#' geomean(c(1, 0, 4))                 # 0
#'
#' # Weighted
#' geomean(c(1, 4, 16), w = c(1, 1, 1))     # 4 (equal weights = unweighted)
#' geomean(c(1, 4, 16), w = c(1, 2, 1))     # weighted toward 4
#' geomean(c(0.6, 0.8, 0.95), w = c(2, 1, 1))

geomean <- function(x, w = NULL, na.rm = TRUE) {
  # ── Input validation ──────────────────────────────────────────
  if (is.logical(x)) {
    x <- as.numeric(x)
  }
  if (!is.numeric(x)) {
    cli::cli_abort("{.arg x} must be numeric or logical, not {.cls {class(x)}}.")
  }
  if (!isTRUE(na.rm) && !isFALSE(na.rm)) {
    cli::cli_abort("{.arg na.rm} must be a single logical value.")
  }
  
  weighted <- !is.null(w)
  
  if (weighted) {
    if (!is.numeric(w)) {
      cli::cli_abort("{.arg w} must be numeric, not {.cls {class(w)}}.")
    }
    if (length(x) != length(w)) {
      cli::cli_abort(
        "{.arg x} and {.arg w} must have the same length \\
        ({length(x)} vs {length(w)})."
      )
    }
  }
  
  # ── NA filtering ──────────────────────────────────────────────
  if (na.rm) {
    ok <- if (weighted) !is.na(x) & !is.na(w) else !is.na(x)
    x <- x[ok]
    if (weighted) w <- w[ok]
  }
  
  # ── Empty / remaining NA handling ─────────────────────────────
  if (length(x) == 0L) return(NA_real_)
  if (anyNA(x)) return(NA_real_)
  if (weighted && anyNA(w)) return(NA_real_)
  
  # ── Weight-specific checks ────────────────────────────────────
  if (weighted) {
    if (any(w < 0)) {
      cli::cli_abort("{.arg w} must be non-negative.")
    }
    if (sum(w) == 0) {
      cli::cli_warn(
        "All weights are zero; weighted geometric mean is undefined."
      )
      return(NaN)
    }
  }
  
  # ── Domain checks on x ────────────────────────────────────────
  if (any(x < 0)) {
    cli::cli_warn(
      "Negative values in {.arg x}; geometric mean is undefined."
    )
    return(NaN)
  }
  if (any(x == 0)) return(0)
  
  # ── Compute ───────────────────────────────────────────────────
  if (weighted) {
    exp(stats::weighted.mean(log(x), w))
  } else {
    exp(mean(log(x)))
  }
}

test_that("geomean computes correctly on positive values", {
  expect_equal(geomean(c(1, 4, 16)), 4)
  expect_equal(geomean(c(2, 8)), 4)
  expect_equal(geomean(c(0.6, 0.8, 0.95)),
               exp(mean(log(c(0.6, 0.8, 0.95)))))
})

test_that("geomean handles NA via na.rm", {
  expect_equal(geomean(c(1, NA, 4)), 2)
  expect_equal(geomean(c(1, NA, 4), na.rm = FALSE), NA_real_)
  expect_equal(geomean(c(NA, NA)), NA_real_)
})

test_that("geomean handles zeros and negatives", {
  expect_equal(geomean(c(1, 0, 4)), 0)
  expect_warning(out <- geomean(c(-1, 2)), "[Nn]egative")
  expect_true(is.nan(out))
})

test_that("geomean handles empty input", {
  expect_equal(geomean(numeric(0)), NA_real_)
})

test_that("geomean rejects non-numeric input", {
  expect_error(geomean("a"), "must be numeric")
  expect_error(geomean(factor(c("a", "b"))), "must be numeric")
})

test_that("geomean coerces logical input", {
  # TRUE = 1, FALSE = 0; geomean of (1, 0) = 0 due to zero
  expect_equal(geomean(c(TRUE, FALSE)), 0)
  # All TRUE = all 1s; geomean = 1
  expect_equal(geomean(c(TRUE, TRUE, TRUE)), 1)
  # NA_logical coerced to NA_real_, removed by na.rm
  expect_equal(geomean(c(TRUE, NA, TRUE)), 1)
})

test_that("geomean handles all-NA input", {
  expect_equal(geomean(c(NA_real_, NA_real_)), NA_real_)
  expect_equal(geomean(c(NA, NA)), NA_real_)  # logical NA, coerced
})

test_that("geomean validates na.rm", {
  expect_error(geomean(c(1, 2), na.rm = NA), "single logical")
  expect_error(geomean(c(1, 2), na.rm = c(TRUE, FALSE)), "single logical")
})

test_that("geomean computes weighted geometric mean", {
  expect_equal(geomean(c(1, 4, 16), w = c(1, 1, 1)), 4)
  expect_equal(geomean(c(1, 100), w = c(1, 0)), 1)
})

test_that("geomean validates weight inputs", {
  expect_error(geomean(c(1, 2), w = c(1, -1)))
  expect_error(geomean(c(1, 2), w = c(1, 2, 3)))
})

test_that("geomean handles NA in weights", {
  expect_equal(
    geomean(c(1, 2, 4), w = c(1, NA, 1)),
    sqrt(1 * 4)
  )
})

# Pre-populate the GHO indicator-catalog cache used by gho_clean() so
# unit tests don't trigger a network call when looking up indicator
# names. Tests that need a different cache state (e.g. empty catalog,
# unknown codes) can override .dsi_cache$gho_indicator_catalog locally
# and restore via on.exit().
#
# Helper files are sourced into the package namespace, so `.dsi_cache`
# is reachable directly without a `DSIR:::` prefix.
.dsi_cache$gho_indicator_catalog <- tibble::tibble(
  IndicatorCode = c("WHOSIS_000001"),
  IndicatorName = c("Life expectancy at birth (years)"),
  Language      = c("EN")
)

test_that("who_countries has the expected shape and columns", {
  expect_s3_class(who_countries, "tbl_df")
  expect_equal(nrow(who_countries), 194L)
  expect_named(who_countries,
               c("iso3", "iso2", "m49_code",
                 "name_official", "name_short",
                 "who_region", "is_pic"))
})

test_that("who_countries identifiers are well-formed", {
  expect_true(all(nchar(who_countries$iso3) == 3L))
  expect_true(all(who_countries$iso3 == toupper(who_countries$iso3)))
  expect_true(all(nchar(who_countries$iso2) == 2L))
  expect_true(all(nchar(who_countries$m49_code) == 3L))
  # m49_code is stored as 3-char zero-padded character.
  expect_type(who_countries$m49_code, "character")
})

test_that("who_countries has no missing identifiers", {
  expect_false(anyNA(who_countries$iso3))
  expect_false(anyNA(who_countries$iso2))
  expect_false(anyNA(who_countries$m49_code))
  expect_false(anyNA(who_countries$name_official))
  expect_false(anyNA(who_countries$name_short))
  expect_false(anyNA(who_countries$who_region))
  expect_false(anyNA(who_countries$is_pic))
})

test_that("who_countries identifiers are unique", {
  expect_equal(anyDuplicated(who_countries$iso3),     0L)
  expect_equal(anyDuplicated(who_countries$iso2),     0L)
  expect_equal(anyDuplicated(who_countries$m49_code), 0L)
})

test_that("who_countries who_region values are from the WHO set", {
  expect_setequal(unique(who_countries$who_region),
                  c("AFR", "AMR", "SEAR", "EUR", "EMR", "WPR"))
})

test_that("who_countries is_pic is logical and only TRUE in WPR", {
  expect_type(who_countries$is_pic, "logical")
  expect_equal(sum(who_countries$is_pic), 14L)
  pic_rows <- who_countries[who_countries$is_pic, ]
  expect_true(all(pic_rows$who_region == "WPR"))
})

test_that("who_countries regional row counts match documentation", {
  tab <- table(who_countries$who_region)
  expect_equal(unname(tab["AFR"]),  47L)
  expect_equal(unname(tab["AMR"]),  35L)
  expect_equal(unname(tab["SEAR"]), 10L)
  expect_equal(unname(tab["EUR"]),  53L)
  expect_equal(unname(tab["EMR"]),  21L)
  expect_equal(unname(tab["WPR"]),  28L)
})

test_that("who_countries reflects WHO EB156 reassignment of Indonesia to WPR", {
  idn <- who_countries[who_countries$iso3 == "IDN", ]
  expect_equal(idn$who_region, "WPR")
})

test_that("wpro_cty is a non-empty character vector of ISO3 codes", {
  expect_type(wpro_cty, "character")
  expect_gt(length(wpro_cty), 0)
  expect_true(all(nchar(wpro_cty) == 3))
  expect_true(all(wpro_cty == toupper(wpro_cty)))
})

# Each regional ISO3 vector should equal the corresponding slice of
# who_countries$iso3 (sorted), since they are all derived from it.

expect_region_vector <- function(vec, region, expected_length) {
  expect_type(vec, "character")
  expect_length(vec, expected_length)
  expect_true(all(nchar(vec) == 3L))
  expect_true(all(vec == toupper(vec)))
  expect_false(anyDuplicated(vec) > 0L)
  expect_setequal(
    vec,
    who_countries$iso3[who_countries$who_region == region]
  )
  expect_equal(vec, sort(vec))
}

test_that("afro_cty matches the AFR slice of who_countries", {
  expect_region_vector(afro_cty, "AFR", 47L)
})

test_that("amro_cty matches the AMR slice of who_countries", {
  expect_region_vector(amro_cty, "AMR", 35L)
})

test_that("searo_cty matches the SEAR slice of who_countries", {
  expect_region_vector(searo_cty, "SEAR", 10L)
  # Indonesia moved to WPR in May 2025, so SEAR must NOT contain IDN.
  expect_false("IDN" %in% searo_cty)
})

test_that("euro_cty matches the EUR slice of who_countries", {
  expect_region_vector(euro_cty, "EUR", 53L)
})

test_that("emro_cty matches the EMR slice of who_countries", {
  expect_region_vector(emro_cty, "EMR", 21L)
})

test_that("wpro_cty matches the WPR slice of who_countries", {
  expect_region_vector(wpro_cty, "WPR", 28L)
  expect_true("IDN" %in% wpro_cty)
})

test_that("pic_cty is the is_pic subset of WPR", {
  expect_type(pic_cty, "character")
  expect_length(pic_cty, 14L)
  expect_true(all(pic_cty %in% wpro_cty))
  expect_setequal(
    pic_cty,
    who_countries$iso3[who_countries$is_pic]
  )
})

---
title: "DSIR"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{DSIR}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment  = "#>", 
  eval     = TRUE
)
```

```{r setup}
library(DSIR)
library(dplyr)
library(ggplot2)
```

<img src="../man/figures/logo.jpg" align="right" height="120" alt="DSIR logo" />

DSIR is a small R package for global health data work. It consists of 
WHO Member State metadata, lightweight clients for the GHO and UN SDG 
APIs, and reusable WHO-style `ggplot2` and `flextable` themes. DSIR 
is designed for health professionals, WHO staff, and global health 
researchers — the kind of users who do the same routine tasks every day.

This vignette walks through the typical workflow: looking up countries, 
fetching data from GHO and SDG, cleaning the raw response, and 
producing publication-style charts and tables.

## WHO Member State metadata

The `who_countries` tibble lists all 194 WHO Member States with their 
ISO3, ISO2, UN M49 codes, official names, short names, and WHO region. 
For Western Pacific countries, an extra column `is_pic` identifies the 
14 Pacific Island Countries.

```{r}
who_countries
```

For convenience, DSIR offers pre-defined vectors of ISO3 codes for 
each WHO region.

```{r}
wpro_cty
length(wpro_cty)   # 28 Member States in WPR (since May 2025)
```

The `is_pic` flag is useful because Pacific Island Countries are often 
analysed as a group, given their distinct demographic and geographic 
profiles.

```{r}
who_countries |>
  filter(is_pic) |>
  select(iso3, name_short)
```

When you have a vector of ISO3 codes and need to know which WHO 
region each belongs to, `iso3_to_region()` provides the lookup. 
It is vectorised and returns `NA` for codes that do not match a 
WHO Member State.

```{r}
iso3_to_region(c("PHL", "FRA", "ZAF", "USA", "XYZ"))
# "WPR" "EUR" "AFR" "AMR" NA
```

This is convenient when joining external datasets (which often arrive
keyed only by ISO3) to the WHO regional structure.

The companion helper `iso3_to_m49()` converts ISO3 codes to UN M49
numeric codes — useful because the WHO GHO API is keyed by ISO3
(`"PHL"`) while the UN SDG API is keyed by M49 (`"608"`). The M49
values are returned as three-character zero-padded strings, exactly
as stored in `who_countries$m49_code`.

```{r}
iso3_to_m49(c("PHL", "FRA", "JPN"))
# "608" "250" "392"

# Case-insensitive; non-Member areas return NA
iso3_to_m49(c("phl", "PRI"))
# "608" NA
```

In practice you can usually skip the explicit conversion: `sdg_data()`
and `sdg_coverage()` accept ISO3 codes for their `area` argument and
do the lookup internally (see the SDG section below).

## Checking availability before fetching

GHO has thousands of indicators, but any single indicator may not cover
the countries or years you need. Before issuing a full download with
`gho_data()`, three lightweight helpers let you ask the server what is
available without transferring any observations.

`gho_has_data()` is a quick yes / no for a given indicator and filter —
useful when screening a list of candidate indicators.

```{r}
# Does WHO have life-expectancy data for France?
gho_has_data("WHOSIS_000001", area = "FRA")
# TRUE

# Bulk-screen several indicators at once
inds <- c("WHOSIS_000001", "NCDMORT3070", "MDG_0000000026")
vapply(inds, gho_has_data, logical(1), area = "PHL")
```

It returns `TRUE`, `FALSE`, or `NA` (for request failures, including a
non-existent indicator code — GHO returns HTTP 404 in that case).

`gho_count()` returns the number of rows the same filter would produce,
which is useful for sizing a download.

```{r}
gho_count("WHOSIS_000001", area = wpro_cty)
```

`gho_coverage()` summarises year coverage and observation counts per
country. The payload is small because only `SpatialDim` and `TimeDim`
are requested from the server.

```{r}
gho_coverage("WHOSIS_000001", area = c("FRA", "DEU", "JPN"))
#>   location year_min year_max n_obs
#> 1 DEU          2000     2021    66
#> 2 FRA          2000     2021    66
#> 3 JPN          2000     2021    66
```

## Fetching indicator data from GHO

To fetch indicators from GHO, the typical workflow is three steps:
search for the indicator code, fetch the data, then clean the response.
The `area` argument accepts a long ISO3 vector, so a whole region can
be pulled in one call.

### Step 1: Search for an indicator

```{r}
gho_indicators("UHC") |> head()
```

Pick an `IndicatorCode` from the result — this is the value you pass 
to `gho_data()` in the next step.

### Step 2: Fetch the data

```{r}
uhc <- gho_data(
  indicator    = "UHC_INDEX_REPORTED",
  spatial_type = "country",
  area         = wpro_cty,
  year_from    = 2015
)

uhc |> glimpse()
```

Note that `area` accepts long ISO3 vectors — here we fetch all 28 WPR 
countries in one call.

### Step 3: Clean the raw response

`gho_clean()` produces the **unified DSIR cleaned-indicator schema** —
the same 15-column shape as `sdg_clean()`. Columns include `source`
(`"gho"`), `id`, `indicator`, `location`, `iso3`, `location_name`
(empty for GHO), `year`, `value`, `value_num`, `low`, `high`, `series`
(empty for GHO), and the three optional GHO dimensions
`dim1`–`dim3`. Columns absent from the raw response are filled with
typed `NA`.

```{r}
uhc_clean <- gho_clean(uhc)
uhc_clean
```

## Aggregating indicators with geomean()

Some health indicators are constructed as the geometric mean of 
component values rather than the arithmetic mean. The UHC Service 
Coverage Index, for example, aggregates 14 tracer indicators using 
nested geometric means. DSIR provides `geomean()` for this:

```{r}
# Unweighted geometric mean
geomean(c(0.6, 0.8, 0.95))
#> 0.7720589

# With optional weights — useful when tracers have different 
# methodological importance
geomean(c(0.6, 0.8, 0.95), w = c(2, 1, 1))
```

`geomean()` handles missing values, zeros, and negative values 
sensibly — see `?geomean` for details. It is a small helper, but 
it removes a common source of bugs when re-implementing index 
calculations from indicator components.

## Plotting with theme_dsi() and theme_dsi_facet()

DSIR provides two paired `ggplot2` themes tuned for WHO-style charts — 
clean panels, modest grids, and a consistent accent colour. Use them as 
drop-in replacements for `theme_minimal()` and `theme_bw()` respectively 
whenever a chart is heading into a WHO deliverable.

The rule of thumb is simple: **single-panel plots use `theme_dsi()`, 
faceted plots use `theme_dsi_facet()`**. The two share typography, title 
treatment, and legend handling, but differ in how they frame the data — 
the facet variant adds panel borders, light strip backgrounds, and 
breathing room between panels, all of which would look heavy on a 
single-panel chart.

### Single panel: `theme_dsi()`

`theme_dsi()` keeps the chart chrome minimal — a half-frame axis, light grid lines, and the WHO-blue accent on the axis line. By default the grid runs in both directions; pass `grid = "y"` for the minimalist horizontal-only look.

```{r, fig.width = 7, fig.height = 4}
uhc_clean |>
  filter(iso3 %in% c("AUS", "CHN", "PHL", "FJI")) |>
  left_join(who_countries, by = "iso3") |>
  ggplot(aes(x = year, y = value_num, group = iso3, color = name_short)) +
  geom_line(linewidth = .8) +
  geom_point(size = 1.8) +
  theme_dsi() +
  labs(
    title    = "UHC Service Coverage Index, selected WPR Member States",
    subtitle = "2015 onwards",
    x = NULL, y = "SCI", color = NULL
  )
```

For bar charts, pair `theme_dsi()` with `scale_y_dsi_col()` (or `scale_x_dsi_col()` when `value` is mapped to `x`) — these are thin wrappers around `scale_*_continuous()` that remove the lower axis expansion, so columns sit flush with the axis instead of floating above it.

```{r, fig.width = 7, fig.height = 4}
uhc_clean |>
  filter(year == max(year)) |>
  left_join(who_countries, by = "iso3") |>
  arrange(desc(value_num)) |>
  head(10) |>
  ggplot(aes(reorder(name_short, value_num), value_num)) +
  geom_col(fill = "#0093D5") +
  coord_flip() +
  scale_y_dsi_col() +
  theme_dsi(grid = "x") +
  labs(
    title    = "UHC Service Coverage Index, top 10 WPR Member States",
    subtitle = "Latest available year",
    x = NULL, y = "SCI"
  )
```

### Faceted: `theme_dsi_facet()`

When the same chart is split across many small panels, the half-frame 
look becomes visually noisy — the accent-blue axis line repeats across 
every facet. `theme_dsi_facet()` switches to a full panel border, adds a 
light grey strip background to clearly mark each facet's label, and 
introduces panel spacing so adjacent panels don't run together.

```{r, fig.width = 8, fig.height = 5}
uhc_clean |>
  left_join(who_countries, by = "iso3") |>
  filter(is_pic) |>
  ggplot(aes(x = year, y = value_num)) +
  geom_line(color = "#0093D5", linewidth = 0.8) +
  geom_point(color = "#0093D5", size = 1.5) +
  facet_wrap(~ name_short, ncol = 4) +
  theme_dsi_facet() +
  labs(
    title    = "UHC Service Coverage Index, Pacific Island Countries",
    subtitle = "Each panel shows one country's trajectory",
    x = NULL, y = "SCI"
  )
```

The `strip_fill` argument lets you change the strip background colour 
for emphasis — for example, a light-blue tone derived from the WHO 
accent for a deliverable where the strips themselves carry meaning:

```{r, fig.width = 8, fig.height = 5}
uhc_clean |>
  left_join(who_countries, by = "iso3") |>
  filter(is_pic) |>
  ggplot(aes(x = year, y = value_num)) +
  geom_line(color = "#0093D5", linewidth = 0.8) +
  facet_wrap(~ name_short, ncol = 4) +
  theme_dsi_facet(strip_fill = "#E5F4FB") +
  labs(title = "UHC SCI, PIC — with custom strip colour",
       x = NULL, y = "SCI")
```

## Tables with dsi_flextable_defaults()

`dsi_flextable_defaults()` sets WHO-style defaults for `flextable` 
globally — booktabs theme, bold headers, modest padding. Call it once 
near the top of your report and every subsequent `flextable()` picks 
up the formatting.

```{r}
library(flextable)
dsi_flextable_defaults(font_family = "Geogria")

uhc_clean |>
  filter(year == max(year)) |>
  left_join(who_countries, by = "iso3") |>
  select(name_short, value_num) |>
  arrange(desc(value_num)) |>
  flextable() |>
  set_table_properties("autofit", width = .6) %>%
  set_caption("UHC SCI in WPR, latest year")
```

## Working with SDG indicators

`sdg_data()` and `sdg_clean()` follow the same fetch-then-tidy
pattern as their GHO counterparts. The main differences are that
indicator codes use the dotted SDG format (e.g. `"3.4.1"`) and that
`value`, `low`, and `high` are kept as character — the SDG API
returns non-numeric entries (`"<0.1"`, aggregate notes) for some
rows, so coerce with `as.numeric()` only when you are ready to drop
them.

`sdg_indicators()` accepts an optional `search` argument with the
same behaviour as `gho_indicators()` — multiple keywords are AND-ed
together and matched case-insensitively against the indicator
description. The filter runs client-side because the UN SDG
indicator list is short (~250 rows) and the endpoint is not OData.

```{r}
# All indicators that mention both mortality and cancer
sdg_indicators("mortality cancer")

# Same as above, but with explicit terms (allows whitespace inside a term)
sdg_indicators(c("maternal", "mortality"))
```

The `area` argument of `sdg_data()` and `sdg_coverage()` accepts
either ISO3 codes (converted internally via `iso3_to_m49()`) or UN
M49 numeric codes — so DSIR's regional vectors (`wpro_cty`,
`afro_cty`, etc.) work directly, the same way they do with the GHO
client. Do not mix the two formats in a single call.

```{r}
# ISO3 — regional vector passed straight through
sdg <- sdg_data(
  indicator = "3.4.1",
  area      = wpro_cty
)
sdg |> glimpse()

# M49 also works (e.g. when copy-pasting codes from sdg_areas())
sdg_data("3.4.1", area = c("608", "250"))
```

```{r}
sdg_clean(sdg)
```

`sdg_clean()` produces the same 15-column schema as `gho_clean()`,
so the two outputs can be combined directly with `bind_indicators()`.
SDG rows populate the `series` column (and the `iso3` column via
[`m49_to_iso3()`] for Member States), while leaving the GHO-only
`dim1`–`dim3` columns as `NA`.

### Combining GHO and SDG with bind_indicators()

When an analysis pulls indicators from both sources, `bind_indicators()`
stacks any number of cleaned tibbles into one. The `source` column
(`"gho"` / `"sdg"`) lets you filter or facet by origin without
remembering which frame came from which API.

```{r}
# Two indicators on the same topic from different APIs:
#   GHO NCDMORT3070 (probability of premature NCD mortality)
#   SDG 3.4.1       (mortality rate from NCDs)
gho_ncd <- gho_data("NCDMORT3070", area = wpro_cty) |> gho_clean()
sdg_ncd <- sdg_data("3.4.1",        area = wpro_cty) |> sdg_clean()
bind_indicators(gho_ncd, sdg_ncd) |> glimpse()
```

### Exploring series with sdg_coverage()

A single SDG indicator often contains several **series** — for
example different vaccines, sex strata, or causes of death — each
with its own country and year coverage. Indicator `"3.b.1"`
(vaccine coverage) is a clear case: it is published as four
separate series (DTP3, MCV2, PCV3, HPV), and the year coverage of
the newer vaccines is much shorter than that of DTP3.

`sdg_coverage()` summarises the year range and observation count
per `(location, series)`, so you can inspect what series exist and
how each is covered before deciding which one to analyse.

```{r}
sdg_coverage("3.b.1", area = c("156", "608"))
#>   location series      year_min year_max n_obs
#> 1 156      SH_ACS_DTP3     2000     2023    24
#> 2 156      SH_ACS_HPV      2018     2023     6
#> 3 156      SH_ACS_MCV2     2000     2023    24
#> 4 156      SH_ACS_PCV3     2017     2023     7
#> 5 608      SH_ACS_DTP3     2000     2023    24
#> 6 608      SH_ACS_HPV      2017     2023     7
#> 7 608      SH_ACS_MCV2     2000     2023    24
#> 8 608      SH_ACS_PCV3     2014     2023    10
```

Note that DSIR intentionally does *not* provide SDG analogues of
`gho_has_data()` and `gho_count()`. SDG data is generally complete
enough that those screening helpers add little value — the more
useful pre-analysis question for SDG is "which series are
available?", which is what `sdg_coverage()` answers.

## Where to next

- Source code lives at <https://github.com/shanlong-who/DSIR>.
- Bug reports, feature requests, and pull requests are all welcome — 
  please file them on the GitHub issue tracker.

---
title: "Visualizing indicators"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Visualizing indicators}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse   = TRUE,
  comment    = "#>",
  fig.width  = 6.5,
  fig.height = 4,
  fig.align  = "center"
)
```

```{r setup, message = FALSE}
library(DSIR)
library(dplyr)
library(ggplot2)
```

`gho_clean()` and `sdg_clean()` return data in the same 15-column
schema, so a handful of `ggplot2` recipes cover most routine indicator
charts. This vignette collects the recipes we reach for most often:
trend lines with confidence bands, sex/age breakdowns, forest-style
country comparisons, dumbbell progress plots, and progress tracking
with `aarr()`.

Chart-building helpers stay out of the package on purpose — each
recipe below is a few lines of plain `ggplot2` on top of the cleaned
schema, and your project code can adapt them freely.

## Example data

The recipes need data in the cleaned schema. A real workflow would
start from the APIs:

```{r, eval = FALSE}
ncd <- gho_data("NCDMORT3070", spatial_type = "country",
                area = c("PHL", "VNM", "KHM", "MNG", "FJI", "LAO")) |>
  gho_clean()
```

So that this vignette builds offline (and to keep the examples
reproducible), we simulate a small dataset with the same shape: an
NCD-mortality-style indicator for six Western Pacific countries,
2000--2023, with a sex breakdown in `dim1` and a confidence band in
`low` / `high`.

```{r simulate}
set.seed(416)

countries <- who_countries |>
  filter(iso3 %in% c("PHL", "VNM", "KHM", "MNG", "FJI", "LAO")) |>
  select(iso3, location_name = name_short)

grid <- expand.grid(
  iso3 = countries$iso3,
  year = 2000:2023,
  dim1 = c("SEX_BTSX", "SEX_MLE", "SEX_FMLE"),
  stringsAsFactors = FALSE
) |>
  as_tibble()

base_rate <- c(PHL = 30, VNM = 26, KHM = 28, MNG = 34, FJI = 31, LAO = 29)
decline   <- c(PHL = 0.010, VNM = 0.016, KHM = 0.012,
               MNG = 0.006, FJI = -0.002, LAO = 0.011)
sex_shift <- c(SEX_BTSX = 1, SEX_MLE = 1.25, SEX_FMLE = 0.8)

ncd <- grid |>
  left_join(countries, by = "iso3") |>
  mutate(
    value_num = base_rate[iso3] * (1 - decline[iso3])^(year - 2000) *
      sex_shift[dim1] * exp(rnorm(n(), sd = 0.01)),
    low       = value_num * 0.93,
    high      = value_num * 1.07,
    source    = "gho",
    id        = "NCDMORT3070",
    indicator = "Probability of premature death from NCDs (%)",
    location  = iso3,
    year      = as.integer(year),
    value     = as.character(round(value_num, 1)),
    series    = NA_character_,
    dim2      = NA_character_,
    dim3      = NA_character_
  ) |>
  select(source, id, indicator, location, iso3, location_name, year,
         value, value_num, low, high, series, dim1, dim2, dim3)

ncd
```

## Trend lines with a confidence band

`low` and `high` map directly onto `geom_ribbon()`. Filter to a single
stratum first — here the both-sexes rows (`dim1 == "SEX_BTSX"`).
`theme_dsi_facet()` is the facet-friendly variant of `theme_dsi()`.

```{r ribbon, fig.height = 4.5}
ncd |>
  filter(dim1 == "SEX_BTSX") |>
  ggplot(aes(year, value_num)) +
  geom_ribbon(aes(ymin = low, ymax = high), fill = "#0093D5", alpha = 0.2) +
  geom_line(color = "#0093D5", linewidth = 0.7) +
  facet_wrap(~ location_name) +
  labs(
    title    = "Premature NCD mortality is declining in most countries",
    subtitle = "Probability of premature death from NCDs, both sexes, 2000-2023",
    x = NULL, y = "%",
    caption  = "Simulated data for illustration."
  ) +
  theme_dsi_facet()
```

## Breakdowns from `dim1` / `dim2` / `dim3`

GHO breakdowns (sex, age group, ...) arrive in `dim1` / `dim2` /
`dim3`. Map the stratum to `colour` and keep palettes colorblind-safe
— `scale_color_viridis_d()` ships with `ggplot2`.

```{r dim-breakdown, fig.height = 4.5}
ncd |>
  filter(dim1 %in% c("SEX_MLE", "SEX_FMLE")) |>
  ggplot(aes(year, value_num, color = dim1)) +
  geom_line(linewidth = 0.7) +
  facet_wrap(~ location_name) +
  scale_color_viridis_d(end = 0.6,
                        labels = c(SEX_FMLE = "Female", SEX_MLE = "Male")) +
  labs(
    title    = "Men die prematurely from NCDs more often than women",
    x = NULL, y = "%", color = NULL,
    caption  = "Simulated data for illustration."
  ) +
  theme_dsi_facet()
```

## Forest-style country comparison

To compare countries at a single point in time with their uncertainty
intervals, take the latest year per country and use
`geom_pointrange()`. Ordering the axis by value makes the ranking
readable.

```{r forest}
latest <- ncd |>
  filter(dim1 == "SEX_BTSX") |>
  slice_max(year, by = iso3)

ggplot(latest, aes(value_num, reorder(location_name, value_num))) +
  geom_pointrange(aes(xmin = low, xmax = high),
                  color = "#0093D5", linewidth = 0.7) +
  labs(
    title   = "Premature NCD mortality, 2023",
    x = "%", y = NULL,
    caption = "Point estimate and 95% interval. Simulated data."
  ) +
  theme_dsi()
```

## Dumbbell progress plot

A dumbbell plot compares two time points per country. Build the wide
shape with two filtered slices and a join — no reshaping package
needed. The colors are from the colorblind-safe Okabe-Ito palette.

```{r dumbbell}
start <- ncd |>
  filter(dim1 == "SEX_BTSX", year == 2000) |>
  select(location_name, start = value_num)
end <- ncd |>
  filter(dim1 == "SEX_BTSX", year == 2023) |>
  select(location_name, end = value_num)

dumbbell <- start |>
  left_join(end, by = "location_name") |>
  mutate(
    direction     = ifelse(end < start, "Decline", "Increase"),
    location_name = reorder(location_name, start)
  )

ggplot(dumbbell) +
  geom_segment(aes(x = start, xend = end,
                   y = location_name, yend = location_name,
                   color = direction),
               linewidth = 1.5) +
  geom_point(aes(start, location_name), size = 3, color = "grey65") +
  geom_point(aes(end, location_name, color = direction), size = 3) +
  scale_color_manual(values = c(Decline = "#0072B2", Increase = "#D55E00")) +
  labs(
    title   = "Change in premature NCD mortality, 2000 to 2023",
    x = "%", y = NULL, color = NULL,
    caption = "Grey dot = 2000, colored dot = 2023. Simulated data."
  ) +
  theme_dsi()
```

## Tracking progress with `aarr()`

`aarr()` computes the average annual rate of reduction — the standard
WHO / UNICEF progress metric. It slots directly into a grouped
`summarise()` on the cleaned schema. Remember the sign convention:
positive means the indicator is declining.

```{r aarr}
progress <- ncd |>
  filter(dim1 == "SEX_BTSX") |>
  summarise(aarr = aarr(year, value_num), .by = c(iso3, location_name))

progress |>
  mutate(aarr_pct = round(100 * aarr, 1)) |>
  arrange(desc(aarr))
```

A back-of-envelope projection carries the latest value forward at the
observed AARR. This is a quick screening device, not a substitute for
the indicator-specific models behind official 2030 projections.

```{r projection}
latest |>
  left_join(progress, by = c("iso3", "location_name")) |>
  mutate(projected_2030 = value_num * (1 - aarr)^(2030 - year)) |>
  select(location_name, value_2023 = value_num, aarr, projected_2030) |>
  arrange(projected_2030)
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/iso3_to_m49.R
\name{iso3_to_m49}
\alias{iso3_to_m49}
\title{Convert ISO3 Codes to UN M49 Numeric Codes}
\usage{
iso3_to_m49(iso3)
}
\arguments{
\item{iso3}{Character vector of ISO3 codes. Case-insensitive;
values are upper-cased before lookup.}
}
\value{
A character vector the same length as \code{iso3}, with M49
codes in the same format as \code{who_countries$m49_code} (three-
character zero-padded strings, e.g. \code{"076"}). Non-Member areas
return \code{NA}.
}
\description{
Maps ISO 3166-1 alpha-3 country codes to UN M49 numeric area codes
using the \code{\link{who_countries}} dataset shipped with DSIR. Useful when
moving from data sources keyed by ISO3 (e.g. the WHO GHO API) to
sources keyed by M49 (e.g. the UN SDG API).
}
\details{
Codes that do not correspond to a WHO Member State return \code{NA}.
This includes Associate Members (e.g. Puerto Rico) and other
non-Member areas that some indicator data sets cover.

Most users will not need to call this function directly:
\code{\link[=sdg_data]{sdg_data()}} and \code{\link[=sdg_coverage]{sdg_coverage()}} accept ISO3 codes for their
\code{area} argument and convert internally. This helper is exported
for cases where you want to inspect or manipulate the conversion
yourself.
}
\examples{
iso3_to_m49(c("PHL", "FRA", "JPN"))
# "608" "250" "392"

# Case-insensitive
iso3_to_m49("phl")
# "608"

# Non-Member areas return NA
iso3_to_m49(c("PRI", "PHL"))
# NA "608"
}
\seealso{
\code{\link{who_countries}}, \code{\link[=iso3_to_region]{iso3_to_region()}}, \code{\link[=sdg_data]{sdg_data()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sdg.R
\name{sdg_data}
\alias{sdg_data}
\title{Fetch SDG Data}
\usage{
sdg_data(
  indicator,
  area = NULL,
  year_from = NULL,
  year_to = NULL,
  page_size = 1000L
)
}
\arguments{
\item{indicator}{Character vector of indicator codes
(e.g. \code{"1.1.1"}). Use \code{\link[=sdg_indicators]{sdg_indicators()}} to find codes.}

\item{area}{Character vector of country/area codes. Accepts either
ISO3 codes (e.g. \code{c("PHL", "FRA")}) — converted automatically via
\code{\link[=iso3_to_m49]{iso3_to_m49()}} — or UN M49 numeric codes (e.g. \code{c("608", "250")})
as returned by \code{\link[=sdg_areas]{sdg_areas()}}. Do not mix the two formats in a
single call. Default \code{NULL} returns all areas.}

\item{year_from}{Numeric. Start year filter (inclusive).
Default \code{NULL}.}

\item{year_to}{Numeric. End year filter (inclusive).
Default \code{NULL}.}

\item{page_size}{Integer. Number of records per page.
Default \code{1000}, maximum \code{10000}.}
}
\value{
A \link[tibble:tibble]{tibble} of indicator observations, or
an empty tibble when the service is unreachable or there are no
matching rows.
}
\description{
Retrieves data for one or more SDG indicators from the UN SDG
API, with optional filters by area and year.
}
\examples{
\donttest{
# One indicator, one country — the typical entry point
sdg_data("1.1.1", area = "PHL")

# Specific area and year range (M49 code)
sdg_data("3.2.1", area = "156", year_from = 2015, year_to = 2023)

# ISO3 codes work directly — DSIR's regional vectors can be passed in
sdg_data("3.4.1", area = c("PHL", "FRA", "JPN"))
}
}
\seealso{
\code{\link[=sdg_indicators]{sdg_indicators()}}, \code{\link[=sdg_areas]{sdg_areas()}}, \code{\link[=iso3_to_m49]{iso3_to_m49()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gho.R
\name{gho_data}
\alias{gho_data}
\title{Fetch GHO Data}
\usage{
gho_data(
  indicator,
  spatial_type = NULL,
  area = NULL,
  year_from = NULL,
  year_to = NULL
)
}
\arguments{
\item{indicator}{Character scalar. The indicator code
(e.g. \code{"NCDMORT3070"}). Use \code{\link[=gho_indicators]{gho_indicators()}} to find codes.}

\item{spatial_type}{Character. Spatial dimension to filter on:
one of \code{"country"}, \code{"region"}, \code{"global"}, or \code{NULL} (all
levels, the default).}

\item{area}{Character vector of country or region codes
(e.g. \code{c("FRA", "DEU")}). Default \code{NULL} returns all areas.}

\item{year_from}{Numeric. Start year filter (inclusive).
Default \code{NULL}.}

\item{year_to}{Numeric. End year filter (inclusive).
Default \code{NULL}.}
}
\value{
A \link[tibble:tibble]{tibble} of indicator observations, or
an empty tibble when the service is unreachable.
}
\description{
Retrieves observations for a specific indicator from the WHO GHO
OData API, with optional filters by spatial level, country /
region and year range.
}
\examples{
\donttest{
# Country-level data for one indicator
gho_data("NCDMORT3070", spatial_type = "country")

# Specific countries and years
gho_data("WHOSIS_000001", area = c("FRA", "DEU"), year_from = 2015)
}
}
\seealso{
\code{\link[=gho_indicators]{gho_indicators()}}, \code{\link[=gho_dimensions]{gho_dimensions()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sdg_coverage.R
\name{sdg_coverage}
\alias{sdg_coverage}
\title{Explore Series Coverage of an SDG Indicator}
\usage{
sdg_coverage(indicator, area = NULL, year_from = NULL, year_to = NULL)
}
\arguments{
\item{indicator}{Character vector of SDG indicator codes
(e.g. \code{"3.4.1"}).}

\item{area}{Character vector of country/area codes. Accepts either
ISO3 codes (e.g. \code{c("PHL", "FRA")}) — converted automatically via
\code{\link[=iso3_to_m49]{iso3_to_m49()}} — or UN M49 numeric codes (e.g. \code{c("608", "250")})
as returned by \code{\link[=sdg_areas]{sdg_areas()}}. Do not mix the two formats in a
single call. Default \code{NULL} returns all areas. Unknown ISO3 codes
are dropped with a warning before the network call.}

\item{year_from}{Numeric. Start year filter (inclusive).
Default \code{NULL}.}

\item{year_to}{Numeric. End year filter (inclusive).
Default \code{NULL}.}
}
\value{
A \link[tibble:tibble]{tibble} with one row per
\verb{(location, series)} and columns:
\itemize{
\item \code{location} (chr) — area code (\code{geoAreaCode}).
\item \code{series} (chr) — SDG series code.
\item \code{year_min} (int) — earliest year with data.
\item \code{year_max} (int) — latest year with data.
\item \code{n_obs} (int) — number of observations.

Sorted by \code{location} then \code{series}. Empty input or service
failure returns an empty tibble with the same five columns.
}
}
\description{
A single SDG indicator (for example \code{"3.4.1"}, NCD mortality) is
typically published as several \strong{series} stratified by sex, age,
or cause. Different series may have different country and year
coverage. \code{sdg_coverage()} summarises year range and observation
count per \verb{(location, series)} combination, so you can see which
series exist for an indicator and how each one is covered before
committing to a downstream analysis.
}
\details{
Unlike the GHO availability helpers, this function is a
series-exploration tool rather than a payload-saving precheck:
SDG data is generally complete enough that GHO-style
\code{has_data()} / \code{count()} helpers add little value, so they are
intentionally not provided. The SDG API also offers no
payload-reduction option (no \verb{$select} equivalent), so
\code{sdg_coverage()} calls \code{\link[=sdg_data]{sdg_data()}} internally and aggregates
the result client-side.
}
\examples{
\donttest{
# Series available for NCD mortality in China and Brazil
sdg_coverage("3.4.1", area = c("156", "076"))

# Filter to a year range
sdg_coverage("3.4.1", area = "156", year_from = 2015)
}
}
\seealso{
\code{\link[=sdg_data]{sdg_data()}}, \code{\link[=sdg_indicators]{sdg_indicators()}}, \code{\link[=gho_coverage]{gho_coverage()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scale_dsi_col.R
\name{scale_dsi_col}
\alias{scale_dsi_col}
\alias{scale_y_dsi_col}
\alias{scale_x_dsi_col}
\title{Continuous Scales for DSIR Bar / Column Charts}
\usage{
scale_y_dsi_col(...)

scale_x_dsi_col(...)
}
\arguments{
\item{...}{Arguments forwarded to the underlying
\code{\link[ggplot2:scale_y_continuous]{ggplot2::scale_y_continuous()}} / \code{\link[ggplot2:scale_x_continuous]{ggplot2::scale_x_continuous()}}.}
}
\value{
A \code{ggplot2} Scale object, to be added to a plot with \code{+}.
}
\description{
Thin wrappers around \code{\link[ggplot2:scale_y_continuous]{ggplot2::scale_y_continuous()}} and
\code{\link[ggplot2:scale_x_continuous]{ggplot2::scale_x_continuous()}} that remove the default lower expansion
so that columns sit flush with the axis — the convention for WHO and
most publication-style bar charts. The upper expansion is preserved at
5\% so the tallest column has breathing room above (or to the right of) it.
}
\details{
Use \code{scale_y_dsi_col()} for vertical bars (\code{geom_col()} / \code{geom_bar()})
and \code{scale_x_dsi_col()} when bars are horizontal (via \code{coord_flip()} or
\code{geom_col(orientation = "y")}).

Pass any other \verb{scale_*_continuous()} argument
(\code{labels}, \code{breaks}, \code{limits}, ...) through \code{...}.
}
\examples{
library(ggplot2)

# Vertical bars
ggplot(mtcars, aes(factor(cyl))) +
  geom_bar(fill = "#0093D5") +
  scale_y_dsi_col() +
  theme_dsi()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gho.R
\name{gho_clean}
\alias{gho_clean}
\title{Tidy a GHO Data Frame}
\usage{
gho_clean(df)
}
\arguments{
\item{df}{A data frame returned by \code{\link[=gho_data]{gho_data()}}.}
}
\value{
A \link[tibble:tibble]{tibble} with 15 columns: \code{source} (always
\code{"gho"}), \code{id}, \code{indicator}, \code{location}, \code{iso3}, \code{location_name},
\code{year}, \code{value}, \code{value_num}, \code{low}, \code{high}, \code{series} (\code{NA}),
\code{dim1}, \code{dim2}, \code{dim3}. Sorted by \code{location} then \code{year}.
Empty input returns an empty tibble with the same columns and
types.
}
\description{
Selects, renames, and type-casts the most useful columns from a GHO
observation table returned by \code{\link[=gho_data]{gho_data()}}, producing a compact
tibble in the \strong{unified DSIR cleaned-indicator schema} — the same
schema produced by \code{\link[=sdg_clean]{sdg_clean()}}, so the two outputs can be combined
directly with \code{\link[=bind_indicators]{bind_indicators()}}.
}
\details{
The mapping (GHO source → unified column) is:
\itemize{
\item \code{IndicatorCode} → \code{id}
\item \code{IndicatorCode} resolved against the GHO indicator catalog →
\code{indicator} (the human-readable name; cached at session level
after the first call)
\item \code{SpatialDim}    → \code{location}; also \code{iso3} when it matches a WHO
Member State, otherwise \code{iso3 = NA}
\item \code{TimeDim}       → \code{year} (integer)
\item \code{Value}         → \code{value} (character; raw)
\item \code{NumericValue}  → \code{value_num} (numeric)
\item \code{Low}, \code{High}   → \code{low}, \code{high} (numeric)
\item \code{Dim1}, \code{Dim2}, \code{Dim3} → \code{dim1}, \code{dim2}, \code{dim3} (character)
}

The \code{series} column is always \code{NA} for GHO output (it is an SDG-only
concept). The \code{location_name} column is populated by looking up
\code{location} (an ISO3 code or a WHO region code) against the
\code{\link{who_countries}} dataset and a hardcoded set of WHO regional names;
locations that match neither (e.g. non-Member State areas) are left
as \code{NA}.

Source columns absent from \code{df} (e.g. \code{Low} / \code{High} for indicators
without confidence intervals) are filled with typed \code{NA}, so the
output always has the same 15 columns with the same column types.

The GHO data endpoint (\verb{/api/\{IndicatorCode\}}) does not return
\code{IndicatorName}; that field lives on the catalog endpoint queried by
\code{\link[=gho_indicators]{gho_indicators()}}. On the first call within an R session,
\code{gho_clean()} fetches the catalog once and caches it for the rest of
the session, so the \code{indicator} column carries the full
human-readable indicator name. If the catalog cannot be fetched
(e.g. no network), \code{\link[=gho_indicators]{gho_indicators()}} emits a warning and the
\code{indicator} column falls back to \code{NA}.
}
\examples{
\donttest{
gho_data("NCDMORT3070", spatial_type = "country") |>
  gho_clean()
}
}
\seealso{
\code{\link[=gho_data]{gho_data()}}, \code{\link[=sdg_clean]{sdg_clean()}}, \code{\link[=bind_indicators]{bind_indicators()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggpie.R
\name{ggpie}
\alias{ggpie}
\title{Create a Pie Chart with ggplot2}
\usage{
ggpie(
  df,
  .x,
  .y,
  .offset = 1,
  .color = "white",
  .legend = FALSE,
  .label = TRUE,
  .label_size = 3.5
)
}
\arguments{
\item{df}{A data frame.}

\item{.x}{Column name (string) of the categorical variable used
for the slices.}

\item{.y}{Column name (string) of the numeric variable used for
the slice values.}

\item{.offset}{Numeric scalar (> 0). Controls label position
along the slice radius. Default \code{1} places the label at the
middle of the slice. Smaller values (e.g. \code{0.5}) move the
label inward toward the centre; larger values (e.g. \code{2})
move it outward toward the edge or beyond, useful for
donut-style layouts.}

\item{.color}{Border color between slices. Default \code{"white"}.}

\item{.legend}{Logical. Show the legend? Default \code{FALSE}.}

\item{.label}{Logical. Draw \code{"name\\n(pct\%)"} labels on the
slices? Default \code{TRUE}.}

\item{.label_size}{Label text size in mm. Default \code{3.5}.}
}
\value{
A \code{ggplot} object.
}
\description{
Builds a pie chart from a data frame using one categorical column
and one numeric column. Slices are labeled with the category
name and percentage share.
}
\examples{
df <- data.frame(
  category = c("A", "B", "C"),
  value = c(40, 35, 25)
)
ggpie(df, "category", "value")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sdg.R
\name{sdg_goals}
\alias{sdg_goals}
\title{List SDG Goals}
\usage{
sdg_goals(include_children = FALSE)
}
\arguments{
\item{include_children}{Logical. Include targets and indicators
nested under each goal? Default \code{FALSE}.}
}
\value{
A list (or \link[tibble:tibble]{tibble}) of SDG goals, or
\code{NULL} when the service is unreachable.
}
\description{
Fetches the list of Sustainable Development Goals from the UN
SDG API.
}
\examples{
\donttest{
sdg_goals()
sdg_goals(include_children = TRUE)
}
}
\seealso{
\code{\link[=sdg_targets]{sdg_targets()}}, \code{\link[=sdg_indicators]{sdg_indicators()}}, \code{\link[=sdg_data]{sdg_data()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/m49_to_iso3.R
\name{m49_to_iso3}
\alias{m49_to_iso3}
\title{Convert UN M49 Numeric Codes to ISO3 Codes}
\usage{
m49_to_iso3(m49)
}
\arguments{
\item{m49}{Character vector of M49 codes.}
}
\value{
A character vector the same length as \code{m49}. Non-Member
codes (region aggregates, non-Member areas) return \code{NA}.
}
\description{
Maps UN M49 numeric area codes to ISO 3166-1 alpha-3 country codes
using the \code{\link{who_countries}} dataset shipped with DSIR. Counterpart
to \code{\link[=iso3_to_m49]{iso3_to_m49()}} and used internally by \code{\link[=sdg_clean]{sdg_clean()}} to populate
the \code{iso3} column on SDG output.
}
\details{
M49 codes that do not correspond to a WHO Member State return \code{NA}.
This includes region / world aggregates (e.g. \code{"900"} for World,
\code{"001"} for World, \code{"419"} for Latin America and the Caribbean)
and codes for non-Member areas (e.g. Puerto Rico, Tokelau).

Input accepts either the zero-padded form (\code{"076"}) or the bare
form (\code{"76"}); both are normalised before lookup. Non-numeric
input returns \code{NA} (with a single warning from the underlying
\code{\link[=as.integer]{as.integer()}} coercion).
}
\examples{
m49_to_iso3(c("608", "250", "392"))
# "PHL" "FRA" "JPN"

# Zero-padded and bare forms both accepted
m49_to_iso3(c("076", "76"))
# "BRA" "BRA"

# Non-Member areas / aggregates return NA
m49_to_iso3(c("900", "608"))
# NA "PHL"
}
\seealso{
\code{\link{who_countries}}, \code{\link[=iso3_to_m49]{iso3_to_m49()}}, \code{\link[=sdg_clean]{sdg_clean()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sdg.R
\name{sdg_clean}
\alias{sdg_clean}
\title{Tidy an SDG Data Frame}
\usage{
sdg_clean(df)
}
\arguments{
\item{df}{A data frame returned by \code{\link[=sdg_data]{sdg_data()}}.}
}
\value{
A \link[tibble:tibble]{tibble} with 15 columns: \code{source} (always
\code{"sdg"}), \code{id}, \code{indicator}, \code{location}, \code{iso3}, \code{location_name},
\code{year}, \code{value}, \code{value_num}, \code{low}, \code{high}, \code{series}, \code{dim1}
(\code{NA}), \code{dim2} (\code{NA}), \code{dim3} (\code{NA}). Sorted by \code{location} then
\code{year}. Empty input returns an empty tibble with the same columns
and types.
}
\description{
Selects, renames, and type-casts the most useful columns from an
SDG observation table returned by \code{\link[=sdg_data]{sdg_data()}}, producing a compact
tibble in the \strong{unified DSIR cleaned-indicator schema} — the same
schema produced by \code{\link[=gho_clean]{gho_clean()}}, so the two outputs can be combined
directly with \code{\link[=bind_indicators]{bind_indicators()}}.
}
\details{
The mapping (SDG source → unified column) is:
\itemize{
\item \code{indicator} (list-column, flattened) → \code{id} (e.g. \code{"3.4.1"})
\item \code{seriesDescription}                  → \code{indicator} (human-readable
label; \code{NA} if the API response does not include it)
\item \code{geoAreaCode}                        → \code{location} (UN M49 numeric,
as character); also \code{iso3} via \code{\link[=m49_to_iso3]{m49_to_iso3()}} for WHO Member
States — region / world aggregates and non-Member areas get
\code{iso3 = NA}
\item \code{location_name} is resolved by looking up \code{iso3} against
\code{\link{who_countries}} (so a WHO Member State has the same
\code{location_name} here and in \code{\link[=gho_clean]{gho_clean()}} output), with a
fallback to the SDG API's raw \code{geoAreaName} for non-Member-State
rows (e.g. regional / world aggregates)
\item \code{timePeriodStart}                    → \code{year} (integer)
\item \code{value}                              → \code{value} (character; raw)
and \code{value_num} (numeric; \code{NA} for non-numeric entries like
\code{"<0.1"} or aggregate notes)
\item \code{lowerBound}, \code{upperBound}           → \code{low}, \code{high} (numeric)
\item \code{series}                             → \code{series}
}

Three columns are always present but never populated for SDG
output: \code{dim1}, \code{dim2}, \code{dim3} (GHO-only concepts).
}
\examples{
\donttest{
sdg_data("3.2.1", area = "156", year_from = 2015) |>
  sdg_clean()
}
}
\seealso{
\code{\link[=sdg_data]{sdg_data()}}, \code{\link[=gho_clean]{gho_clean()}}, \code{\link[=bind_indicators]{bind_indicators()}},
\code{\link[=m49_to_iso3]{m49_to_iso3()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{pic_cty}
\alias{pic_cty}
\title{Pacific Island Country ISO3 codes}
\format{
A character vector of 14 ISO 3166-1 alpha-3 codes.
}
\usage{
pic_cty
}
\description{
Character vector of ISO3 codes for the 14 WHO Member States classified
as Pacific Island Countries (PICs). Sorted alphabetically.
}
\details{
All 14 PICs are within the Western Pacific Region, so
\code{all(pic_cty \%in\% wpro_cty)} is \code{TRUE}. Non-Member Pacific areas
(e.g. New Caledonia, French Polynesia, American Samoa, Tokelau) are
not included.

The 14 PIC Member States are: Cook Islands, Fiji, Kiribati, Marshall
Islands, Micronesia (Federated States of), Nauru, Niue, Palau, Papua
New Guinea, Samoa, Solomon Islands, Tonga, Tuvalu, and Vanuatu.
}
\examples{
length(pic_cty)              # 14
all(pic_cty \%in\% wpro_cty)   # TRUE
}
\seealso{
\code{\link{who_countries}}, \code{\link{wpro_cty}}.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/flextable_defaults.R
\name{dsi_flextable_defaults}
\alias{dsi_flextable_defaults}
\title{Set DSIR Flextable Defaults}
\usage{
dsi_flextable_defaults(
  font_size = 12,
  font_family = "",
  font_color = "#333333",
  border_color = "black",
  padding = c(3, 3, 4, 4)
)
}
\arguments{
\item{font_size}{Font size in points. Default \code{12}.}

\item{font_family}{Font family name. Default \code{""} keeps the
existing \code{flextable} default; try \code{"Cambria"} for the original
DSIR look on Windows.}

\item{font_color}{Font color. Default \code{"#333333"}.}

\item{border_color}{Border color. Default \code{"black"}.}

\item{padding}{Numeric vector of length 1 (applied to all sides)
or length 4 (\code{top}, \code{bottom}, \code{left}, \code{right}).
Default \code{c(3, 3, 4, 4)}.}
}
\value{
Invisibly returns \code{NULL}. Called for its side effect of
mutating the \code{flextable} global defaults via
\code{\link[flextable:set_flextable_defaults]{flextable::set_flextable_defaults()}}.
}
\description{
Applies a consistent set of \code{flextable} formatting defaults for
publication-ready tables (booktabs theme, bold headers, modest
padding). Pick any font you like — the default \code{""} leaves the
\code{flextable} default in place so the call is safe on systems where
Cambria is not installed.
}
\examples{
dsi_flextable_defaults()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{who_region_vectors}
\alias{who_region_vectors}
\alias{afro_cty}
\alias{amro_cty}
\alias{searo_cty}
\alias{euro_cty}
\alias{emro_cty}
\alias{wpro_cty}
\title{WHO regional Member State ISO3 vectors}
\format{
Character vectors of ISO 3166-1 alpha-3 codes.

An object of class \code{character} of length 47.

An object of class \code{character} of length 35.

An object of class \code{character} of length 10.

An object of class \code{character} of length 53.

An object of class \code{character} of length 21.

An object of class \code{character} of length 28.
}
\usage{
afro_cty

amro_cty

searo_cty

euro_cty

emro_cty

wpro_cty
}
\description{
Convenience character vectors of ISO3 codes for the WHO Member States
in each region. Each vector is the regional subset of
\code{\link{who_countries}}'s \code{iso3} column, sorted alphabetically.
}
\details{
These are derived directly from \code{\link{who_countries}} and exist for
ergonomic filtering, e.g.
\code{dplyr::filter(data, iso3 \%in\% wpro_cty)}.
If you need to update group membership, edit
\code{data-raw/who_countries.R} and re-run that script — the vectors
regenerate from the master table.

\describe{
\item{\code{afro_cty}}{47 Member States in the WHO African Region.}
\item{\code{amro_cty}}{35 Member States in the WHO Region of the Americas.}
\item{\code{searo_cty}}{10 Member States in the WHO South-East Asia Region.}
\item{\code{euro_cty}}{53 Member States in the WHO European Region.}
\item{\code{emro_cty}}{21 Member States in the WHO Eastern Mediterranean
Region.}
\item{\code{wpro_cty}}{28 Member States in the WHO Western Pacific Region.
Includes Indonesia from May 2025 (per WHO EB156).}
}
}
\examples{
length(wpro_cty)        # 28
"IDN" \%in\% wpro_cty     # TRUE — Indonesia in WPR since May 2025

}
\seealso{
\code{\link{who_countries}}, \code{\link{pic_cty}}.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theme_dsi.R
\name{theme_dsi}
\alias{theme_dsi}
\title{DSIR ggplot2 theme — WHO publication-style}
\usage{
theme_dsi(
  base_size = 12,
  base_family = "",
  accent = "#0093D5",
  grid_color = "grey92",
  grid = c("both", "x", "y", "none"),
  legend_position = "bottom"
)
}
\arguments{
\item{base_size}{Base font size in points. Default \code{12}.}

\item{base_family}{Base font family. Default \code{""} (system default).
Set to \code{"sans"}, \code{"Arial"}, \code{"Helvetica"}, or \code{"Calibri"} for a
specific look. Empty default keeps the package portable on CRAN's
Linux check machines, where Calibri is unavailable.}

\item{accent}{Accent colour used for axis lines and as a default for
highlight elements. Default \code{"#0093D5"} (WHO blue). Pass any colour
string.}

\item{grid_color}{Colour of the major grid. Default
\code{"grey92"} — a very light grey, close to bbplot and OECD style.}

\item{grid}{Which direction(s) to draw major grid lines. One of
\code{"both"} (default — draws both horizontal and vertical, works
correctly under \code{coord_flip()}), \code{"y"} (only horizontal grid lines,
the look used in DSIR <= 0.5.x), \code{"x"} (only vertical grid lines),
or \code{"none"}.}

\item{legend_position}{Position of the legend. Default \code{"bottom"}.
Pass \code{"none"}, \code{"top"}, \code{"right"}, or a numeric vector \code{c(x, y)}.}
}
\value{
A \code{ggplot2} theme object.
}
\description{
A clean, modern theme tuned for WHO and global-health publications.
Removes the panel border, draws light grid lines, and uses a muted
text colour so that the data — not the chart chrome — is the visual
focus. The \code{grid} argument controls which direction(s) the grid
lines run, so the theme works equally well for vertical bars,
horizontal bars (via \code{coord_flip()}), scatter plots, and line charts.
}
\examples{
library(ggplot2)

# Default — grid in both directions, works under coord_flip()
ggplot(mtcars, aes(wt, mpg, color = factor(cyl))) +
  geom_point(size = 3) +
  theme_dsi() +
  labs(title = "Fuel efficiency by weight",
       x = "Weight (1000 lbs)", y = "Miles per gallon",
       color = "Cylinders")

# Minimal look — only horizontal grid lines
ggplot(mtcars, aes(wt, mpg)) +
  geom_point(size = 3, color = "#0093D5") +
  theme_dsi(grid = "y") +
  labs(x = "Weight (1000 lbs)", y = "Miles per gallon")

}
\seealso{
\code{\link[=theme_dsi_facet]{theme_dsi_facet()}} for a sibling theme tuned for faceted plots.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sdg.R
\name{sdg_indicators}
\alias{sdg_indicators}
\title{List SDG Indicators}
\usage{
sdg_indicators(search = NULL)
}
\arguments{
\item{search}{Optional character. Search keywords matched against
the \code{description} column (case-insensitive). All terms must
match (AND semantics). Accepts either:
\itemize{
\item a single string, which is split on whitespace into terms
(e.g. \code{"mortality cancer"} keeps rows whose description
contains both "mortality" and "cancer"), or
\item a character vector, whose elements are used as terms verbatim
(so a term may itself contain whitespace, e.g.
\code{c("mortality rate", "attributed")}).
}

The filter is applied client-side using
\code{\link[=grepl]{grepl()}} with \code{fixed = TRUE} because the UN SDG
\verb{/Indicator/List} endpoint is not OData and exposes no
server-side search parameter; the full list is small
(~250 rows) so this is cheap.}
}
\value{
A list (or \link[tibble:tibble]{tibble}) of SDG indicators,
or \code{NULL} when the service is unreachable. When \code{search} matches
no rows, an empty tibble with the same columns as the unfiltered
response is returned.
}
\description{
Fetches the list of SDG indicators from the UN SDG API, with
optional keyword filtering on the indicator description.
}
\examples{
\donttest{
# Full list
sdg_indicators()

# Single keyword
sdg_indicators("mortality")

# Multi-keyword — AND semantics
sdg_indicators("mortality cancer")
sdg_indicators(c("maternal", "mortality"))
}
}
\seealso{
\code{\link[=sdg_targets]{sdg_targets()}}, \code{\link[=sdg_data]{sdg_data()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gho.R
\name{gho_dimensions}
\alias{gho_dimensions}
\title{List Dimensions of a GHO Indicator}
\usage{
gho_dimensions(indicator, dimension = "SpatialDimType")
}
\arguments{
\item{indicator}{Character scalar. The indicator code
(e.g. \code{"NCDMORT3070"}).}

\item{dimension}{Character. Name of the dimension column in the
indicator data. Common values include \code{"SpatialDim"},
\code{"SpatialDimType"}, \code{"TimeDim"}, \code{"Dim1"}, \code{"Dim2"}, and
\code{"Dim3"}. Case-sensitive (it is sent to the server as an OData
\verb{$select} field name). Default \code{"SpatialDimType"}.}
}
\value{
A character vector of unique, sorted dimension values,
or an empty character vector when the service is unreachable
or the dimension is missing.
}
\description{
Returns the unique values of a given dimension across all
observations of a GHO indicator. Useful for discovering which
ages, sexes, regions, or other breakdowns are available before
calling \code{\link[=gho_data]{gho_data()}}.
}
\details{
Only the requested column is downloaded (via the OData \verb{$select}
query option), so this is a lightweight metadata query even for
indicators with hundreds of thousands of observations. A
\code{dimension} that is not a column of the GHO data table (e.g. a
misspelling) makes the server reject the request; the failure
surfaces as a warning and an empty character vector.
}
\examples{
\donttest{
gho_dimensions("NCDMORT3070")
gho_dimensions("NCDMORT3070", dimension = "Dim1")
}
}
\seealso{
\code{\link[=gho_data]{gho_data()}}, \code{\link[=gho_indicators]{gho_indicators()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theme_dsi.R
\name{theme_dsi_facet}
\alias{theme_dsi_facet}
\title{DSIR ggplot2 theme for faceted plots}
\usage{
theme_dsi_facet(
  base_size = 12,
  base_family = "",
  accent = "#0093D5",
  grid_color = "grey92",
  strip_fill = "grey95",
  strip_color = "grey20",
  grid = c("both", "x", "y", "none"),
  legend_position = "bottom"
)
}
\arguments{
\item{base_size}{Base font size in points. Default \code{12}.}

\item{base_family}{Base font family. Default \code{""} (system default).
Set to \code{"sans"}, \code{"Arial"}, \code{"Helvetica"}, or \code{"Calibri"} for a
specific look. Empty default keeps the package portable on CRAN's
Linux check machines, where Calibri is unavailable.}

\item{accent}{Accent colour, kept for parity with \code{\link[=theme_dsi]{theme_dsi()}} so
that the argument set is interchangeable. Not currently used in the
facet variant — the panel border replaces the accent-coloured axis
lines. Default \code{"#0093D5"} (WHO blue).}

\item{grid_color}{Colour of the major grid (both axes). Default
\code{"grey92"}.}

\item{strip_fill}{Background fill colour for facet strips. Default
\code{"grey95"} — a light neutral grey. Avoid blues, which clash with
the WHO-blue \code{accent}.}

\item{strip_color}{Text colour inside facet strips. Default \code{"grey20"}.}

\item{grid}{Which direction(s) to draw major grid lines. One of
\code{"both"} (default — both horizontal and vertical), \code{"y"} (only
horizontal), \code{"x"} (only vertical), or \code{"none"}. Matches the
identical argument on \code{\link[=theme_dsi]{theme_dsi()}} so the two themes can be
swapped without changing other settings.}

\item{legend_position}{Position of the legend. Default \code{"bottom"}.
Pass \code{"none"}, \code{"top"}, \code{"right"}, or a numeric vector \code{c(x, y)}.}
}
\value{
A \code{ggplot2} theme object.
}
\description{
A sibling of \code{\link[=theme_dsi]{theme_dsi()}} tuned for faceted plots. Where \code{theme_dsi()}
uses half-frame axis lines and only horizontal grid lines — a look that
suits a single panel — repeating those across every facet looks heavy
and the panels run together. \code{theme_dsi_facet()} replaces the axis
lines with a light panel border, draws grid lines on both axes, gives
the strip a soft background, and inserts whitespace between panels.
}
\details{
Use \code{theme_dsi()} for single-panel plots and \code{theme_dsi_facet()} for
plots with \code{facet_wrap()} or \code{facet_grid()}. Shared elements (text
styles, title block, legend, plot margins) match \code{theme_dsi()} exactly,
so the two themes feel like the same family.
}
\examples{
library(ggplot2)
ggplot(mtcars, aes(wt, mpg)) +
  geom_point(size = 2, color = "#0093D5") +
  facet_wrap(~ cyl, labeller = label_both) +
  theme_dsi_facet() +
  labs(title = "Fuel efficiency by cylinder count",
       x = "Weight (1000 lbs)", y = "Miles per gallon")

}
\seealso{
\code{\link[=theme_dsi]{theme_dsi()}} for the single-panel sibling.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DSIR-package.R
\docType{package}
\name{DSIR-package}
\alias{DSIR}
\alias{DSIR-package}
\title{DSIR: Data Science Infrastructure for Global Health in R}
\description{
A small, opinionated toolkit for global health data analysis.
It bundles a publication-ready \code{ggplot2} theme (\code{\link[=theme_dsi]{theme_dsi()}},
\code{\link[=theme_dsi_facet]{theme_dsi_facet()}}) with flush-axis bar-chart scales
(\code{\link[=scale_y_dsi_col]{scale_y_dsi_col()}}, \code{\link[=scale_x_dsi_col]{scale_x_dsi_col()}}),
sensible \code{flextable} defaults (\code{\link[=dsi_flextable_defaults]{dsi_flextable_defaults()}}),
a quick pie chart helper (\code{\link[=ggpie]{ggpie()}}), regional country-code
datasets (\link{wpro_cty}) with WHO region and UN M49 lookups
(\code{\link[=iso3_to_region]{iso3_to_region()}}, \code{\link[=iso3_to_m49]{iso3_to_m49()}}, \code{\link[=m49_to_iso3]{m49_to_iso3()}}), a
geometric mean helper for indicator aggregation (\code{\link[=geomean]{geomean()}}),
thin clients for the WHO Global Health Observatory API
(\code{\link[=gho_indicators]{gho_indicators()}}, \code{\link[=gho_data]{gho_data()}}, \code{\link[=gho_dimensions]{gho_dimensions()}},
\code{\link[=gho_has_data]{gho_has_data()}}, \code{\link[=gho_count]{gho_count()}}, \code{\link[=gho_coverage]{gho_coverage()}}) and the UN
Sustainable Development Goals API (\code{\link[=sdg_goals]{sdg_goals()}}, \code{\link[=sdg_targets]{sdg_targets()}},
\code{\link[=sdg_indicators]{sdg_indicators()}}, \code{\link[=sdg_areas]{sdg_areas()}}, \code{\link[=sdg_data]{sdg_data()}}, \code{\link[=sdg_coverage]{sdg_coverage()}}),
plus a unified cleaning / binding pipeline (\code{\link[=gho_clean]{gho_clean()}},
\code{\link[=sdg_clean]{sdg_clean()}}, \code{\link[=bind_indicators]{bind_indicators()}}) that puts GHO and SDG output
into the same 15-column schema.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/shanlong-who/DSIR}
  \item \url{https://shanlong-who.github.io/DSIR/}
  \item Report bugs at \url{https://github.com/shanlong-who/DSIR/issues}
}

}
\author{
\strong{Maintainer}: Shanlong Ding \email{dings@who.int} (\href{https://orcid.org/0000-0001-9048-6670}{ORCID})

Authors:
\itemize{
  \item Shanlong Ding \email{dings@who.int} (\href{https://orcid.org/0000-0001-9048-6670}{ORCID})
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{who_countries}
\alias{who_countries}
\title{WHO Member States with regional and Pacific classifications}
\format{
A tibble with 194 rows and 7 columns:
\describe{
\item{iso3}{ISO 3166-1 alpha-3 code (3-letter, e.g. \code{"PHL"}).}
\item{iso2}{ISO 3166-1 alpha-2 code (2-letter, e.g. \code{"PH"}).}
\item{m49_code}{UN M49 numeric code, stored as a 3-character string with
leading zeros where present (e.g. \code{"008"} for Albania). Stored as
character because some downstream APIs (notably the UN SDG API) expect
the leading-zero form.}
\item{name_official}{WHO official English name (e.g.
\code{"Iran (Islamic Republic of)"}). Use for formal documents and reports.}
\item{name_short}{A shorter form suitable for charts and tables (e.g.
\code{"Iran"}). Equal to \code{name_official} for countries whose official name
is already concise. See Details.}
\item{who_region}{WHO region code: one of \code{"AFR"}, \code{"AMR"}, \code{"SEAR"},
\code{"EUR"}, \code{"EMR"}, \code{"WPR"}.}
\item{is_pic}{Logical. \code{TRUE} for the 14 Pacific Island Country (PIC)
Member States in WPR, \code{FALSE} otherwise.}
}
}
\source{
\itemize{
\item WHO official region listing:
\url{https://www.who.int/countries}
\item ISO 3166-1 codes and UN M49 numeric codes: UN Statistics Division
\url{https://unstats.un.org/unsd/methodology/m49/}
}
}
\usage{
who_countries
}
\description{
A tibble of the 194 World Health Organization (WHO) Member States, with
standard country identifiers and WHO/Pacific groupings used across DSIR
analytical workflows.
}
\details{
\strong{Scope.} WHO has 194 Member States plus 2 Associate Members
(Puerto Rico, Tokelau) and reports on additional non-Member areas
(e.g. West Bank and Gaza Strip). This dataset includes only the 194
Member States. Cook Islands and Niue are full WHO Member States and
are included even though they are not UN member states.

\strong{Region coverage} (as of May 2025, after WHO EB156 reassignment of
Indonesia from SEAR to WPR):
\itemize{
\item AFR (African Region): 47
\item AMR (Region of the Americas): 35
\item SEAR (South-East Asia Region): 10
\item EUR (European Region): 53
\item EMR (Eastern Mediterranean Region): 21
\item WPR (Western Pacific Region): 28
}

\strong{Short names.} \code{name_short} differs from \code{name_official} for 13
countries where the official name is a parenthetical descriptor or is
otherwise long. Examples: \code{"DPR Korea"}, \code{"DR Congo"}, \code{"Lao PDR"},
\code{"United Kingdom"}. These short forms follow conventions used in WHO
regional reports and OECD Health at a Glance: Asia/Pacific.

\strong{PICs.} The Pacific Island Countries flag (\code{is_pic}) marks the 14
WHO Member States in the Pacific sub-region: Cook Islands, Fiji,
Kiribati, Marshall Islands, Micronesia (Federated States of), Nauru,
Niue, Palau, Papua New Guinea, Samoa, Solomon Islands, Tonga, Tuvalu,
and Vanuatu. Non-Member Pacific areas (e.g. New Caledonia, French
Polynesia, American Samoa) are not included in this dataset.
}
\examples{
# All WPR Member States, sorted alphabetically by short name
wpr <- who_countries[who_countries$who_region == "WPR", ]
wpr <- wpr[order(wpr$name_short), c("iso3", "name_short")]
head(wpr)

# Filter a data frame to PIC member states
# df_pic <- subset(my_data, country_iso3 \%in\% who_countries$iso3[who_countries$is_pic])
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gho.R
\name{gho_indicators}
\alias{gho_indicators}
\title{List GHO Indicators}
\usage{
gho_indicators(search = NULL)
}
\arguments{
\item{search}{Optional character. Search keywords matched against
\code{IndicatorName} (case-insensitive). All terms must match
(AND semantics). Accepts either:
\itemize{
\item a single string, which is split on whitespace into terms
(e.g. \code{"child mortality"} matches indicators containing both
"child" and "mortality"), or
\item a character vector, whose elements are used as terms verbatim
(whitespace inside an element is treated as part of the term).
}

Single quotes in any term are escaped for the OData filter.}
}
\value{
A \link[tibble:tibble]{tibble} with columns \code{IndicatorCode},
\code{IndicatorName} and \code{Language}. Returns an empty tibble (with
a message) when the service is unreachable.
}
\description{
Fetches the catalog of indicators from the WHO Global Health
Observatory (GHO) OData API.
}
\examples{
\donttest{
# All indicators
inds <- gho_indicators()

# Single keyword
gho_indicators("mortality")

# Multiple keywords from one string (AND): both terms must appear
gho_indicators("child mortality")

# Or pass terms as a vector
gho_indicators(c("child", "mortality"))
}
}
\seealso{
\code{\link[=gho_data]{gho_data()}}, \code{\link[=gho_dimensions]{gho_dimensions()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geomean.R
\name{geomean}
\alias{geomean}
\title{Geometric Mean}
\usage{
geomean(x, w = NULL, na.rm = TRUE)
}
\arguments{
\item{x}{A numeric vector. Zeros produce a result of \code{0}.
Negative values produce \code{NaN} with a warning, since the
geometric mean is undefined for negative numbers.}

\item{w}{Optional numeric vector of weights, the same length as
\code{x}. Must be non-negative. If \code{NULL} (default), the unweighted
geometric mean is returned.}

\item{na.rm}{Logical. Should missing values in \code{x} (and \code{w}, if
provided) be removed before computation? Default \code{TRUE}.}
}
\value{
A numeric scalar. Returns \code{NA_real_} when the input is
empty, when it is entirely \code{NA}, or when \code{na.rm = FALSE} and
any element is \code{NA}. Returns \code{NaN} with a warning when \code{x}
contains negative values, or when all weights are zero.
}
\description{
Computes the geometric mean of a numeric vector, with optional
weights. Useful for aggregating multiplicative quantities such as
ratio-based health indicators (e.g. UHC service-coverage tracers,
where the composite index is the geometric mean of component
coverage values).
}
\details{
Pass \code{w} to compute a weighted geometric mean, defined as
\code{exp(weighted.mean(log(x), w))}.
}
\examples{
# Unweighted
geomean(c(1, 4, 16))                # 4
geomean(c(0.6, 0.8, 0.95))          # ~0.772 — typical UHC tracer aggregation
geomean(c(1, NA, 4))                # 2
geomean(c(1, NA, 4), na.rm = FALSE) # NA_real_
geomean(c(1, 0, 4))                 # 0

# Weighted
geomean(c(1, 4, 16), w = c(1, 1, 1))     # 4 (equal weights = unweighted)
geomean(c(1, 4, 16), w = c(1, 2, 1))     # weighted toward 4
geomean(c(0.6, 0.8, 0.95), w = c(2, 1, 1))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bind_indicators.R
\name{bind_indicators}
\alias{bind_indicators}
\title{Bind Cleaned Indicator Tibbles}
\usage{
bind_indicators(...)
}
\arguments{
\item{...}{Two or more tibbles returned by \code{\link[=gho_clean]{gho_clean()}} or
\code{\link[=sdg_clean]{sdg_clean()}} (or any data frame with the same column set). \code{NULL}
arguments are dropped. Calling with no inputs — or only \code{NULL}
inputs — returns the empty 15-column tibble.}
}
\value{
A single \link[tibble:tibble]{tibble} with the unified cleaned-
indicator schema (15 columns). Row order is \code{c(input_1, input_2, ...)}, preserving within-input order.
}
\description{
Combines two or more tibbles produced by \code{\link[=gho_clean]{gho_clean()}} or
\code{\link[=sdg_clean]{sdg_clean()}} into a single tibble. Because both cleaners output the
same 15-column schema, the result is a uniform table that can be
filtered, joined, or visualised without source-specific code paths;
use the \code{source} column to tell GHO rows apart from SDG rows.
}
\details{
Inputs do not need to be in any particular order. \code{NULL} inputs are
silently dropped, which makes it ergonomic to write code like
\code{bind_indicators(maybe_gho, maybe_sdg)} where some sources may not
have been fetched.
}
\examples{
\donttest{
gho <- gho_data("NCDMORT3070", area = wpro_cty) |> gho_clean()
sdg <- sdg_data("3.4.1",        area = wpro_cty) |> sdg_clean()
bind_indicators(gho, sdg)
}
}
\seealso{
\code{\link[=gho_clean]{gho_clean()}}, \code{\link[=sdg_clean]{sdg_clean()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sdg.R
\name{sdg_areas}
\alias{sdg_areas}
\title{List SDG Geographic Areas}
\usage{
sdg_areas()
}
\value{
A \link[tibble:tibble]{tibble} with area codes and names, or
\code{NULL} when the service is unreachable.
}
\description{
Fetches the list of geographic areas available from the UN SDG
database.
}
\examples{
\donttest{
sdg_areas()
}
}
\seealso{
\code{\link[=sdg_data]{sdg_data()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sdg.R
\name{sdg_targets}
\alias{sdg_targets}
\title{List SDG Targets}
\usage{
sdg_targets(include_children = FALSE)
}
\arguments{
\item{include_children}{Logical. Include indicators nested under
each target? Default \code{FALSE}.}
}
\value{
A list (or \link[tibble:tibble]{tibble}) of SDG targets, or
\code{NULL} when the service is unreachable.
}
\description{
Fetches the list of SDG targets from the UN SDG API.
}
\examples{
\donttest{
sdg_targets()
}
}
\seealso{
\code{\link[=sdg_goals]{sdg_goals()}}, \code{\link[=sdg_indicators]{sdg_indicators()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gho.R
\name{gho_has_data}
\alias{gho_has_data}
\title{Check Whether a GHO Indicator Has Data for a Filter}
\usage{
gho_has_data(
  indicator,
  spatial_type = NULL,
  area = NULL,
  year_from = NULL,
  year_to = NULL
)
}
\arguments{
\item{indicator}{Character scalar. The indicator code
(e.g. \code{"NCDMORT3070"}). Use \code{\link[=gho_indicators]{gho_indicators()}} to find codes.}

\item{spatial_type}{Character. Spatial dimension to filter on:
one of \code{"country"}, \code{"region"}, \code{"global"}, or \code{NULL} (all
levels, the default).}

\item{area}{Character vector of country or region codes
(e.g. \code{c("FRA", "DEU")}). Default \code{NULL} returns all areas.}

\item{year_from}{Numeric. Start year filter (inclusive).
Default \code{NULL}.}

\item{year_to}{Numeric. End year filter (inclusive).
Default \code{NULL}.}
}
\value{
A logical scalar:
\itemize{
\item \code{TRUE} if at least one observation exists for the filter.
\item \code{FALSE} if the server returns an empty result.
\item \code{NA} if the request fails (network failure, unreachable host,
or the indicator code does not exist and the server returns an
HTTP error). A warning is emitted in the failure case.
}
}
\description{
Sends a minimal request (\verb{$top=1&$select=Id}) to the WHO GHO OData
API to find out whether any observations exist for the given
indicator and filter combination, without downloading the full
result set. Useful as a quick precheck before \code{\link[=gho_data]{gho_data()}}.
}
\examples{
\donttest{
# Does WHO have life-expectancy data for France?
gho_has_data("WHOSIS_000001", area = "FRA")

# Quickly screen a list of indicators before downloading any data
inds <- c("WHOSIS_000001", "NCDMORT3070")
vapply(inds, gho_has_data, logical(1), area = "FRA")
}
}
\seealso{
\code{\link[=gho_data]{gho_data()}}, \code{\link[=gho_count]{gho_count()}}, \code{\link[=gho_coverage]{gho_coverage()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/iso3_to_region.R
\name{iso3_to_region}
\alias{iso3_to_region}
\title{Look Up the WHO Region for ISO3 Codes}
\usage{
iso3_to_region(iso3, long = FALSE)
}
\arguments{
\item{iso3}{Character vector of ISO3 codes. Case-sensitive
(uppercase, as in \code{\link{who_countries}}).}

\item{long}{Logical. If \code{TRUE}, return long-form region names
(e.g. \code{"Western Pacific"}). If \code{FALSE} (default), return the
short codes used elsewhere in DSIR: \code{"AFR"}, \code{"AMR"}, \code{"SEAR"},
\code{"EUR"}, \code{"EMR"}, \code{"WPR"}.}
}
\value{
A character vector the same length as \code{iso3}.
}
\description{
Maps ISO 3166-1 alpha-3 country codes to WHO region codes using
the \code{\link{who_countries}} dataset shipped with DSIR. Stays in sync with
WHO governance changes reflected in DSIR — for example, Indonesia's
reassignment from SEAR to WPR following EB156 (May 2025).
}
\details{
Codes that do not correspond to a WHO Member State return \code{NA}.
This includes Associate Members (Puerto Rico, Tokelau) and other
non-Member areas that some indicator data sets cover.
}
\examples{
iso3_to_region(c("PHL", "FRA", "USA", "COK"))
# "WPR" "EUR" "AMR" "WPR"

iso3_to_region(c("IDN", "JPN"), long = TRUE)
# "Western Pacific" "Western Pacific"  (Indonesia in WPR since May 2025)

# Non-Member areas return NA
iso3_to_region(c("PRI", "TKL", "PHL"))
# NA NA "WPR"
}
\seealso{
\code{\link{who_countries}}, \code{\link{wpro_cty}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aarr.R
\name{aarr}
\alias{aarr}
\title{Average Annual Rate of Reduction (AARR)}
\usage{
aarr(year, value, method = c("regression", "endpoint"), na.rm = TRUE)
}
\arguments{
\item{year}{Numeric vector of years.}

\item{value}{Numeric vector of indicator values, the same length as
\code{year}. Values must be positive (the computation is on the log
scale); any zero or negative value yields \code{NA_real_} with a
warning.}

\item{method}{Character. \code{"regression"} (default) or \code{"endpoint"}.
See Details.}

\item{na.rm}{Logical. Should pairs with a missing \code{year} or \code{value}
be removed before computation? Default \code{TRUE}. When \code{FALSE}, any
missing element makes the result \code{NA_real_}.}
}
\value{
A numeric scalar: the average annual rate of reduction as a
\emph{fraction} (\code{0.024} = 2.4\% per year). Multiply by 100 to compare
with published WHO / UNICEF tables, which print percentages.
Returns \code{NA_real_} (with a warning) when fewer than two distinct
years remain after \code{NA} handling, when any value is zero or
negative, or when \code{year} or \code{value} contains non-finite values.
}
\description{
Computes the average annual rate of reduction of an indicator over
time — the standard WHO / UNICEF metric for tracking progress in
declining indicators such as maternal, neonatal, and under-five
mortality, stunting prevalence, or premature NCD mortality.
}
\details{
Two estimation methods are offered:
\itemize{
\item \code{"regression"} (default; the UNICEF-recommended approach): an
ordinary least-squares line is fitted to \code{log(value)} against
\code{year}, and the AARR is \code{1 - exp(b)}, where \code{b} is the fitted
slope. All observations contribute, so the estimate is robust to
noise in individual years.
\item \code{"endpoint"}: only the earliest and latest years are used:
\code{1 - (v1 / v0) ^ (1 / (y1 - y0))}, where \code{v0} and \code{v1} are the
values at the earliest year \code{y0} and the latest year \code{y1}.
Intermediate observations are ignored. If several observations
share the earliest or latest year, their mean is used (but see
the note on duplicated years below).
}

\strong{Sign convention.} A \emph{positive} AARR means the indicator is
\emph{declining} (progress, for a mortality-type indicator): \code{0.024}
means an average decline of 2.4\% per year. A \emph{negative} AARR means
the indicator is increasing. Note this is the reverse of a growth
rate.

\strong{Duplicated years} usually mean the data still mix several strata
— for example both sexes plus male / female in \code{dim1}, or several
\code{series} codes in data cleaned by \code{\link[=gho_clean]{gho_clean()}} / \code{\link[=sdg_clean]{sdg_clean()}}.
\code{aarr()} warns and proceeds (the regression pools the strata), but
you almost always want to filter to a single stratum first.

\strong{Relation to published figures.} Published WHO / UNICEF tables
print the AARR as a percentage (\code{4.4} meaning 4.4\% per year);
multiply the value returned by this function by 100 to compare.
Note also that WHO's \emph{Trends in Maternal Mortality} reports print a
continuous-time rate, \code{-log(v1 / v0) / (y1 - y0)} (there called
ARR), which agrees closely but not exactly with the discrete AARR
computed here — small differences from those tables are expected.
}
\examples{
# A perfectly exponential 2.4\%/yr decline recovers exactly 0.024
years  <- 2000:2015
values <- 100 * (1 - 0.024) ^ (years - 2000)
aarr(years, values)                       # 0.024
100 * aarr(years, values)                 # 2.4 — as printed in reports

# Endpoint method uses only the earliest and latest years
aarr(years, values, method = "endpoint")  # also 0.024 here

# An increasing indicator gives a negative AARR
aarr(2010:2020, 50 * 1.01 ^ (0:10))       # about -0.01

# Back-of-envelope projection to 2030 at the observed AARR
r <- aarr(years, values)
values[length(values)] * (1 - r) ^ (2030 - 2015)
}
\seealso{
\code{\link[=geomean]{geomean()}} for ratio-based indicator aggregation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gho.R
\name{gho_count}
\alias{gho_count}
\title{Count Observations for a GHO Indicator Filter}
\usage{
gho_count(
  indicator,
  spatial_type = NULL,
  area = NULL,
  year_from = NULL,
  year_to = NULL
)
}
\arguments{
\item{indicator}{Character scalar. The indicator code
(e.g. \code{"NCDMORT3070"}). Use \code{\link[=gho_indicators]{gho_indicators()}} to find codes.}

\item{spatial_type}{Character. Spatial dimension to filter on:
one of \code{"country"}, \code{"region"}, \code{"global"}, or \code{NULL} (all
levels, the default).}

\item{area}{Character vector of country or region codes
(e.g. \code{c("FRA", "DEU")}). Default \code{NULL} returns all areas.}

\item{year_from}{Numeric. Start year filter (inclusive).
Default \code{NULL}.}

\item{year_to}{Numeric. End year filter (inclusive).
Default \code{NULL}.}
}
\value{
An integer scalar — the number of observations the
server would return for the same filter via \code{\link[=gho_data]{gho_data()}}.
Returns \code{NA_integer_} (with a warning) if the request fails.
}
\description{
Sends a \verb{$top=0&$count=true} request to the WHO GHO OData API,
which returns the matching row count without transferring any
observations. Useful for sizing a download before issuing it.
}
\examples{
\donttest{
# How many rows would gho_data() pull for France?
gho_count("WHOSIS_000001", area = "FRA")

# Compare coverage across regions
gho_count("NCDMORT3070", spatial_type = "country")
gho_count("NCDMORT3070", spatial_type = "region")
}
}
\seealso{
\code{\link[=gho_data]{gho_data()}}, \code{\link[=gho_has_data]{gho_has_data()}}, \code{\link[=gho_coverage]{gho_coverage()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gho.R
\name{gho_coverage}
\alias{gho_coverage}
\title{Summarise Per-Location Data Coverage of a GHO Indicator}
\usage{
gho_coverage(
  indicator,
  spatial_type = "country",
  area = NULL,
  year_from = NULL,
  year_to = NULL,
  dim1 = NULL,
  dim2 = NULL,
  dim3 = NULL
)
}
\arguments{
\item{indicator}{Character scalar. The indicator code
(e.g. \code{"WHOSIS_000001"}).}

\item{spatial_type}{Character. Spatial dimension to filter on:
one of \code{"country"}, \code{"region"}, \code{"global"}. Defaults to
\code{"country"} since per-country coverage is the typical use
case. Pass \code{NULL} for all spatial levels.}

\item{area}{Character vector of country or region codes
(e.g. \code{c("FRA", "DEU")}). Default \code{NULL} returns all areas
for the chosen \code{spatial_type}.}

\item{year_from}{Numeric. Start year filter (inclusive).
Default \code{NULL}.}

\item{year_to}{Numeric. End year filter (inclusive).
Default \code{NULL}.}

\item{dim1, dim2, dim3}{Character vector of values to keep for the
\code{Dim1} / \code{Dim2} / \code{Dim3} breakdown columns, filtered server-side
(e.g. \code{dim1 = "SEX_BTSX"} for both-sexes rows only, or
\code{dim1 = c("SEX_MLE", "SEX_FMLE")}). The meaning of each dimension
varies by indicator (\code{Dim1} is sex for one indicator, an age
group for another); use \code{\link[=gho_dimensions]{gho_dimensions()}} to discover the values
available for a given indicator. Rows where the dimension is
empty (\code{null}) are excluded by the filter. Default \code{NULL} (no
filtering).}
}
\value{
A \link[tibble:tibble]{tibble} with one row per location and
columns:
\itemize{
\item \code{location} (chr) — the \code{SpatialDim} value (typically ISO3).
\item \code{year_min} (int) — earliest year with data.
\item \code{year_max} (int) — latest year with data.
\item \code{n_obs} (int) — number of observations.

Sorted by \code{location}. Empty input or service failure returns
an empty tibble with the same four columns.
}
}
\description{
Fetches only the \code{SpatialDim} and \code{TimeDim} columns for a GHO
indicator (much lighter than \code{\link[=gho_data]{gho_data()}}) and summarises the
year range and observation count per location. Useful for
answering "which countries have data, and for what years?"
before committing to a full download.
}
\examples{
\donttest{
# Year coverage of life expectancy for three countries
gho_coverage("WHOSIS_000001", area = c("FRA", "DEU", "JPN"))

# All countries with any life-expectancy data, since 2010
gho_coverage("WHOSIS_000001", year_from = 2010)
}
}
\seealso{
\code{\link[=gho_data]{gho_data()}}, \code{\link[=gho_has_data]{gho_has_data()}}, \code{\link[=gho_count]{gho_count()}}.
}
